'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var tslib = require('tslib');
var intact = require('intact');
var react = require('react');
var reactDom = require('react-dom');

function isNullOrUndefined(o) {
    return o === null || o === undefined;
}
var toString = Object.prototype.toString;
var isArray = Array.isArray || function (arr) {
    return toString.call(arr) === '[object Array]';
};
function isInvalid(o) {
    return o === null || o === false || o === true || o === undefined;
}
function isStringOrNumber(o) {
    var type = typeof o;
    return type === 'string' || type === 'number';
}
function isFunction(o) {
    return typeof o === 'function';
}
function isNumber(o) {
    return typeof o === 'number';
}
function isString(o) {
    return typeof o === 'string';
}
function isObject(o) {
    return o !== null && typeof o === 'object';
}

var FakePromise = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function () {
    function FakePromise(executor) {
        this.resolved = false;
        this.callbacks = [];
        executor(this.resolve.bind(this));
    }
    FakePromise.all = function (promises) {
        if (promises.all)
            return promises.all;
        var callbacks = [];
        var resolved = false;
        var resolvedCount = 0;
        promises.then = isAllPromisesResolved;
        promises.value.forEach(function (promise) {
            promise.then(isAllPromisesResolved);
        });
        function callCallback() {
            var cb;
            while (cb = callbacks.shift()) {
                cb();
            }
            promises.done = true;
        }
        function isAllPromisesResolved() {
            resolvedCount++;
            if (promises.value.length === resolvedCount) {
                resolved = true;
                callCallback();
            }
        }
        return (promises.all = {
            then: function (cb) {
                callbacks.push(cb);
                if (!promises.value.length || resolved) {
                    callCallback();
                }
                return this;
            }
        });
    };
    FakePromise.prototype.then = function (cb) {
        this.callbacks.push(cb);
        if (this.resolved) {
            this.resolve();
        }
    };
    FakePromise.prototype.resolve = function () {
        this.resolved = true;
        var cb;
        while (cb = this.callbacks.shift()) {
            cb();
        }
    };
    return FakePromise;
}());
// let id = 0;
var FakePromises = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function () {
    function FakePromises() {
        this.value = [];
        this.then = null;
        this.done = false;
        this.all = null;
    }
    // public id = id++;
    FakePromises.prototype.add = function (promise) {
        this.value.push(promise);
        var then = this.then;
        if (then) {
            promise.then(then);
        }
    };
    FakePromises.prototype.reset = function () {
        this.value = [];
        this.then = null;
        this.done = false;
        this.all = null;
    };
    return FakePromises;
}());

var internalInstanceKey;
function precacheFiberNode(node, placeholder) {
    if (!internalInstanceKey) {
        var keys = Object.keys(placeholder);
        internalInstanceKey = keys[0];
    }
    var fiber = placeholder[internalInstanceKey];
    node[internalInstanceKey] = fiber;
    return fiber;
}
var listeningMarker;
var bind = Function.prototype.bind;
// excerpt from react definition
var IS_EVENT_HANDLE_NON_MANAGED_NODE = 1;
var IS_NON_DELEGATED = 2;
Function.prototype.bind = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var obj = args[0], domEventName = args[1], eventSystemFlags = args[2], targetContainer = args[3], nativeEvent = args[4];
    if (obj === null && isString(domEventName) && isNumber(eventSystemFlags) && targetContainer instanceof Element) {
        var isReactListening = false;
        if (!listeningMarker) {
            var keys = Object.keys(targetContainer);
            var key = keys.find(function (key) { return key.startsWith('_reactListening'); });
            if (key) {
                listeningMarker = key;
                isReactListening = true;
            }
            else {
                isReactListening = false;
            }
        }
        else {
            isReactListening = targetContainer[listeningMarker] && nativeEvent instanceof Event;
        }
        if (isReactListening && (eventSystemFlags & IS_EVENT_HANDLE_NON_MANAGED_NODE) === 0 && (eventSystemFlags & IS_NON_DELEGATED) === 0) {
            /**
             * Because we only add listeners to the react root container, the comment root container in Wrapper will ignore doing this.
             * But React will check the container (isMatchingRootContainer). it will always return false since the comment parentNode is not
             * the targetContainer. Therefore, we fake the parentNode by returning the targetContainer to skip the check in React.
             * Eventually we restore the value after calling the function.
             */
            var _fn_1 = this;
            var fn = function (name, eventSystemFlags, targetContainer, nativeEvent) {
                var targetInst = getClosestInstanceFromNode(nativeEvent.target);
                var commentRoot = getCommentRoot(targetInst);
                var containerInfo;
                if (commentRoot) {
                    containerInfo = commentRoot.stateNode.containerInfo;
                    commentRoot.stateNode.containerInfo = targetContainer;
                }
                var ret = _fn_1(name, eventSystemFlags, targetContainer, nativeEvent);
                if (commentRoot) {
                    commentRoot.stateNode.containerInfo = containerInfo;
                }
                return ret;
            };
            return bind.call(fn, null, domEventName, eventSystemFlags, targetContainer, nativeEvent);
        }
    }
    return bind.call.apply(bind, tslib.__spreadArray([this], args, false));
};
function getClosestInstanceFromNode(targetNode) {
    var targetInst = targetNode[internalInstanceKey];
    if (targetInst) {
        // Don't return HostRoot or SuspenseComponent here.
        return targetInst;
    } // If the direct event target isn't a React owned DOM node, we need to look
    // to see if one of its parents is a React owned DOM node.
    var parentNode = targetNode.parentNode;
    while (parentNode) {
        targetInst = parentNode[internalInstanceKey];
        if (targetInst) {
            return targetInst;
        }
        targetNode = parentNode;
        parentNode = targetNode.parentNode;
    }
    return null;
}
function getCommentRoot(targetInst) {
    var node = targetInst;
    while (true) {
        if (node === null)
            return;
        var nodeTag = node.tag;
        if (nodeTag === 3 /* HostRoot */) {
            return node;
        }
        node = node.return;
    }
}

var Context = react.createContext(null);
var containerComment = ' react-mount-point-unstable ';
var Wrapper = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function () {
    function Wrapper($props, $vNode, $SVG, $mountedQueue, $senior) {
        this.$props = $props;
        this.$vNode = $vNode;
        this.$SVG = $SVG;
        this.$mountedQueue = $mountedQueue;
        this.$senior = $senior;
        this.$inited = true;
        this.container = document.createComment(containerComment);
        this.isEmptyReactComponent = false;
        this.$lastInput = intact.createVNode('div');
        // fakeInput.transition = $vNode.transition;
        // fakeInput.transition = {
        // // only need leave hook to prevent Intact from removing the real dom,
        // // because it will be removed by React.
        // leave: noop
        // } as unknown as TransitionHooks;
    }
    Wrapper.prototype.$init = function (props) { };
    Wrapper.prototype.$render = function (lastVNode, vNode, parentDom, anchor) {
        if (lastVNode) {
            intact.removeVNodeDom(lastVNode, parentDom);
        }
        var container = this.container;
        /**
         * React will insert child before container, if container is a comment node.
         * So we should insert the container into the corresponding position
         */
        if (anchor) {
            parentDom.insertBefore(container, anchor);
        }
        else {
            parentDom.appendChild(container);
        }
        rewriteParentElementApi(parentDom, !existPortal(this));
        this.render(vNode, parentDom);
    };
    Wrapper.prototype.$update = function (lastVNode, vNode, parentDom, anchor) {
        this.render(vNode, parentDom);
    };
    Wrapper.prototype.$unmount = function (vNode, nextVNode) {
        var _this = this;
        var container = this.container;
        /**
         * If we update intact component lead to this unmountion,
         * the dom will be removed by react immediately,
         * so we set a flag `hasRemoved` to indicate this case.
         * But maybe the wrapped element returns null, and react
         * will not remove the void text node that we created in render method.
         * So if it is in this case, we should remove the void text node manually.
         */
        var dom = this.$lastInput.dom;
        var hasRemoved = false;
        reactDom.render(null, container, function () {
            hasRemoved = true;
            if (_this.isEmptyReactComponent) {
                container.parentNode.removeChild(dom);
            }
        });
        if (hasRemoved) {
            this.$lastInput.dom = container;
        }
    };
    Wrapper.prototype.render = function (vNode, parentDom) {
        var parent = this.$senior;
        var parentComponent = getParent(this);
        var instance = this;
        var container = this.container;
        var vnode = getReactElement(vNode);
        /**
         * pass the $senior as value instead of parentComponent
         * because parentComponent is the the toplevel component and
         * not always equal to the $senior
         * e.g. <A><B><div><C /></div></B></A>
         */
        vnode = react.createElement(Context.Provider, { value: parent }, vnode);
        // if the parent component has providers, pass them to subtree
        var providers = parentComponent.$reactProviders;
        providers.forEach(function (value, provider) {
            vnode = react.createElement(provider, { value: value }, vnode);
        });
        this.isEmptyReactComponent = false;
        var promise = new FakePromise(function (resolve) {
            reactDom.unstable_renderSubtreeIntoContainer(parentComponent, vnode, container, function () {
                /**
                 * add dom to the $lastInput for findDomFromVNode
                 * the real dom will be inserted before the container
                 * but the react component may return null,
                 * so we can't get the previousSibling
                 *
                 * We will add the real element to the container
                 * by rewriting the parentNode.insertBefore api
                 *
                 * If we wrap an React component that returns Intact component directly
                 * e.g. <A><Context.Provider><B /><Context.Provider></A>
                 * the realElement will be the <template>
                 */
                var dom = container._realElement;
                if (!dom ||
                    dom.tagName === 'TEMPLATE' &&
                        dom.getAttribute('data-intact-react') !== null) {
                    // maybe this react component return null,
                    // we create a text as the dom
                    dom = document.createTextNode('');
                    parentDom.insertBefore(dom, container);
                    dom._isEmpty = true;
                }
                if (dom._isEmpty) {
                    instance.isEmptyReactComponent = true;
                }
                dom._mountPoint = container;
                instance.$lastInput.dom = dom;
                // console.log(rootContainer);
                // console.dir(container);
                // debugger;
                // getRootContainer(parentComponent);
                // Object.defineProperty((container as any)._reactRootContainer._internalRoot, 'containerInfo', {
                // get() {
                // return getRootContainer(parentComponent); 
                // }
                // });
                resolve();
            });
        });
        // if the promised has be resolved, it indicate that this render is sync.
        // Maybe we call update in intact directly, and it is not in React's render context
        if (!promise.resolved) {
            parentComponent.$promises.add(promise);
        }
    };
    return Wrapper;
}());
function getParent(instance) {
    var $senior = instance.$senior;
    do {
        if ($senior._reactInternals) {
            return $senior;
        }
    } while ($senior = $senior.$senior);
    // should not hit this
    /* istanbul ignore next */
    return null;
}
function existPortal(instance) {
    var $senior = instance.$senior;
    do {
        if ($senior.$isPortal) {
            return true;
        }
        if ($senior.constructor === Wrapper) {
            return false;
        }
    } while ($senior = $senior.$senior);
    return false;
}
function rewriteParentElementApi(parentElement, preventListener) {
    if (!parentElement._hasRewrite) {
        var removeChild_1 = parentElement.removeChild;
        parentElement.removeChild = function (child) {
            if (child.parentNode) {
                removeChild_1.call(parentElement, child);
                var realElement = child._realElement;
                if (realElement) {
                    child._realElement = null;
                    realElement._mountPoint = null;
                }
            }
            else {
                // if the node has been removed, then remove the mount point
                var container = child._mountPoint;
                removeChild_1.call(parentElement, container);
                child._mountPoint = null;
                container._realElement = null;
            }
        };
        var insertBefore_1 = parentElement.insertBefore;
        parentElement.insertBefore = function (child, beforeChild) {
            if (beforeChild.nodeType === 8 && beforeChild.nodeValue === containerComment) {
                // it's the wrapper container, we add the real element to it
                beforeChild._realElement = child;
            }
            insertBefore_1.call(parentElement, child, beforeChild);
        };
        // let react don't add listeners to the root container
        if (preventListener) {
            parentElement[listeningMarker] = true;
        }
        parentElement._hasRewrite = true;
    }
}
function getReactElement(vNode) {
    var props = vNode.props;
    var vnode = props.vnode;
    // react vNode has been frozen, so we must clone it to change
    var _props = null;
    for (var key in props) {
        if (key === 'vnode')
            continue;
        if (!_props)
            _props = {};
        var value = props[key];
        if (key.substr(0, 3) === 'ev-') {
            _props[eventsMap[key]] = value;
        }
        else {
            _props[key] = value;
        }
    }
    return _props ? react.cloneElement(vnode, _props) : vnode;
}
var eventsMap = {
    'ev-click': 'onClick',
    'ev-contextmenu': 'onContextMenu',
    'ev-mouseenter': 'onMouseEnter',
    'ev-mouseleave': 'onMouseLeave',
};

var cid = 'IntactReactFunctional';
function functionalWrapper(Component) {
    function Ctor(props, context) {
        if (context) {
            // invoked by React
            var vNodes = Component(normalizeProps(props, Component.events), true);
            if (isArray(vNodes)) {
                return react.createElement(react.Fragment, null, vNodes.map(function (vNode, index) {
                    return normalizeIntactVNodeToReactVNode(vNode, index);
                }));
            }
            return normalizeIntactVNodeToReactVNode(vNodes, 0);
        }
        else {
            // invoked by Intact
            return Component(props);
        }
    }
    var element = react.forwardRef(function (props, ref) {
        if (ref)
            props = tslib.__assign(tslib.__assign({}, props), { forwardRef: ref });
        return react.createElement(Ctor, props);
    });
    element._$type = Ctor;
    element.$cid = cid;
    return element;
}
function normalizeIntactVNodeToReactVNode(vNode, key) {
    var props = vNode.props;
    return react.createElement(vNode.tag, props ? tslib.__assign({ key: key }, props) : { key: key });
}

function normalize(vNode) {
    if (isInvalid(vNode))
        return null;
    // if a element has one child which is string or number
    // intact will set text content directly to update its children
    // this will lead to that the container which return
    // by Wrapper has been removed.
    // so we should convert string or number child
    // to VNode to let intact update it one by one
    if (isStringOrNumber(vNode)) {
        return intact.createTextVNode(vNode);
    }
    // maybe return by functional component, see unit test: `render intact functional component`
    if (isNumber(vNode.type)) {
        return vNode;
    }
    if (isIntactComponent(vNode)) {
        var key = vNode.key;
        var type = vNode.type;
        var tag = type.$cid === cid ? type._$type : type;
        var props = normalizeProps(vNode.props, tag.events);
        if (!isNullOrUndefined(key)) {
            props.key = key;
        }
        return intact.createComponentVNode(4, tag, props, key, normalizeRef(vNode.ref));
    }
    if (vNode.type === react.Fragment) {
        return normalizeChildren(vNode.props.children);
    }
    var ret = intact.createComponentVNode(4, Wrapper, { vnode: vNode }, vNode.key);
    // transition has two functions
    // 1. prevent Intact from removing the real dom,
    //    because it will be removed by React.
    // TODO: 2. let intact never call clearDom method to remove all children
    //    because it will cause `react-mount-point-unstable` to be missing
    // ret.transition = {
    // leave: noop,
    // } as unknown as TransitionHooks;
    return ret;
}
function normalizeChildren(vNodes) {
    if (isArray(vNodes)) {
        var ret_1 = [];
        vNodes.forEach(function (vNode) {
            if (isArray(vNode)) {
                ret_1.push.apply(ret_1, normalizeChildren(vNode));
            }
            else {
                ret_1.push(normalize(vNode));
            }
        });
        return ret_1;
    }
    return normalize(vNodes);
}
function normalizeProps(props, events) {
    // if (!props) return null;
    var blocks = null;
    var normalizedProps = {};
    var tmp;
    for (var key in props) {
        var value = props[key];
        if (key === 'children') {
            normalizedProps[key] = normalizeChildren(value);
        }
        else if (tmp = getEventName(key, events)) {
            normalizedProps[tmp] = value;
        }
        else if (key.startsWith('slot')) {
            if (!blocks)
                blocks = normalizedProps.$blocks = {};
            blocks[hyphenate(key.substring(4))] = normalizeBlock(value);
        }
        else if (key === 'forwardRef') {
            normalizedProps.ref = value;
        }
        else {
            normalizedProps[key] = value;
        }
    }
    return normalizedProps;
}
function normalizeRef(ref) {
    return isObject(ref) ? function (i) { ref.current = i; } : ref;
}
function isIntactComponent(vNode) {
    var type = vNode.type;
    return !!type.$cid;
}
function getEventName(propName, events) {
    var third = propName[2];
    if (!third)
        return;
    var first = propName[0];
    var second = propName[1];
    var tmp;
    if (first === 'o' && second === 'n') {
        if (third === '$') {
            // e.g. on$change-value
            return "ev-$".concat(propName.substring(3).replace(/\-/g, ':'));
        }
        else if ((tmp = third.charCodeAt(0)) && tmp >= 65 && tmp <= 90) {
            var eventName = propName.substring(2);
            if (eventName === 'Change') {
                // i.e onChange -> ev-change
                return "ev-change";
            }
            else if (eventName.startsWith('Change')) {
                // e.g. onChangeValue -> ev-$change:value
                return "ev-$change:".concat(lowerFirst(eventName.substring(6)));
            }
            else {
                // e.g. onClick -> ev-click; onKeyPress -> ev-keypress
                var loweredFirst = lowerFirst(eventName.replace(/-/g, ':'));
                if (!events || !events[loweredFirst]) {
                    // is native event
                    loweredFirst = loweredFirst.toLowerCase();
                }
                return "ev-".concat(loweredFirst);
            }
        }
    }
}
function cache(fn) {
    var cache = Object.create(null);
    return function (str) {
        return cache[str] || (cache[str] = fn(str));
    };
}
var lowerFirst = cache(function (str) {
    return str.charAt(0).toLowerCase() + str.slice(1);
});
var hyphenateRE = /\B([A-Z])/g;
var hyphenate = cache(function (str) {
    return (str.charAt(0) + str.slice(1).replace(hyphenateRE, '-$1')).toLowerCase();
});
function normalizeBlock(block) {
    if (isFunction(block)) {
        return function (parent) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            return normalizeChildren(block.apply(void 0, args));
        };
    }
    return function () {
        return normalizeChildren(block);
    };
}

var PROMISES = '_$IntactReactPromises';
var Component = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function (_super) {
    tslib.__extends(Component, _super);
    function Component(props, $vNodeOrContext, $SVG, $mountedQueue, $senior) {
        var _this = this;
        if ($vNodeOrContext && intact.isComponentClass($vNodeOrContext)) {
            // Intact component in intact
            _this = _super.call(this, props, $vNodeOrContext, $SVG, $mountedQueue, $senior) || this;
        }
        else {
            // super(null, null as any, false, [], null);
            // Intact component in React
            // @ts-ignore
            var normalizedProps = normalizeProps(props, _this.constructor.events);
            var parent_1 = $vNodeOrContext;
            var mountedQueue = getMountedQueue(parent_1);
            _this = _super.call(this, normalizedProps, null, false, mountedQueue, parent_1) || this;
            _this.$inited = true;
            // create $vNode
            _this.$vNode = intact.createComponentVNode(4, _this.constructor, normalizedProps);
            _this.$elementRef = react.createRef();
            if (_this.constructor.$doubleVNodes) {
                _this.$elementAlternateRef = react.createRef();
            }
            _this.$isReact = true;
            var promises = _this.$promises = new FakePromises();
            _this.$parentPromises = intact.inject(PROMISES, null);
            intact.provide(PROMISES, promises);
        }
        return _this;
    }
    Component.prototype.render = function () {
        // save all Context.Proiver, because React will clear the value
        // on commit phase, and we cannot get it when we render react element
        // in Wrapper
        var providers = this.$reactProviders = new Map();
        var returnFiber = this._reactInternals;
        var _loop_1 = function () {
            var tag = returnFiber.tag;
            if (tag === 10) { // is ContextProiver
                var type = returnFiber.type;
                providers.set(type, type._context._currentValue);
            }
            else if (tag === 3) { // HostRoot
                /**
                 * React will update from root, if root has pendingContext, it will compare
                 * the last value and the current value to change `didPerformWorkStatckCursor`,
                 * if the cursor is true, all children will be updated
                 * @FIXME: Maybe we can use all Consumer to wrap the placeholder element
                 *
                 * always let hasContextChanged return true to make React update the component,
                 * even if it props has not changed
                 * see unit test: `shuold update children when provider's children...`
                 * issue: https://github.com/ksc-fe/kpc/issues/533
                 **/
                var stateNode = returnFiber.stateNode;
                if (!stateNode.__intactReactDefinedProperty) {
                    var context_1;
                    Object.defineProperty(stateNode, 'pendingContext', {
                        get: function () {
                            return context_1 || (returnFiber.context ? tslib.__assign({}, returnFiber.context) : Object.create(null));
                        },
                        set: function (v) {
                            context_1 = v;
                        }
                    });
                    stateNode.__intactReactDefinedProperty = true;
                }
                return "break";
            }
        };
        while (returnFiber = returnFiber.return) {
            var state_1 = _loop_1();
            if (state_1 === "break")
                break;
        }
        var vNode = createTemplateVNode(this.$elementRef, '1');
        return this.constructor.$doubleVNodes ?
            [vNode, createTemplateVNode(this.$elementAlternateRef, '2')] :
            vNode;
    };
    Component.prototype.componentDidMount = function () {
        var _this = this;
        var vNode = this.$vNode;
        var placeholder = this.$elementRef.current;
        var parentElement = this.$parentElement = placeholder.parentElement;
        // React will assign a emptyObject which is frozen to refs on mountClassInstance
        // we have to re-assign it
        this.refs = {};
        // we may call hooks in init lifecycle, so set instance here,
        // $init method will set it to null
        intact.setInstance(this);
        this.$init(vNode.props);
        vNode.children = this;
        var sibling = placeholder;
        while (sibling = sibling.nextSibling) {
            if (sibling.nodeType !== 8 || sibling.nodeValue !== containerComment) {
                break;
            }
        }
        this.$render(null, vNode, parentElement, sibling, this.$mountedQueue);
        this.$done(function () {
            handleElementOnMounted(parentElement, placeholder, intact.findDomFromVNode(vNode, true));
            if (_this.constructor.$doubleVNodes) {
                handleElementOnMounted(parentElement, _this.$elementAlternateRef.current, intact.findDomFromVNode(vNode, false));
            }
        });
    };
    Component.prototype.componentDidUpdate = function () {
        var normalizedProps = normalizeProps(this.props, this.constructor.events);
        var vNode = intact.createComponentVNode(4, this.constructor, normalizedProps);
        vNode.children = this;
        var lastVNode = this.$vNode;
        this.$vNode = vNode;
        var promises = this.$promises;
        if (promises.done) {
            promises.reset();
            var mountedQueue = this.$mountedQueue = getMountedQueue(this.$senior);
            this.$update(lastVNode, vNode, this.$parentElement, null, mountedQueue, false);
            this.$done(null);
        }
        else {
            this.$update(lastVNode, vNode, this.$parentElement, null, this.$mountedQueue, false);
        }
    };
    Component.prototype.componentWillUnmount = function () {
        this.$unmount(this.$vNode, null);
    };
    Component.prototype.$done = function (callback) {
        var _this = this;
        var done = function () {
            var p = FakePromise.all(_this.$promises);
            if (callback) {
                return p.then(callback);
            }
            return p;
        };
        var $parentPromises = this.$parentPromises;
        if ($parentPromises && !$parentPromises.done) {
            $parentPromises.add(new FakePromise(function (resolve) {
                done().then(resolve);
            }));
        }
        else {
            done().then(function () {
                intact.callAll(_this.$mountedQueue);
            });
        }
    };
    Component.prototype.setState = function () { };
    Component.$cid = 'IntactReact';
    Component.normalize = normalizeChildren;
    Component.functionalWrapper = functionalWrapper;
    Component.contextType = Context;
    Component.FakePromise = FakePromise;
    // if the flag is true, it indicate the component returns 2 vNodes,
    // only support two, becasue kpc components don't return vNodes exceed 2.
    Component.$doubleVNodes = false;
    return Component;
}(intact.Component));
Component.prototype.isReactComponent = true;
function getMountedQueue(parent) {
    if (parent) {
        parent.$mountedQueue;
        var parentPromises = parent.$provides[PROMISES];
        if (!parentPromises.done) {
            return parent.$mountedQueue;
        }
    }
    return [];
}
function createTemplateVNode(ref, key) {
    return react.createElement('template', {
        ref: ref,
        key: key,
        'data-intact-react': '',
    });
}
function handleElementOnMounted(parentElement, placeholder, element) {
    parentElement.removeChild(placeholder);
    // replace some properties like React do
    var fiber = precacheFiberNode(element, placeholder);
    // if we return a React element directly, we can not replace its props by placeholder's props,
    // otherwise its event props will be removed
    // updateFiberProps(element, placeholder);
    fiber.stateNode = element;
    var alternate = fiber.alternate;
    if (alternate) {
        alternate.stateNode = element;
    }
}

exports.Component = Component;
exports.normalize = normalizeChildren;
Object.keys(intact).forEach(function (k) {
    if (k !== 'default' && !exports.hasOwnProperty(k)) Object.defineProperty(exports, k, {
        enumerable: true,
        get: function () { return intact[k]; }
    });
});
