import { createContext, cloneElement, createElement, forwardRef, Fragment as Fragment$1, createRef as createRef$1 } from 'react';
import { render as render$1, unstable_renderSubtreeIntoContainer } from 'react-dom';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    if (typeof b !== "function" && b !== null)
        throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __spreadArray(to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
}

function isEventProp(propName) {
    return propName.substr(0, 3) === 'ev-';
}
function isNullOrUndefined(o) {
    return o === null || o === undefined;
}
var toString$1 = Object.prototype.toString;
var isArray = Array.isArray || function (arr) {
    return toString$1.call(arr) === '[object Array]';
};
function isInvalid(o) {
    return o === null || o === false || o === true || o === undefined;
}
function isStringOrNumber(o) {
    var type = typeof o;
    return type === 'string' || type === 'number';
}
function isNull(o) {
    return o === null;
}
function isUndefined(o) {
    return o === void 0;
}
function isFunction(o) {
    return typeof o === 'function';
}
function isNumber(o) {
    return typeof o === 'number';
}
function isString(o) {
    return typeof o === 'string';
}
function isObject(o) {
    return o !== null && typeof o === 'object';
}
var ERROR_MSG = 'a runtime error occured! Use Intact in development environment to find the error.';
function throwError$1(message) {
    if (!message) {
        message = ERROR_MSG;
    }
    throw new Error("Intact Error: ".concat(message));
}
function error(message) {
    console.error(message);
}
var hasOwn = Object.prototype.hasOwnProperty;
var EMPTY_OBJ = {};
var noop = function () { };
var hasDocumentAvailable = typeof document !== 'undefined';

function get(object, path) {
    if (hasOwn.call(object, path) || !isString(path))
        return object[path];
    var pathArray = castPath(path);
    var length = pathArray.length;
    var index = 0;
    while (!isNullOrUndefined(object) && index < length) {
        object = object[pathArray[index++]];
    }
    return (index && index === length) ? object : undefined;
}
function set(object, path, value) {
    var changeTraces = [];
    if (hasOwn.call(object, path)) {
        changeTraces.push({ path: path, newValue: value, oldValue: object[path] });
        object[path] = value;
        return changeTraces;
    }
    var pathArray = castPath(path);
    var length = pathArray.length;
    var lastIndex = length - 1;
    var index = -1;
    var nested = object;
    var prefix = '';
    while (!isNullOrUndefined(nested) && ++index < length) {
        var key = pathArray[index];
        var oldValue = nested[key];
        var newValue = value;
        if (index !== lastIndex) {
            // data is immutable, clone it before modifing it
            newValue = isObject(oldValue) ? clone(oldValue) : reIsIndex.test(pathArray[index + 1]) ? [] : {};
        }
        nested[key] = newValue;
        nested = nested[key];
        prefix += key;
        changeTraces.push({ path: prefix, newValue: newValue, oldValue: oldValue });
        prefix += '.';
    }
    return changeTraces;
}
var castPathCache = {};
var rePropName = /[^\.\[\]]+/g;
var reIsIndex = /^(?:0|[1-9]\d*)$/;
function castPath(path) {
    if (castPathCache[path])
        return castPathCache[path];
    return (castPathCache[path] = path.match(rePropName));
}
function clone(object) {
    if (isArray(object)) {
        return object.slice();
    }
    return __assign({}, object);
}

function normalizeEventName(name) {
    return name.substr(3);
}
function replaceChild(parentDom, newDom, lastDom) {
    parentDom.replaceChild(newDom, lastDom);
}
function setTextContent(dom, children) {
    dom.textContent = children;
}
function removeChild(parentDom, dom) {
    parentDom.removeChild(dom);
}
function insertOrAppend(parentDom, newNode, anchor) {
    if (isNull(anchor)) {
        parentDom.appendChild(newNode);
    }
    else {
        parentDom.insertBefore(newNode, anchor);
    }
}
var REFERENCE = { value: false };
var Fragment = '$F';
function removeVNodeDom(vNode, parentDom) {
    var type;
    var dom;
    do {
        type = vNode.type;
        if (type & 3043 /* HtmlElement */) {
            var transition = vNode.transition;
            if (isNullOrUndefined(transition)) {
                removeChild(parentDom, vNode.dom);
            }
            else {
                dom = vNode.dom;
                transition.leave(dom, function () { return removeChild(parentDom, dom); });
            }
            return;
        }
        var children = vNode.children;
        if (type & 8 /* ComponentClass */) {
            vNode = children.$lastInput;
        }
        else if (type & 16 /* ComponentFunction */) {
            vNode = children;
        }
        else if (type & 4096 /* Fragment */) {
            if (vNode.childrenType === 2 /* HasVNodeChildren */) {
                vNode = children;
            }
            else {
                for (var i = 0; i < children.length; i++) {
                    removeVNodeDom(children[i], parentDom);
                }
                return;
            }
        }
    } while (vNode);
}
function findDomFromVNode(vNode, startEdge) {
    var type;
    while (vNode) {
        type = vNode.type;
        if (type & 3043 /* HtmlElement */) {
            return vNode.dom;
        }
        vNode = findChildVNode(vNode, startEdge, type);
    }
    return null;
}
function findChildVNode(vNode, startEdge, type) {
    var children = vNode.children;
    if (type & 8 /* ComponentClass */) {
        return children.$lastInput;
    }
    if (type & 4096 /* Fragment */) {
        return vNode.childrenType === 2 /* HasVNodeChildren */ ?
            children :
            children[startEdge ? 0 : children.length - 1];
    }
    return children;
}
function moveVNodeDom(vNode, parentDom, anchor) {
    do {
        var type = vNode.type;
        if (type & 3043 /* HtmlElement */) {
            // TODO: move with transtion
            insertOrAppend(parentDom, vNode.dom, anchor);
            return;
        }
        var children = vNode.children;
        if (type & 8 /* ComponentClass */) {
            vNode = children.$lastInput;
        }
        else if (type & 16 /* ComponentFunction */) {
            vNode = children;
        }
        else if (type & 4096 /* Fragment */) {
            if (vNode.childrenType === 2 /* HasVNodeChildren */) {
                vNode = children;
            }
            else {
                for (var i = 0; i < children.length; i++) {
                    moveVNodeDom(children[i], parentDom, anchor);
                }
                return;
            }
        }
    } while (vNode);
}
function getComponentName(tag) {
    return tag.name || tag.displayName; // || tag.constructor.name || (tag.toString().match(/^function\s*([^\s(]+)/) || [])[1];
}
function callAll(mountedQueue) {
    for (var i = 0; i < mountedQueue.length; i++) {
        mountedQueue[i]();
    }
}
var xlinkNS = "http://www.w3.org/1999/xlink";
var xmlNS = "http://www.w3.org/XML/1998/namespace";
var namespaces = {
    'xlink:href': xlinkNS,
    'xlink:arcrole': xlinkNS,
    'xlink:actuate': xlinkNS,
    'xlink:show': xlinkNS,
    'xlink:role': xlinkNS,
    'xlink:title': xlinkNS,
    'xlink:type': xlinkNS,
    'xml:base': xmlNS,
    'xml:lang': xmlNS,
    'xml:space': xmlNS,
};
var hooks = {
    beforeInsert: null,
};

// @reference Vue
function validateProps(vNode) {
    var tag = vNode.tag;
    var typeDefs = tag.typeDefs;
    var componentName = getComponentName(tag);
    var props = vNode.props;
    if (isNullOrUndefined(props) || isNullOrUndefined(typeDefs))
        return;
    for (var prop in __assign(__assign({}, props), typeDefs)) {
        var value = props[prop];
        var expectedType = typeDefs[prop];
        if (!isPlainObject(expectedType)) {
            expectedType = { type: expectedType };
        }
        if (isNullOrUndefined(value)) {
            var required = expectedType.required;
            if (isFunction(required)) {
                required = required(props);
            }
            if (required) {
                error("Missing required prop on component \"".concat(componentName, "\": \"").concat(prop, "\"."));
                return;
            }
            continue;
        }
        var type = expectedType.type;
        if (isNullOrUndefined(type) && isEventProp(prop)) {
            type = Function;
        }
        if (!isNullOrUndefined(type)) {
            if (!isArray(type)) {
                type = [type];
            }
            var _valid = false;
            var _isStringOrNumber = false;
            var expectedTypes = [];
            for (var i = 0; i < type.length; i++) {
                var t = type[i];
                if (isNullOrUndefined(t))
                    continue;
                var _a = assertType(value, t), expectedType_1 = _a.expectedType, valid = _a.valid, isStringOrNumber_1 = _a.isStringOrNumber;
                expectedTypes.push(expectedType_1 || '');
                _isStringOrNumber = isStringOrNumber_1;
                if (valid) {
                    _valid = valid;
                    break;
                }
            }
            if (!_valid) {
                error("Invalid type of prop \"".concat(prop, "\" on component \"").concat(componentName, "\". ") +
                    "Expected ".concat(expectedTypes.join(', '), ", but got ") +
                    "".concat(toRawType(value, _isStringOrNumber), "."));
                return;
            }
        }
        var validator = expectedType.validator;
        if (validator) {
            var result = validator(value);
            if (result === false) {
                error("Invalid prop \"".concat(prop, "\" on component \"").concat(componentName, "\": custom validator check failed."));
            }
            else if (result !== true) {
                error("Invalid prop \"".concat(prop, "\" on component \"").concat(componentName, "\": ").concat(result));
            }
        }
    }
}
var simpleCheckRE = /^(String|Number|Boolean|Function|Symbol)$/;
function assertType(value, type) {
    if (isNumber(type)) {
        return { valid: type === value, expectedType: type, isStringOrNumber: true };
    }
    else if (isString(type)) {
        return { valid: type === value, expectedType: "\"".concat(type, "\""), isStringOrNumber: true };
    }
    var valid;
    var expectedType = getType(type);
    if (simpleCheckRE.test(expectedType)) {
        var t = typeof value;
        valid = t === expectedType.toLowerCase();
        if (valid && t === 'number' && value !== value) {
            // for NaN
            valid = false;
        }
        if (!valid && t === 'object') {
            valid = value instanceof type;
        }
    }
    else if (expectedType === 'Object') {
        valid = isPlainObject(value);
    }
    else if (expectedType === 'Array') {
        valid = isArray(value);
    }
    else {
        valid = value instanceof type;
    }
    return { valid: valid, expectedType: expectedType, isStringOrNumber: false };
}
function getType(type) {
    var match = type && type.toString().match(/^\s*(?:function|class) (\w+)/);
    return match ? match[1] : '';
}
var toString = Object.prototype.toString;
function toRawType(value, isStringOrNumber) {
    if (isStringOrNumber) {
        if (isString(value)) {
            return "\"".concat(value, "\"");
        }
        return value;
    }
    if (value !== value)
        return 'NaN';
    return toString.call(value).slice(8, -1);
}
function isPlainObject(value) {
    return toString.call(value) === '[object Object]';
}

var VNode = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function () {
    function VNode(type, tag, childrenType, children, className, props, key, ref) {
        this.dom = null;
        this.type = type;
        this.tag = tag;
        this.childrenType = childrenType;
        this.children = children;
        this.className = className;
        this.props = props;
        this.key = key === undefined ? null : key;
        this.ref = ref === undefined ? null : ref;
        // for Transition
        this.transition = null;
        // FIXME: Is it necessary to initialize these properties to prevent V8 from de-optimization
        this.position = null;
        this.newPosition = null;
    }
    return VNode;
}());
// export function createVNode<
// T extends ComponentConstructor<U>,
// U extends ComponentClass<P>,
// P extends {}
// >(
// tag: T,
// props?: Props<P, U> | null,
// children?: Children | null
// ): VNodeComponentClass<U>;
// export function createVNode<
// T extends ComponentFunction<P, R>,
// P extends {},
// R extends Element | ComponentClass
// >(
// tag: T,
// props: Props<P, R> | null,
// children?: Children | null
// ): VNodeComponentFunction<T>;
// export function createVNode<T extends string>(
// tag: T,
// props?: Props<any, Element> | null,
// children?: Children | null
// ): VNode<T>;
function createVNode(tag, props, children) {
    var type;
    var isElement = false;
    switch (typeof tag) {
        case 'string':
            isElement = true;
            type = getTypeForVNodeElement(tag);
            break;
        case 'function':
            type = 4 /* ComponentUnknown */;
            break;
        default:
            throwError$1("createVNode expects to get a string or function, but get a type: \"".concat(JSON.stringify(tag), "\""));
    }
    var key = null;
    var ref = null;
    var newProps = null;
    var className = null;
    if (!isNullOrUndefined(props)) {
        for (var prop in props) {
            if (isElement && prop === 'className') {
                className = props.className;
            }
            else if (prop === 'key') {
                key = props.key;
            }
            else if (prop === 'ref') {
                ref = props.ref;
            }
            else if (prop === 'children') {
                children = props.children;
            }
            else {
                if (isNullOrUndefined(newProps))
                    newProps = {};
                newProps[prop] = props[prop];
            }
        }
    }
    if (isElement) {
        if (type & 4096 /* Fragment */) {
            return createFragment(children, 0 /* UnknownChildren */, key);
        }
        return createElementVNode(type, tag, children, 0 /* UnknownChildren */, className, newProps, key, ref);
    }
    var hasChildren = !isNullOrUndefined(children);
    var hasKey = !isNullOrUndefined(key);
    if (hasChildren || hasKey || ref) {
        if (isNullOrUndefined(newProps))
            newProps = {};
        if (hasChildren)
            newProps.children = children;
        if (hasKey)
            newProps.key = key;
        if (ref)
            newProps.ref = ref;
    }
    return createComponentVNode(type, tag, newProps, key, ref);
}
function getTypeForVNodeElement(tag) {
    switch (tag) {
        case 'input':
            return 64 /* InputElement */;
        case 'select':
            return 128 /* SelectElement */;
        case 'textarea':
            return 256 /* TextareaElement */;
        case 'svg':
            return 512 /* SvgElement */;
        case Fragment:
            return 4096 /* Fragment */;
        default:
            return 2 /* CommonElement */;
    }
}
function createElementVNode(type, tag, children, childrenType, className, props, key, ref) {
    if (isNullOrUndefined(childrenType)) {
        childrenType = 1 /* HasInvalidChildren */;
    }
    var vNode = new VNode(type, tag, childrenType, children, className, props, key, ref);
    if (childrenType === 0 /* UnknownChildren */) {
        normalizeChildren$1(vNode, children);
    }
    return vNode;
}
function createComponentVNode(type, tag, props, key, ref) {
    if ((type & 24 /* ComponentKnown */) === 0) {
        if (tag.prototype && tag.prototype.$render) {
            type = 8 /* ComponentClass */;
        }
        else {
            type = 16 /* ComponentFunction */;
        }
    }
    return new VNode(type, tag, 1 /* HasInvalidChildren */, null, null, props, key, ref);
}
function createUnknownComponentVNode(tag, props, key, ref) {
    // for react forwardRef element, see intact-react 
    var type = tag._$type;
    if (type) {
        tag = type;
    }
    return createComponentVNode(4 /* ComponentUnknown */, tag, props, key, ref);
}
function createTextVNode(text, key) {
    return new VNode(1 /* Text */, null, 1 /* HasInvalidChildren */, text, null, null, key, null);
}
function createVoidVNode() {
    return createTextVNode('', null);
}
function createUnescapeTextVNode(text) {
    return new VNode(1024 /* UnescapeText */, null, 1 /* HasInvalidChildren */, text, null, null, null, null);
}
function createCommentVNode(comment, key) {
    return new VNode(32 /* HtmlComment */, null, 1 /* HasInvalidChildren */, comment, null, null, key, null);
}
function createFragment(children, childrenType, key) {
    var fragment = createElementVNode(4096 /* Fragment */, 4096 /* Fragment */, children, childrenType, null, null, key, null);
    switch (fragment.childrenType) {
        case 1 /* HasInvalidChildren */:
            fragment.children = createVoidVNode();
            fragment.childrenType = 2 /* HasVNodeChildren */;
            break;
        case 16 /* HasTextChildren */:
            fragment.children = [createTextVNode(children)];
            fragment.childrenType = 4 /* HasNonKeyedChildren */;
            break;
    }
    return fragment;
}
function directClone(vNode) {
    var type = vNode.type & -8193 /* ClearInUse */;
    var props = vNode.props;
    // FIXME: need clone props?
    // if (type & Types.Component) {
    // if (!isNullOrUndefined(props)) {
    // const propsToClone = props;
    // props = {};
    // for (const key in propsToClone) {
    // props[key] = propsToClone[key];
    // }
    // }
    // }
    var newVNode;
    if (type & 4096 /* Fragment */) {
        var childrenType = vNode.childrenType;
        newVNode = new VNode(type, vNode.tag, childrenType, childrenType === 2 /* HasVNodeChildren */ ?
            directClone(vNode.children) :
            vNode.children.map(directClone), null, null, vNode.key, null);
    }
    else {
        var children = vNode.children;
        var childrenType = vNode.childrenType;
        newVNode = new VNode(type, vNode.tag, childrenType, 
        // should slice children array, if it is a element,
        // because we will asign new cloned vNode to the array
        // if the child vNode should be cloned
        type & 962 /* Element */ && childrenType & 12 /* MultipleChildren */ ?
            vNode.children.slice() :
            children, vNode.className, props, vNode.key, vNode.ref);
    }
    newVNode.transition = vNode.transition;
    // newVNode.newPosition = vNode.newPosition;
    // newVNode.position = vNode.position;
    return newVNode;
}
function normalizeChildren$1(vNode, children) {
    var newChildren;
    var newChildrenType = 1 /* HasInvalidChildren */;
    if (isInvalid(children)) {
        newChildren = children;
    }
    else if (isStringOrNumber(children)) {
        newChildrenType = 16 /* HasTextChildren */;
        newChildren = children;
    }
    else if (isArray(children)) {
        var len = children.length;
        var reference = { index: 0 };
        for (var i = 0; i < len; i++) {
            var n = children[i];
            if (isInvalid(n) || isArray(n)) {
                newChildren = newChildren || children.slice(0, i);
                _normalizeVNodes(children, newChildren, i, reference);
                break;
            }
            else if (isStringOrNumber(n)) {
                newChildren = newChildren || children.slice(0, i);
                newChildren.push(applyKey(createTextVNode(n), reference));
            }
            else {
                var key = n.key;
                var type = n.type;
                var needsCloning = (type & 24576 /* InUseOrNormalized */) > 0;
                var isNullOrUndefinedKey = isNullOrUndefined(key);
                var isPrefixed = (type & 32768 /* PrefixedKey */) > 0;
                if (needsCloning || isNullOrUndefinedKey || isPrefixed) {
                    newChildren = newChildren || children.slice(0, i);
                    if (needsCloning || isPrefixed) {
                        n = directClone(n);
                    }
                    if (isNullOrUndefinedKey || isPrefixed) {
                        applyKey(n, reference);
                    }
                    newChildren.push(n);
                }
                else if (newChildren) {
                    newChildren.push(n);
                }
                n.type |= 16384 /* Normalized */;
            }
            reference.index++;
        }
        newChildren = newChildren || children;
        if (newChildren.length === 0) {
            newChildrenType = 1 /* HasInvalidChildren */;
        }
        else {
            newChildrenType = 8 /* HasKeyedChildren */;
        }
    }
    else {
        newChildren = children;
        if (children.type & 24576 /* InUseOrNormalized */) {
            newChildren = directClone(children);
        }
        newChildren.type |= 16384 /* Normalized */;
        newChildrenType = 2 /* HasVNodeChildren */;
    }
    vNode.children = newChildren;
    vNode.childrenType = newChildrenType;
    return vNode;
}
function applyKey(vNode, reference) {
    vNode.key = '$' + reference.index;
    vNode.type |= 32768 /* PrefixedKey */;
    return vNode;
}
function _normalizeVNodes(vNodes, result, index, reference) {
    for (var len = vNodes.length; index < len; index++) {
        var n = vNodes[index];
        if (!isInvalid(n)) {
            if (isArray(n)) {
                _normalizeVNodes(n, result, 0, reference);
                continue;
            }
            else {
                if (isStringOrNumber(n)) {
                    n = applyKey(createTextVNode(n), reference);
                }
                else {
                    var type = n.type;
                    var isPrefixed = (type & 32768 /* PrefixedKey */) > 0;
                    if (type & 24576 /* InUseOrNormalized */ || isPrefixed) {
                        n = directClone(n);
                    }
                    n.type |= 16384 /* Normalized */;
                    if (isNullOrUndefined(n.key) || isPrefixed) {
                        applyKey(n, reference);
                    }
                }
                result.push(n);
            }
        }
        reference.index++;
    }
}
function normalizeRoot(vNode, parentVNode) {
    var root;
    if (isInvalid(vNode)) {
        root = createVoidVNode();
    }
    else if (isStringOrNumber(vNode)) {
        root = createTextVNode(vNode, null);
    }
    else if (isArray(vNode)) {
        root = createFragment(vNode, 0 /* UnknownChildren */, null);
    }
    else {
        root = vNode.type & 8192 /* InUse */ ? directClone(vNode) : vNode;
    }
    var transition = parentVNode.transition;
    if (!isNullOrUndefined(transition)) {
        // if it has multipe vNodes, only transition on the first vNode
        // if ("production" !== 'production') {
        // const type = root.type;
        // if (!(type & Types.Component || type & Types.Element)) {
        // throwError(`Component inside <Transtion> must render a single elememt node.`);
        // }
        // }
        root.transition = transition;
    }
    return root;
}

function linkEvent(data, event) {
    if (isFunction(event)) {
        return { data: data, event: event };
    }
    return null;
}
function isLinkEvent(o) {
    return !isNull(o) && typeof o === 'object';
}
function isSameLinkEvent(lastValue, nextValue) {
    return (isLinkEvent(lastValue) &&
        lastValue.event === nextValue.event &&
        lastValue.data === nextValue.data);
}
function wrapLinkEvent(nextValue) {
    var ev = nextValue.event;
    return function (e) {
        ev(nextValue.data, e);
    };
}

function getDelegatedEventObject(v) {
    return {
        'ev-click': v,
        'ev-dblclick': v,
        'ev-focusin': v,
        'ev-focusout': v,
        'ev-keydown': v,
        'ev-keypress': v,
        'ev-keyup': v,
        'ev-mousedown': v,
        'ev-mousemove': v,
        'ev-mouseup': v,
        'ev-touchend': v,
        'ev-touchmove': v,
        'ev-touchstart': v,
        'ev-contextmenu': v,
    };
}
var attachedEventCounts = /*#__PURE__*/ getDelegatedEventObject(0);
var attachedEvents = /*#__PURE__*/ getDelegatedEventObject(null);
var delegatedEvents = /*#__PURE__*/ getDelegatedEventObject(true);
function updateOrAddDelegatedEvent(name, dom) {
    var eventsObject = dom.$EV;
    if (!eventsObject) {
        eventsObject = dom.$EV = getDelegatedEventObject(null);
    }
    if (!eventsObject[name]) {
        if (++attachedEventCounts[name] === 1) {
            attachedEvents[name] = attachEventToDocument(name);
        }
    }
    return eventsObject;
}
function unmountDelegatedEvent(name, dom) {
    var eventsObject = dom.$EV;
    if (eventsObject && eventsObject[name]) {
        if (--attachedEventCounts[name] === 0) {
            document.removeEventListener(normalizeEventName(name), attachedEvents[name]);
            attachedEvents[name] = null;
        }
        eventsObject[name] = null;
    }
    return eventsObject;
}
function handleDelegatedEvent(name, lastEvent, nextEvent, dom) {
    if (isFunction(nextEvent)) {
        updateOrAddDelegatedEvent(name, dom)[name] = nextEvent;
    }
    else if (isLinkEvent(nextEvent)) {
        if (isSameLinkEvent(lastEvent, nextEvent)) {
            return;
        }
        updateOrAddDelegatedEvent(name, dom)[name] = nextEvent;
    }
    else {
        unmountDelegatedEvent(name, dom);
    }
}
function attachEventToDocument(name) {
    var attachedEvent = // name === 'ev-click' || name === 'ev-dblclick' ?
     
    // rootClickEvent(name) :
    rootEvent(name);
    document.addEventListener(normalizeEventName(name), attachedEvent);
    return attachedEvent;
}
// function rootClickEvent(name: string) {
// return function(event: Event) {
// if ((event as MouseEvent).button !== 0) {
// event.stopPropagation();
// return;
// }
// dispatchEvents(event, true, name, extendEventProperites(event));
// }
// }
function rootEvent(name) {
    var isClick = name === 'ev-click' || name === 'ev-dblclick';
    return function (event) {
        dispatchEvents(event, isClick, name, extendEventProperites(event));
    };
}
function extendEventProperites(event) {
    var eventData = {
        dom: document
    };
    event.stopPropagation = stopPropagation;
    Object.defineProperty(event, 'currentTarget', {
        configurable: true,
        get: function () {
            return eventData.dom;
        }
    });
    return eventData;
}
function stopPropagation() {
    this.cancelBubble = true;
    if (this.stopImmediatePropagation) {
        this.stopImmediatePropagation();
    }
}
function dispatchEvents(event, isClick, name, eventData) {
    var dom = getTargetNode(event);
    var count = attachedEventCounts[name];
    do {
        if (isClick && dom.disabled) {
            return;
        }
        var eventsObject = dom.$EV;
        if (eventsObject) {
            var currentEvent = eventsObject[name];
            if (currentEvent) {
                eventData.dom = dom;
                currentEvent.event ?
                    currentEvent.event(currentEvent.data, event) :
                    currentEvent(event);
                if (--count === 0 || event.cancelBubble) {
                    return;
                }
            }
        }
        dom = dom.parentNode;
    } while (!isNull(dom));
}
function getTargetNode(event) {
    return isFunction(event.composedPath) ?
        event.composedPath()[0] :
        /* istanbul ignore next */
        event.target;
}

function attachEvent(dom, eventName, handler) {
    var previousKey = "$".concat(eventName);
    var previousArgs = dom[previousKey];
    if (previousArgs) {
        dom.removeEventListener(previousArgs[0], previousArgs[1]);
        dom[previousKey] = null;
    }
    if (isFunction(handler)) {
        dom.addEventListener(eventName, handler);
        dom[previousKey] = [eventName, handler];
    }
}
function attachModelEvent(dom, eventName, handler) {
    var previousKey = "$$model:".concat(eventName);
    var previousEvent = dom[previousKey];
    if (previousEvent) {
        dom.removeEventListener(eventName, previousEvent);
        dom[previousKey] = null;
    }
    if (isFunction(handler)) {
        dom.addEventListener(eventName, handler);
        dom[previousKey] = handler;
    }
}

function applyValueInput(nextProps, dom, isControlled) {
    var type = nextProps.type, value = nextProps.value, checked = nextProps.checked, multiple = nextProps.multiple, defaultValue = nextProps.defaultValue;
    var hasValue = true;
    if (isNullOrUndefined(value)) {
        value = '';
        hasValue = false;
    }
    // FIXME: it seems like it need not handle because patchProp will handle it
    // if (type && type !== dom.type) {
    // dom.setAttribute('type', type);
    // }
    if (!isNullOrUndefined(multiple) && multiple !== dom.multiple) {
        dom.multiple = multiple;
    }
    if (!isNullOrUndefined(defaultValue) && !isControlled) {
        dom.defaultValue = defaultValue + '';
    }
    if (isCheckedType(type)) {
        if (isControlled || hasValue) {
            dom.value = value;
        }
        if (!isNullOrUndefined(checked)) {
            dom.checked = checked;
        }
    }
    else {
        if ((isControlled || hasValue) && dom.value !== value) {
            dom.defaultValue = value;
            dom.value = value;
        }
        else if (!isNullOrUndefined(checked)) {
            dom.checked = checked;
        }
    }
}
function isCheckedType(type) {
    return type === 'checkbox' || type === 'radio';
}

// export function selectEvents(dom: HTMLSelectElement, event?: EventListener) {
// attachModelEvent(dom, 'change', event);
// }
function applyValueSelect(nextProps, dom, mounting, vNode, isControlled) {
    var multiple = nextProps.multiple;
    if (!isNullOrUndefined(multiple) && multiple !== dom.multiple) {
        dom.multiple = multiple;
    }
    var index = nextProps.selectedIndex;
    if (index === -1) {
        dom.selectedIndex = -1;
    }
    var childrenType = vNode.childrenType;
    if (childrenType !== 1 /* HasInvalidChildren */) {
        var value = nextProps.value;
        if (isNumber(index) && index > -1 && dom.options[index]) {
            value = dom.options[index].value;
        }
        if (mounting && isNullOrUndefined(value)) {
            value = nextProps.defaultValue;
        }
        if (!isNullOrUndefined(value) || isControlled || multiple) {
            REFERENCE.value = false;
            updateChildOptions(vNode, value, REFERENCE);
            if (!REFERENCE.value) {
                dom.selectedIndex = -1;
            }
        }
    }
}
function updateChildOptions(vNode, value, flag) {
    if (vNode.tag === 'option') {
        updateChildOption(vNode, value, flag);
    }
    else {
        var children = vNode.children;
        var type = vNode.type;
        if (type & 8 /* ComponentClass */) {
            updateChildOptions(children.$lastInput, value, flag);
        }
        else if (type & 16 /* ComponentFunction */) {
            updateChildOptions(children, value, flag);
        }
        else if (vNode.childrenType === 2 /* HasVNodeChildren */) {
            updateChildOptions(children, value, flag);
        }
        else if (vNode.childrenType & 12 /* MultipleChildren */) {
            for (var i = 0, len = children.length; i < len; ++i) {
                updateChildOptions(children[i], value, flag);
            }
        }
    }
}
function updateChildOption(vNode, value, flag) {
    var props = vNode.props || EMPTY_OBJ;
    var dom = vNode.dom;
    // dom.value = props.value;
    if (props.value === value || isArray(value) && value.indexOf(props.value) !== -1) {
        flag.value = true;
        dom.selected = true;
    }
    else if (!isNullOrUndefined(value)) {
        // if exist value ignore the selected prop of option
        dom.selected = false;
    }
    else if (!isNullOrUndefined(props.selected)) {
        var selected = props.selected || false;
        if (selected)
            flag.value = true;
        dom.selected = selected;
    }
}

// export function textareaEvents(dom: HTMLTextAreaElement, event?: EventListener) {
// attachModelEvent(dom, 'input', event);
// }
function applyValueTextArea(nextProps, dom, mounting, isControlled) {
    var value = nextProps.value;
    var domValue = dom.value;
    if (!isControlled) {
        if (mounting) {
            var defaultValue = nextProps.defaultValue;
            if (!isNullOrUndefined(defaultValue) && defaultValue !== domValue) {
                dom.value = dom.defaultValue = defaultValue;
            }
        }
    }
    else if (domValue !== value) {
        dom.value = dom.defaultValue = isNullOrUndefined(value) ? '' : value;
    }
}

function processElement(type, vNode, dom, nextProps, mounting, isControlled) {
    if (type & 64 /* InputElement */) {
        applyValueInput(nextProps, dom, isControlled);
    }
    else if (type & 128 /* SelectElement */) {
        applyValueSelect(nextProps, dom, mounting, vNode, isControlled);
    }
    else if (type & 256 /* TextareaElement */) {
        applyValueTextArea(nextProps, dom, mounting, isControlled);
    }
}

function isRef(o) {
    return o.__is_ref;
}
function createRef(defaultValue) {
    return {
        value: isUndefined(defaultValue) ? null : defaultValue,
        __is_ref: true,
    };
}
/**
 * We must mount ref after all operations completed, because when we patch children,
 * one child will be mounted before being removed. e.g.
 * patchTest(
 *     h('div', null, [h('div'), h('div', {ref})]),
 *     h('div', null, [h('div', {ref})]),
 * );
 */
function mountRef(ref, value, mountedQueue) {
    if (ref) {
        /**
         * We mount ref before doing anything else, so that we can obtain the correct ref anywhere.
         */
        mountedQueue.unshift(function () {
            if (isFunction(ref)) {
                ref(value);
            }
            else if (isRef(ref)) {
                ref.value = value;
            }
        });
    }
}
function unmountRef(ref) {
    if (ref) {
        if (isFunction(ref)) {
            ref(null);
        }
        else if (isRef(ref)) {
            ref.value = null;
        }
    }
}

function remove(vNode, parentDom, reusing) {
    if (!reusing) {
        unmount(vNode, null);
    }
    removeVNodeDom(vNode, parentDom);
}
function unmount(vNode, nextVNode) {
    var type = vNode.type;
    var children = vNode.children;
    if (type & 962 /* Element */) {
        var props = vNode.props;
        unmountRef(vNode.ref);
        var childrenType = vNode.childrenType;
        if (!isNullOrUndefined(props)) {
            for (var prop in props) {
                if (delegatedEvents[prop]) {
                    unmountDelegatedEvent(prop, vNode.dom);
                }
            }
        }
        if (childrenType & 12 /* MultipleChildren */) {
            unmountAllChildren(children);
        }
        else if (childrenType === 2 /* HasVNodeChildren */) {
            unmount(children, null);
        }
    }
    else if (children) {
        if (type & 8 /* ComponentClass */) {
            unmountComponentClass(vNode, nextVNode);
        }
        else if (type & 16 /* ComponentFunction */) {
            unmount(children, null);
        }
        else if (type & 4096 /* Fragment */) {
            if (vNode.childrenType & 12 /* MultipleChildren */) {
                unmountAllChildren(children);
            }
        }
    }
}
function unmountComponentClass(vNode, nextVNode) {
    vNode.children.$unmount(vNode, nextVNode);
    unmountRef(vNode.ref);
}
function unmountAllChildren(children) {
    for (var i = 0, len = children.length; i < len; ++i) {
        unmount(children[i], null);
    }
}
function clearDom(dom) {
    dom.textContent = '';
}
function removeAllChildren(children, dom, vNode, reusing) {
    if (!reusing) {
        unmountAllChildren(children);
    }
    if (vNode.type & 4096 /* Fragment */) {
        removeVNodeDom(vNode, dom);
    }
    else {
        for (var i = 0; i < children.length; i++) {
            removeVNodeDom(children[i], dom);
        }
        // clearDom(dom);
    }
}

function mountProps$1(vNode, type, props, dom, isSVG) {
    var isFormElement = (type & 448 /* FormElement */) > 0;
    REFERENCE.value = false;
    for (var prop in props) {
        patchProp$1(prop, null, props[prop], dom, isSVG, isFormElement, REFERENCE, null);
    }
    if (isFormElement) {
        processElement(type, vNode, dom, props, true, REFERENCE.value);
    }
}
function patchProp$1(prop, lastValue, nextValue, dom, isSVG, isFormElement, hasControlledValue, lastVNode) {
    var value;
    switch (prop) {
        case 'children':
        case 'className':
        case 'defaultValue':
        case 'key':
        case 'ref':
        case 'multiple':
        case 'selectedIndex':
        case '$blocks':
            break;
        // case 'autoFocus':
        // (dom as any).autofocus = !!nextValue;
        // break;
        case 'autofocus':
        case 'allowfullscreen':
        case 'autoplay':
        case 'capture':
        case 'checked':
        case 'controls':
        case 'default':
        case 'disabled':
        case 'hidden':
        case 'indeterminate':
        case 'loop':
        case 'muted':
        case 'novalidte':
        case 'open':
        case 'readOnly':
        case 'required':
        case 'reversed':
        case 'scoped':
        case 'seamless':
        case 'selected':
        case 'spellcheck':
            dom[prop] = !!nextValue;
            break;
        // special bool value
        case 'formnovalidate':
            dom.formNoValidate = !!nextValue;
            break;
        case 'defaultChecked':
        case 'value':
        case 'volume':
            // if (hasControlledValue && prop === 'value') break;
            if (prop === 'value') {
                dom.$VA = nextValue;
                if (isFormElement) {
                    hasControlledValue.value = true;
                    break;
                }
            }
            value = isNullOrUndefined(nextValue) ? '' : nextValue;
            if (dom[prop] !== value) {
                dom[prop] = value;
            }
            break;
        // TODO: scrollLeft/scrollTop must be handled after style and mounted (use mountedQueue)
        case 'scrollLeft':
        case 'scrollTop':
            value = isNullOrUndefined(nextValue) ? 0 : nextValue;
            if (dom[prop] !== value) {
                dom[prop] = value;
            }
            break;
        case 'innerHTML':
            patchInnerHTML(lastValue, nextValue, dom, lastVNode);
            break;
        case 'style':
            patchStyle(lastValue, nextValue, dom);
            break;
        // handle v-model
        case '$model:value':
            dom.$M = nextValue;
            break;
        case 'trueValue':
            dom.$TV = nextValue;
            break;
        case 'falseValue':
            dom.$FV = nextValue;
            break;
        case 'ev-$model:change':
            patchModelEvent('change', lastValue, nextValue, dom);
            break;
        case 'ev-$model:input':
            patchModelEvent('input', lastValue, nextValue, dom);
            break;
        default:
            if (delegatedEvents[prop]) {
                handleDelegatedEvent(prop, lastValue, nextValue, dom);
            }
            else if (isEventProp(prop)) {
                patchEvent(prop, lastValue, nextValue, dom);
            }
            else if (isNullOrUndefined(nextValue)) {
                dom.removeAttribute(prop);
            }
            else if (isSVG && namespaces[prop]) {
                dom.setAttributeNS(namespaces[prop], prop, nextValue);
            }
            else {
                dom.setAttribute(prop, nextValue);
            }
            break;
    }
}
function patchStyle(lastValue, nextValue, dom) {
    if (isNullOrUndefined(nextValue)) {
        dom.removeAttribute('style');
    }
    else {
        var domStyle = dom.style;
        if (isString(nextValue)) {
            domStyle.cssText = nextValue;
        }
        else {
            var style = void 0;
            var value = void 0;
            var hasLastValue = !isNullOrUndefined(lastValue);
            if (hasLastValue && !isString(lastValue)) {
                for (style in nextValue) {
                    value = nextValue[style];
                    if (value !== lastValue[style]) {
                        domStyle[style] = value;
                    }
                }
                for (style in lastValue) {
                    if (isNullOrUndefined(nextValue[style])) {
                        domStyle[style] = '';
                    }
                }
            }
            else {
                if (hasLastValue) {
                    // remove style firstly if lastValue is string
                    dom.removeAttribute('style');
                }
                for (style in nextValue) {
                    value = nextValue[style];
                    domStyle[style] = value;
                }
            }
        }
    }
    // handle Transition display
    var display = dom.$TD;
    if (!isUndefined(display)) {
        dom.style.display = display;
    }
}
function patchEvent(name, lastValue, nextValue, dom) {
    if (isLinkEvent(nextValue)) {
        if (isSameLinkEvent(lastValue, nextValue)) {
            return;
        }
        nextValue = wrapLinkEvent(nextValue);
    }
    attachEvent(dom, normalizeEventName(name), nextValue);
}
function patchModelEvent(name, lastValue, nextValue, dom) {
    if (isLinkEvent(nextValue)) {
        if (isSameLinkEvent(lastValue, nextValue)) {
            return;
        }
        nextValue = wrapLinkEvent(nextValue);
    }
    attachModelEvent(dom, name, nextValue);
}
function patchInnerHTML(lastValue, nextValue, dom, lastVNode) {
    if (isNullOrUndefined(lastValue)) {
        lastValue = '';
    }
    if (isNullOrUndefined(nextValue)) {
        nextValue = '';
    }
    if (lastValue !== nextValue) {
        if (!isNull(lastVNode)) {
            if (lastVNode.childrenType & 12 /* MultipleChildren */) {
                unmountAllChildren(lastVNode.children);
            }
            else if (lastVNode.childrenType === 2 /* HasVNodeChildren */) {
                unmount(lastVNode.children, null);
            }
            lastVNode.children = null;
            lastVNode.childrenType = 1 /* HasInvalidChildren */;
        }
        dom.innerHTML = nextValue;
    }
}

function mount(vNode, parentDom, parentComponent, isSVG, anchor, mountedQueue) {
    var type = (vNode.type |= 8192 /* InUse */);
    if (type & 962 /* Element */) {
        mountElement(vNode, parentDom, parentComponent, isSVG, anchor, mountedQueue);
    }
    else if (type & 8 /* ComponentClass */) {
        mountComponentClass(null, vNode, parentDom, parentComponent, isSVG, anchor, mountedQueue);
    }
    else if (type & 16 /* ComponentFunction */) {
        mountComponentFunction(vNode, parentDom, parentComponent, isSVG, anchor, mountedQueue);
    }
    else if (type & 1 /* Text */ /*  || type & Types.Void */) {
        mountText(vNode, parentDom, anchor);
    }
    else if (type & 4096 /* Fragment */) {
        mountFragment(vNode, parentDom, parentComponent, isSVG, anchor, mountedQueue);
    }
    else if (type & 32 /* HtmlComment */) {
        mountComment(vNode, parentDom, anchor);
    }
    else ;
}
function mountElement(vNode, parentDom, parentComponent, isSVG, anchor, mountedQueue) {
    var type = vNode.type, props = vNode.props, className = vNode.className, childrenType = vNode.childrenType, tag = vNode.tag, transition = vNode.transition;
    isSVG = isSVG || (type & 512 /* SvgElement */) > 0;
    var dom = vNode.dom = documentCreateElement(tag, isSVG);
    if (!isNullOrUndefined(className) && className !== '') {
        if (isSVG) {
            dom.setAttribute('class', className);
        }
        else {
            dom.className = className;
        }
    }
    var children = vNode.children;
    // We should mount Ref firstly, because we may get parent ref in children component
    // in $render method, e.g. Portal of KPC
    // mountRef(vNode.ref, dom);
    // @MODIFY: we have to mount ref in mountedQueuek
    if (childrenType === 16 /* HasTextChildren */) {
        setTextContent(dom, children);
    }
    else if (childrenType !== 1 /* HasInvalidChildren */) {
        var childrenIsSVG = isSVG && tag !== 'foreignObject';
        if (childrenType === 2 /* HasVNodeChildren */) {
            if (children.type & 8192 /* InUse */) {
                vNode.children = children = directClone(children);
            }
            mount(children, dom, parentComponent, childrenIsSVG, null, mountedQueue);
        }
        else if (childrenType === 8 /* HasKeyedChildren */ ||
            childrenType === 4 /* HasNonKeyedChildren */) {
            mountArrayChildren(children, vNode, dom, parentComponent, childrenIsSVG, null, mountedQueue);
        }
    }
    var beforeInsert = hooks.beforeInsert;
    if (!isNullOrUndefined(beforeInsert)) {
        beforeInsert(vNode, parentComponent);
    }
    if (!isNullOrUndefined(parentDom)) {
        insertOrAppend(parentDom, dom, anchor);
    }
    if (!isNullOrUndefined(props)) {
        mountProps$1(vNode, type, props, dom, isSVG);
    }
    if (!isNullOrUndefined(transition) && transition.show) {
        transition.beforeEnter(dom);
        mountedQueue.push(function () { return transition.enter(dom); });
    }
    mountRef(vNode.ref, dom, mountedQueue);
}
function mountComponentClass(lastVNode, vNode, parentDom, parentComponent, isSVG, anchor, mountedQueue) {
    var props = vNode.props;
    var instance = new vNode.tag(props, vNode, isSVG, mountedQueue, parentComponent);
    // instance.$SVG = isSVG;
    // instance.$vNode = vNode;
    // instance.$mountedQueue = mountedQueue;
    // instance.$senior = parentComponent;
    instance.$init(props);
    vNode.children = instance;
    instance.$render(lastVNode, vNode, parentDom, anchor, mountedQueue);
    mountRef(vNode.ref, instance, mountedQueue);
    // mountedQueue.push(() => instance.$mount(lastVNode, vNode))
}
function mountComponentFunction(vNode, parentDom, parentComponent, isSVG, anchor, mountedQueue) {
    mount((vNode.children = normalizeRoot(vNode.tag(vNode.props || EMPTY_OBJ), vNode)), parentDom, parentComponent, isSVG, anchor, mountedQueue);
}
function mountText(vNode, parentDom, anchor) {
    var dom = vNode.dom = document.createTextNode(vNode.children);
    if (!isNullOrUndefined(parentDom)) {
        insertOrAppend(parentDom, dom, anchor);
    }
}
function mountFragment(vNode, parentDom, parentComponent, isSVG, anchor, mountedQueue) {
    // let children = vNode.children;
    // let childrenType = vNode.childrenType;
    // if (childrenType & ChildrenTypes.MultipleChildren && (children as VNode[]).length === 0) {
    // childrenType = vNode.childrenType = ChildrenTypes.HasVNodeChildren;
    // children = vNode.children = createVNode();
    // }
    if (vNode.childrenType === 2 /* HasVNodeChildren */) {
        mount(vNode.children, parentDom, parentComponent, isSVG, anchor, mountedQueue);
    }
    else {
        mountArrayChildren(vNode.children, vNode, parentDom, parentComponent, isSVG, anchor, mountedQueue);
    }
}
function mountComment(vNode, parentDom, anchor) {
    var dom = vNode.dom = document.createComment(vNode.children);
    if (!isNullOrUndefined(parentDom)) {
        insertOrAppend(parentDom, dom, anchor);
    }
}
function mountArrayChildren(children, parentVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue) {
    for (var i = 0; i < children.length; i++) {
        var vNode = children[i];
        if (vNode.type & 8192 /* InUse */) {
            // if (!cloned) {
            // parentVNode.children = children = children.slice();
            // cloned = true;
            // }
            children[i] = vNode = directClone(vNode);
        }
        mount(vNode, parentDom, parentComponent, isSVG, anchor, mountedQueue);
    }
}
function documentCreateElement(tag, isSVG) {
    if (isSVG) {
        return document.createElementNS("http://www.w3.org/2000/svg", tag);
    }
    return document.createElement(tag);
}
// export function createComponentClassInstance(
// lastVNode: VNodeComponentClass | null,
// vNode: VNodeComponentClass,
// Component: ComponentConstructor,
// props: Props,
// isSVG: boolean,
// mountedQueue: Function[],
// ) {
// }

function patch(lastVNode, nextVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing) {
    var nextType = (nextVNode.type |= 8192 /* InUse */);
    if (lastVNode.type !== nextType || lastVNode.key !== nextVNode.key) {
        replaceWithNewNode(lastVNode, nextVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing);
    }
    else if (nextType & 8 /* ComponentClass */) {
        // if patch two class components, reuse its dom otherwise replace the dom totally.
        patchComponentClass(lastVNode, nextVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing);
    }
    else if (lastVNode.tag !== nextVNode.tag) {
        replaceWithNewNode(lastVNode, nextVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing);
    }
    else if (nextType & 962 /* Element */) {
        patchElement(lastVNode, nextVNode, parentComponent, isSVG, nextType, mountedQueue, reusing);
    }
    else if (nextType & 16 /* ComponentFunction */) {
        patchComponentFunction(lastVNode, nextVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing);
    }
    else if (nextType & 1 /* Text */ || nextType & 32 /* HtmlComment */) {
        patchText(lastVNode, nextVNode);
    }
    else if (nextType & 4096 /* Fragment */) {
        patchFragment(lastVNode, nextVNode, parentDom, parentComponent, isSVG, mountedQueue, reusing);
    }
}
function replaceWithNewNode(lastVNode, nextVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing) {
    if (lastVNode.type & 8192 /* InUse */) {
        if (!reusing) {
            unmount(lastVNode, null);
        }
        if ((nextVNode.type & lastVNode.type & 3043 /* HtmlElement */) && isUndefined(lastVNode.transition)) {
            // single dom
            mount(nextVNode, null, parentComponent, isSVG, null, mountedQueue);
            replaceChild(parentDom, nextVNode.dom, lastVNode.dom);
        }
        else {
            mount(nextVNode, parentDom, parentComponent, isSVG, 
            // we must insert the dom after the old dom, because it may has Transition
            // Inserting it after the old one to make the old dom's position don't change
            findDomFromVNode(lastVNode, true).nextSibling, mountedQueue);
            removeVNodeDom(lastVNode, parentDom);
        }
    }
    else {
        // Last vNode is not in use, it has crashed. Just mount next vNode and ignore last one.
        mount(nextVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue);
    }
}
function patchComponentClass(lastVNode, nextVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing) {
    var nextTag = nextVNode.tag;
    if (lastVNode.tag !== nextTag || reusing) {
        if (!reusing) {
            unmount(lastVNode, nextVNode);
        }
        mountComponentClass(lastVNode, nextVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue);
    }
    else {
        var instance = nextVNode.children = lastVNode.children;
        // If component has crashed, ignore it to stay functional
        /* istanbul ignore next */
        if (isNullOrUndefined(instance))
            return;
        instance.$mountedQueue = mountedQueue;
        instance.$vNode = nextVNode;
        instance.$senior = parentComponent;
        instance.$update(lastVNode, nextVNode, parentDom, anchor, mountedQueue, false);
        var lastRef = lastVNode.ref;
        var nextRef = nextVNode.ref;
        if (lastRef !== nextRef) {
            unmountRef(lastRef);
            mountRef(nextRef, instance, mountedQueue);
        }
    }
}
function patchComponentFunction(lastVNode, nextVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing) {
    var nextChildren = normalizeRoot(nextVNode.tag(nextVNode.props || EMPTY_OBJ), nextVNode);
    patch(lastVNode.children, nextChildren, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing);
    nextVNode.children = nextChildren;
}
function patchElement(lastVNode, nextVNode, parentComponent, isSVG, nextType, mountedQueue, reusing) {
    var dom = nextVNode.dom = lastVNode.dom;
    var lastProps = lastVNode.props || EMPTY_OBJ;
    var nextProps = nextVNode.props || EMPTY_OBJ;
    isSVG = isSVG || (nextType & 512 /* SvgElement */) > 0;
    REFERENCE.value = false;
    var isFormElement = false;
    if (lastProps !== nextProps) {
        if (nextProps !== EMPTY_OBJ) {
            isFormElement = (nextType & 448 /* FormElement */) > 0;
            for (var prop in nextProps) {
                // maybe the lastProps is the same as nextProps, we can't go to patchProp
                if (prop === 'value')
                    REFERENCE.value = true;
                var lastValue = lastProps[prop];
                var nextValue = nextProps[prop];
                if (lastValue !== nextValue) {
                    patchProp$1(prop, lastValue, nextValue, dom, isSVG, isFormElement, REFERENCE, lastVNode);
                }
            }
        }
        if (lastProps !== EMPTY_OBJ) {
            for (var prop in lastProps) {
                if (isNullOrUndefined(nextProps[prop]) && !isNullOrUndefined(lastProps[prop])) {
                    patchProp$1(prop, lastProps[prop], null, dom, isSVG, isFormElement, REFERENCE, lastVNode);
                }
            }
        }
    }
    // patch className
    var nextClassName = nextVNode.className;
    if (lastVNode.className !== nextClassName) {
        var transitionClassname = dom.$TC;
        if (isNullOrUndefined(nextClassName)) {
            if (isUndefined(transitionClassname)) {
                dom.removeAttribute('class');
            }
            else {
                dom.className = Object.keys(transitionClassname).join(' ');
            }
        }
        else if (isSVG) {
            dom.setAttribute('class', nextClassName);
        }
        else if (isUndefined(transitionClassname)) {
            dom.className = nextClassName;
        }
        else {
            dom.className = nextClassName + ' ' + Object.keys(transitionClassname).join(' ');
        }
    }
    var nextChildren = nextVNode.children;
    var nextChildrenType = nextVNode.childrenType;
    if (nextChildrenType === 2 /* HasVNodeChildren */) {
        if (nextChildren.type & 8192 /* InUse */) {
            nextVNode.children = nextChildren = directClone(nextChildren);
        }
    }
    // TODO: patchContendEditableChild
    // if (nextType & )
    patchChildren(lastVNode.childrenType, nextChildrenType, lastVNode.children, nextChildren, dom, parentComponent, isSVG && nextVNode.tag !== 'foreignObject', null, lastVNode, mountedQueue, reusing);
    if (isFormElement) {
        processElement(nextType, nextVNode, dom, nextProps, false, REFERENCE.value);
    }
    var nextRef = nextVNode.ref;
    if (reusing) {
        mountRef(nextRef, dom, mountedQueue);
    }
    else {
        var lastRef = lastVNode.ref;
        if (lastRef !== nextRef) {
            unmountRef(lastRef);
            mountRef(nextRef, dom, mountedQueue);
        }
    }
}
function patchText(lastVNode, nextVNode) {
    var nextText = nextVNode.children;
    var dom = nextVNode.dom = lastVNode.dom;
    if (nextText !== lastVNode.children) {
        dom.nodeValue = nextText;
    }
}
function patchFragment(lastVNode, nextVNode, parentDom, parentComponent, isSVG, mountedQueue, reusing) {
    var lastChildren = lastVNode.children;
    var nextChildren = nextVNode.children;
    var lastChildrenType = lastVNode.childrenType;
    var nextChildrenType = nextVNode.childrenType;
    var anchor = null;
    // if (nextChildrenType & ChildrenTypes.MultipleChildren && (nextChildren as VNode[]).length === 0) {
    // nextChildrenType = nextVNode.childrenType = ChildrenTypes.HasVNodeChildren;
    // nextChildren = nextVNode.children = createVoidVNode();
    // }
    var nextIsSingle = (nextChildrenType & 2 /* HasVNodeChildren */) !== 0;
    if (lastChildrenType & 12 /* MultipleChildren */) {
        var lastLen = lastChildren.length;
        // We need to known Fragment's edge node when
        if (
        // It uses keyed algorithm
        (lastChildrenType & nextChildrenType & 8 /* HasKeyedChildren */) ||
            // It transforms from may to signle
            nextIsSingle ||
            // It will append more nodes
            nextChildren.length > lastLen) {
            // When Fragment has mutliple children there is always at least one vNode
            anchor = findDomFromVNode(lastChildren[lastLen - 1], false).nextSibling;
        }
    }
    patchChildren(lastChildrenType, nextChildrenType, lastChildren, nextChildren, parentDom, parentComponent, isSVG, anchor, lastVNode, mountedQueue, reusing);
}
function patchChildren(lastChildrenType, nextChildrenType, lastChildren, nextChildren, parentDom, parentComponent, isSVG, anchor, parentVNode, mountedQueue, reusing) {
    switch (lastChildrenType) {
        case 2 /* HasVNodeChildren */:
            switch (nextChildrenType) {
                case 2 /* HasVNodeChildren */:
                    patch(lastChildren, nextChildren, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing);
                    break;
                case 1 /* HasInvalidChildren */:
                    remove(lastChildren, parentDom, reusing);
                    break;
                case 16 /* HasTextChildren */:
                    if (!reusing) {
                        unmount(lastChildren, null);
                    }
                    setTextContent(parentDom, nextChildren);
                    break;
                default:
                    replaceOneVNodeWithMulipleVNodes(lastChildren, nextChildren, parentVNode, parentDom, parentComponent, isSVG, mountedQueue, reusing);
                    break;
            }
            break;
        case 1 /* HasInvalidChildren */:
            switch (nextChildrenType) {
                case 2 /* HasVNodeChildren */:
                    mount(nextChildren, parentDom, parentComponent, isSVG, anchor, mountedQueue);
                    break;
                case 1 /* HasInvalidChildren */:
                    break;
                case 16 /* HasTextChildren */:
                    setTextContent(parentDom, nextChildren);
                    break;
                default:
                    mountArrayChildren(nextChildren, parentVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue);
                    break;
            }
            break;
        case 16 /* HasTextChildren */:
            switch (nextChildrenType) {
                case 2 /* HasVNodeChildren */:
                    clearDom(parentDom);
                    mount(nextChildren, parentDom, parentComponent, isSVG, anchor, mountedQueue);
                    break;
                case 1 /* HasInvalidChildren */:
                    clearDom(parentDom);
                    break;
                case 16 /* HasTextChildren */:
                    patchSingleTextChild(lastChildren, nextChildren, parentDom);
                    break;
                default:
                    clearDom(parentDom);
                    mountArrayChildren(nextChildren, parentVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue);
                    break;
            }
            break;
        default:
            switch (nextChildrenType) {
                case 2 /* HasVNodeChildren */:
                    removeAllChildren(lastChildren, parentDom, parentVNode, reusing);
                    mount(nextChildren, parentDom, parentComponent, isSVG, anchor, mountedQueue);
                    break;
                case 1 /* HasInvalidChildren */:
                    removeAllChildren(lastChildren, parentDom, parentVNode, reusing);
                    break;
                case 16 /* HasTextChildren */:
                    removeAllChildren(lastChildren, parentDom, parentVNode, reusing);
                    setTextContent(parentDom, nextChildren);
                    break;
                default:
                    var lastLength = lastChildren.length;
                    var nextLength = nextChildren.length;
                    if (lastLength === 0) {
                        if (nextLength > 0) {
                            mountArrayChildren(nextChildren, parentVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue);
                        }
                    }
                    else if (nextLength === 0) {
                        removeAllChildren(lastChildren, parentDom, parentVNode, reusing);
                    }
                    else if (nextChildrenType === 8 /* HasKeyedChildren */ &&
                        lastChildrenType === 8 /* HasKeyedChildren */) {
                        patchKeyedChildren(lastChildren, nextChildren, parentDom, parentComponent, isSVG, lastLength, nextLength, anchor, parentVNode, mountedQueue, reusing);
                    }
                    else {
                        patchNonKeyedChildren(lastChildren, nextChildren, parentDom, parentComponent, isSVG, lastLength, nextLength, anchor, mountedQueue, reusing);
                    }
                    break;
            }
            break;
    }
}
function replaceOneVNodeWithMulipleVNodes(lastVNode, nextVNodes, parentVNode, parentDom, parentComponent, isSVG, mountedQueue, reusing) {
    if (!reusing) {
        unmount(lastVNode, null);
    }
    mountArrayChildren(nextVNodes, parentVNode, parentDom, parentComponent, isSVG, findDomFromVNode(lastVNode, true), mountedQueue);
    removeVNodeDom(lastVNode, parentDom);
}
function patchSingleTextChild(lastChildren, nextChildren, parentDom) {
    if (lastChildren !== nextChildren) {
        if (lastChildren !== '') {
            parentDom.firstChild.nodeValue = nextChildren;
        }
        else {
            setTextContent(parentDom, nextChildren);
        }
    }
}
function patchNonKeyedChildren(lastChildren, nextChildren, parentDom, parentComponent, isSVG, lastChildrenLength, nextChildrenLength, anchor, mountedQueue, reusing) {
    var commonLength = lastChildrenLength > nextChildrenLength ? nextChildrenLength : lastChildrenLength;
    var i = 0;
    var nextChild;
    var lastChild;
    for (; i < commonLength; ++i) {
        nextChild = nextChildren[i];
        lastChild = lastChildren[i];
        if (nextChild.type & 8192 /* InUse */) {
            nextChild = nextChildren[i] = directClone(nextChild);
        }
        patch(lastChild, nextChild, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing);
        // FIXME: why should update last children?
        // lastChildren[i] = nextChild;
    }
    if (lastChildrenLength < nextChildrenLength) {
        for (i = commonLength; i < nextChildrenLength; ++i) {
            nextChild = nextChildren[i];
            if (nextChild.type & 8192 /* InUse */) {
                nextChild = nextChildren[i] = directClone(nextChild);
            }
            mount(nextChild, parentDom, parentComponent, isSVG, anchor, mountedQueue);
        }
    }
    else if (lastChildrenLength > nextChildrenLength) {
        for (i = commonLength; i < lastChildrenLength; ++i) {
            remove(lastChildren[i], parentDom, reusing);
        }
    }
}
// https://github.com/localvoid/ivi/blob/e23c8572c9ef434c9ac1433e70eff17a88cb8362/packages/ivi/src/vdom/reconciler.ts#L585
// https://yanni4night.github.io/js/2018/02/11/inferno-dom-diff.html
function patchKeyedChildren(a, b, parentDom, parentComponent, isSVG, aLength, bLength, anchor, parentVNode, mountedQueue, reusing) {
    var aEnd = aLength - 1;
    var bEnd = bLength - 1;
    var j = 0;
    var aNode = a[j];
    var bNode = b[j];
    var nextPos;
    var nextNode;
    // Step 1
    outer: {
        // Sync nodes with the same key at the beginning.
        while (aNode.key === bNode.key) {
            if (bNode.type & 8192 /* InUse */) {
                b[j] = bNode = directClone(bNode);
            }
            patch(aNode, bNode, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing);
            // a[j] = bNode;
            ++j;
            if (j > aEnd || j > bEnd) {
                break outer;
            }
            aNode = a[j];
            bNode = b[j];
        }
        aNode = a[aEnd];
        bNode = b[bEnd];
        // Sync nodes with the same key at the end.
        while (aNode.key === bNode.key) {
            if (bNode.type & 8192 /* InUse */) {
                b[bEnd] = bNode = directClone(bNode);
            }
            patch(aNode, bNode, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing);
            // a[aEnd] = bNode;
            --aEnd;
            --bEnd;
            if (j > aEnd || j > bEnd) {
                break outer;
            }
            aNode = a[aEnd];
            bNode = b[bEnd];
        }
    }
    if (j > aEnd) {
        if (j <= bEnd) {
            nextPos = bEnd + 1;
            nextNode = nextPos < bLength ? findDomFromVNode(b[nextPos], true) : anchor;
            while (j <= bEnd) {
                bNode = b[j];
                if (bNode.type & 8192 /* InUse */) {
                    b[j] = bNode = directClone(bNode);
                }
                ++j;
                mount(bNode, parentDom, parentComponent, isSVG, nextNode, mountedQueue);
            }
        }
    }
    else if (j > bEnd) {
        while (j <= aEnd) {
            remove(a[j++], parentDom, reusing);
        }
    }
    else {
        patchKeyedChildrenComplex(a, b, aLength, bLength, aEnd, bEnd, j, parentDom, parentComponent, isSVG, anchor, parentVNode, mountedQueue, reusing);
    }
}
function patchKeyedChildrenComplex(a, b, aLength, bLength, aEnd, bEnd, j, parentDom, parentComponent, isSVG, anchor, parentVNode, mountedQueue, reusing) {
    var aNode;
    var bNode;
    var nextPos;
    var i = 0;
    var aStart = j;
    var bStart = j;
    var aLeft = aEnd - j + 1;
    var bLeft = bEnd - j + 1;
    var sources = new Int32Array(bLeft + 1);
    // Keep track if it's possible to remove whole DOM using textContent = '';
    var canRemoveWholeContent = aLeft === aLength;
    var moved = false;
    var pos = 0;
    var patched = 0;
    // When sizes are small, just loop them through
    if (bLength < 4 || (aLeft | bLeft) < 32) {
        for (i = aStart; i <= aEnd; ++i) {
            aNode = a[i];
            if (patched < bLeft) {
                for (j = bStart; j <= bEnd; ++j) {
                    bNode = b[j];
                    if (aNode.key === bNode.key) {
                        sources[j - bStart] = i + 1;
                        if (canRemoveWholeContent) {
                            canRemoveWholeContent = false;
                            // remove nodes before the key
                            while (aStart < i) {
                                remove(a[aStart++], parentDom, reusing);
                            }
                        }
                        if (pos > j) {
                            moved = true;
                        }
                        else {
                            pos = j;
                        }
                        if (bNode.type & 8192 /* InUse */) {
                            b[j] = bNode = directClone(bNode);
                        }
                        patch(aNode, bNode, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing);
                        ++patched;
                        break;
                    }
                }
                // remove node if it does not find
                if (!canRemoveWholeContent && j > bEnd) {
                    remove(aNode, parentDom, reusing);
                }
            }
            else if (!canRemoveWholeContent) {
                // remove node that exceeds the length
                remove(aNode, parentDom, reusing);
            }
        }
    }
    else {
        var keyIndex = {};
        // Map keys by their index
        for (i = bStart; i <= bEnd; ++i) {
            keyIndex[b[i].key] = i;
        }
        // Try to patch same keys
        for (i = aStart; i <= aEnd; ++i) {
            aNode = a[i];
            if (patched < bLeft) {
                j = keyIndex[aNode.key];
                if (j !== undefined) {
                    if (canRemoveWholeContent) {
                        canRemoveWholeContent = false;
                        while (aStart < i) {
                            remove(a[aStart++], parentDom, reusing);
                        }
                    }
                    sources[j - bStart] = i + 1;
                    if (pos > j) {
                        moved = true;
                    }
                    else {
                        pos = j;
                    }
                    bNode = b[j];
                    if (bNode.type & 8192 /* InUse */) {
                        b[j] = bNode = directClone(bNode);
                    }
                    patch(aNode, bNode, parentDom, parentComponent, isSVG, anchor, mountedQueue, reusing);
                    ++patched;
                }
                else if (!canRemoveWholeContent) {
                    remove(aNode, parentDom, reusing);
                }
            }
            else if (!canRemoveWholeContent) {
                remove(aNode, parentDom, reusing);
            }
        }
    }
    // fast-path: if nothing patched remove all old and add all new
    if (canRemoveWholeContent) {
        removeAllChildren(a, parentDom, parentVNode, reusing);
        mountArrayChildren(b, parentVNode, parentDom, parentComponent, isSVG, anchor, mountedQueue);
    }
    else if (moved) {
        var seq = lisAlgorithm(sources);
        j = seq.length - 1;
        for (i = bLeft - 1; i >= 0; i--) {
            if (sources[i] === 0) {
                pos = i + bStart;
                bNode = b[pos];
                if (bNode.type & 8192 /* InUse */) {
                    b[pos] = bNode = directClone(bNode);
                }
                nextPos = pos + 1;
                mount(bNode, parentDom, parentComponent, isSVG, nextPos < bLength ? findDomFromVNode(b[nextPos], true) : anchor, mountedQueue);
            }
            else if (j < 0 || i !== seq[j]) {
                pos = i + bStart;
                bNode = b[pos];
                nextPos = pos + 1;
                moveVNodeDom(bNode, parentDom, nextPos < bLength ? findDomFromVNode(b[nextPos], true) : anchor);
            }
            else {
                --j;
            }
        }
    }
    else if (patched !== bLeft) {
        // when patched count doesn't match b length we need to insert those new ones
        // loop backwards so we can use insertBefore
        for (i = bLeft - 1; i >= 0; --i) {
            if (sources[i] === 0) {
                pos = i + bStart;
                bNode = b[pos];
                if (bNode.type & 8192 /* InUse */) {
                    b[pos] = bNode = directClone(bNode);
                }
                nextPos = pos + 1;
                mount(bNode, parentDom, parentComponent, isSVG, nextPos < bLength ? findDomFromVNode(b[nextPos], true) : anchor, mountedQueue);
            }
        }
    }
}
var result;
var p;
var maxLen = 0;
// https://en.wikipedia.org/wiki/Longest_increasing_subsequence
function lisAlgorithm(arr) {
    var arrI = 0;
    var i = 0;
    var j = 0;
    var k = 0;
    var u = 0;
    var v = 0;
    var c = 0;
    var len = arr.length;
    if (len > maxLen) {
        maxLen = len;
        result = new Int32Array(len);
        p = new Int32Array(len);
    }
    for (; i < len; ++i) {
        arrI = arr[i];
        if (arrI !== 0) {
            j = result[k];
            if (arr[j] < arrI) {
                p[i] = j;
                result[++k] = i;
                continue;
            }
            u = 0;
            v = k;
            while (u < v) {
                c = (u + v) >> 1;
                if (arr[result[c]] < arrI) {
                    u = c + 1;
                }
                else {
                    v = c;
                }
            }
            if (arrI < arr[result[u]]) {
                if (u > 0) {
                    p[i] = result[u - 1];
                }
                result[u] = i;
            }
        }
    }
    u = k + 1;
    v = result[u - 1];
    var seq = new Int32Array(u);
    while (u-- > 0) {
        seq[u] = v;
        v = p[v];
        result[u] = 0;
    }
    return seq;
}

if (hasDocumentAvailable) {
    /*
    * Defining $EV and $V properties on Node.prototype
    * fixes v8 "wrong map" de-optimization
    */
    if (window.Node) {
        var prototype = Node.prototype;
        prototype.$EV = null; // event
        prototype.$V = null; // vNode
        // v-model
        prototype.$M = null; // model
        prototype.$TV = null; // trueValue
        prototype.$FV = null; // falseValue
        prototype.$VA = null; // value
        // Transtion Element
        prototype.$TC = undefined;
        prototype.$TD = undefined;
        prototype.$ID = undefined;
        prototype._enterCb = undefined;
        prototype._leaveCb = undefined;
        prototype._moveCb = undefined;
    }
}
function render(vNode, parentDom) {
    var mountedQueue = [];
    var lastVNode = parentDom.$V;
    if (isNullOrUndefined(lastVNode)) {
        if (!isNullOrUndefined(vNode)) {
            if (vNode.type & 8192 /* InUse */) {
                vNode = directClone(vNode);
            }
            mount(vNode, parentDom, null, false, null, mountedQueue);
            parentDom.$V = vNode;
        }
    }
    else {
        if (isNullOrUndefined(vNode)) {
            remove(lastVNode, parentDom, false);
            parentDom.$V = null;
        }
        else {
            if (vNode.type & 8192 /* InUse */) {
                vNode = directClone(vNode);
            }
            patch(lastVNode, vNode, parentDom, null, false, null, mountedQueue, false);
            parentDom.$V = vNode;
        }
    }
    callAll(mountedQueue);
}

function isText(o) {
    return (o.type & 1 /* Text */) > 0;
}
function isElement(o) {
    return (o.type & 962 /* Element */) > 0;
}
function isComponentClass(o) {
    return (o.type & 8 /* ComponentClass */) > 0;
}
function isComponentFunction(o) {
    return (o.type & 16 /* ComponentFunction */) > 0;
}
function isFragment(o) {
    return (o.type & 4096 /* Fragment */) > 0;
}
function isComment(o) {
    return (o.type & 32 /* HtmlComment */) > 0;
}
function hasTextChildren(o) {
    return o.childrenType === 16 /* HasTextChildren */;
}
function hasInvalidChildren(o) {
    return o.childrenType === 1 /* HasInvalidChildren */;
}
function hasVNodeChildren(o) {
    return o.childrenType === 2 /* HasVNodeChildren */;
}
function hasMultipleChildren(o) {
    return (o.childrenType & 12 /* MultipleChildren */) > 0;
}

var nextTick = typeof Promise !== 'undefined' ?
    function (callback) { return Promise.resolve().then(function () { return callback(); }); } :
    /* istanbul ignore next */
    function (callback) { return window.setTimeout(callback, 0); };
var microTaskPending = false;
var QUEUE_MAP = new Map();
function renderSyncComponnet(component, lastVNode, nextVNode, parentDom, anchor, mountedQueue) {
    component.$blockRender = true;
    component.trigger('$beforeMount', lastVNode, nextVNode);
    if (isFunction(component.beforeMount)) {
        component.beforeMount(lastVNode, nextVNode);
    }
    component.$blockRender = false;
    var vNode = normalizeRoot(component.$template(EMPTY_OBJ, component.get('$blocks')), nextVNode);
    // reuse the dom even if they are different
    var lastInput = null;
    if (!isNull(lastVNode) && (lastInput = lastVNode.children.$lastInput)) {
        // if (lastVNode !== nextVNode) {
        // we will set nextVNode to lastVNode on render async component, exclude this case
        // unmountComponentClass(lastVNode, nextVNode);
        // }
        patch(lastInput, vNode, parentDom, component, component.$SVG, anchor, mountedQueue, true);
    }
    else {
        mount(vNode, parentDom, component, component.$SVG, anchor, mountedQueue);
    }
    component.$lastInput = vNode;
    mountedQueue.push(function () {
        callAllQueue(component, '$pendingQueue');
        component.$mount(lastVNode, nextVNode);
    });
    component.$rendered = true;
}
function renderAsyncComponent(component, lastVNode, nextVNode, parentDom, anchor, mountedQueue) {
    if (isNull(lastVNode)) {
        // set nextVNode to lastVNode to let renderSyncComponent to patch
        lastVNode = nextVNode;
        var vNode = component.$lastInput = createCommentVNode('async');
        mount(vNode, parentDom, component, component.$SVG, anchor, mountedQueue);
    }
    component.on('$inited', function () {
        mountedQueue = component.$mountedQueue = [];
        renderSyncComponnet(component, lastVNode, nextVNode, parentDom, anchor, mountedQueue);
        // call queue before mountedQueue, because queue are callbacks set in init
        callAllQueue(component, '$pendingQueue');
        callAll(mountedQueue);
    });
}
function updateSyncComponent(component, lastVNode, nextVNode, parentDom, anchor, mountedQueue, force) {
    component.$blockRender = true;
    if (!force) {
        patchProps(component, lastVNode.props, nextVNode.props, component.constructor.defaults());
    }
    component.trigger('$beforeUpdate', lastVNode, nextVNode);
    if (isFunction(component.beforeUpdate)) {
        {
            /* istanbul ignore next */
            component.beforeUpdate(lastVNode, nextVNode);
        }
    }
    component.$blockRender = false;
    var vNode = normalizeRoot(component.$template(EMPTY_OBJ, component.get('$blocks')), nextVNode);
    patch(component.$lastInput, vNode, parentDom, component, component.$SVG, anchor, mountedQueue, false);
    component.$lastInput = vNode;
    mountedQueue.push(function () {
        component.trigger('$updated', lastVNode, nextVNode);
    });
    if (isFunction(component.updated)) {
        mountedQueue.push(function () {
            {
                /* istanbul ignore next */
                component.updated(lastVNode, nextVNode);
            }
        });
    }
}
function updateAsyncComponent(component, lastVNode, nextVNode, parentDom, anchor, mountedQueue, force) {
    component.on('$inited', function () {
        mountedQueue = component.$mountedQueue = [];
        updateSyncComponent(component, lastVNode, nextVNode, parentDom, anchor, mountedQueue, force);
        callAll(mountedQueue);
    });
}
function componentInited(component, triggerReceiveEvents) {
    triggerReceiveEvents && triggerReceiveEvents();
    component.$inited = true;
    component.trigger('$inited');
}
function mountProps(component, nextProps) {
    var props = component.$props;
    var changeTraces = [];
    for (var prop in nextProps) {
        var nextValue = nextProps[prop];
        if (isUndefined(nextValue))
            continue;
        patchProp(component, props, prop, props[prop], nextValue, changeTraces);
    }
    // a callback to trigger $receive events
    return function () { return triggerReceiveEvents(component, changeTraces, true); };
}
function patchProps(component, lastProps, nextProps, defaultProps) {
    lastProps || (lastProps = EMPTY_OBJ);
    nextProps || (nextProps = EMPTY_OBJ);
    if (lastProps !== nextProps) {
        var changeTraces = [];
        var props = component.$props;
        if (nextProps !== EMPTY_OBJ) {
            for (var prop in nextProps) {
                // const lastValue = rollbackToDefault(prop, lastProps[prop as keyof typeof lastProps], defaultProps);
                var lastValue = props[prop]; // use actual props instead of
                var nextValue = rollbackToDefault(prop, nextProps[prop], defaultProps);
                patchProp(component, props, prop, lastValue, nextValue, changeTraces);
            }
        }
        if (lastProps !== EMPTY_OBJ) {
            for (var prop in lastProps) {
                if (!isUndefined(lastProps[prop]) && !hasOwn.call(nextProps, prop)) {
                    patchProp(component, props, prop, props[prop], defaultProps[prop], changeTraces);
                }
            }
        }
        triggerReceiveEvents(component, changeTraces, false);
    }
}
function patchProp(component, props, prop, lastValue, nextValue, changeTraces) {
    if (lastValue === nextValue)
        return;
    props[prop] = nextValue;
    if (!isEventProp(prop)) {
        changeTraces.push({ path: prop, newValue: nextValue, oldValue: lastValue });
    }
}
function setProps(component, newProps) {
    var props = component.$props;
    var changeTracesGroup = [];
    for (var propName in newProps) {
        var lastValue = get(props, propName);
        var nextValue = newProps[propName];
        if (lastValue !== nextValue) {
            changeTracesGroup.push(set(props, propName, nextValue));
        }
    }
    var changesLength = changeTracesGroup.length;
    if (changesLength > 0) {
        // trigger $change events
        for (var i = 0; i < changesLength; i++) {
            var changeTraces = changeTracesGroup[i];
            // we should iterate from back to front to trigger event in order like a.b.c -> a.b -> a
            for (var j = changeTraces.length - 1; j > -1; j--) {
                var _a = changeTraces[j], path = _a.path, newValue = _a.newValue, oldValue = _a.oldValue;
                callModelEvent(component, path, newValue);
                component.trigger("$change:".concat(path), newValue, oldValue);
            }
        }
        component.forceUpdate(function () {
            // trigger $changed events
            for (var i = 0; i < changesLength; i++) {
                var changeTraces = changeTracesGroup[i];
                // we should iterate from back to front to trigger event in order like a.b.c -> a.b -> a
                for (var j = changeTraces.length - 1; j > -1; j--) {
                    var _a = changeTraces[j], path = _a.path, newValue = _a.newValue, oldValue = _a.oldValue;
                    component.trigger("$changed:".concat(path), newValue, oldValue);
                }
            }
        });
    }
}
function callModelEvent(component, path, newValue) {
    var modelEvent = component.$props["ev-$model:".concat(path)];
    if (modelEvent) {
        modelEvent(newValue);
    }
}
function forceUpdate(component, callback) {
    if (!component.$inited) {
        if (isFunction(callback)) {
            (component.$pendingQueue || (component.$pendingQueue = [])).push(callback);
        }
    }
    else if (component.$blockRender) {
        // component is before rendering or updating
        if (isFunction(callback)) {
            // append pendingQueue firstly to make the changed event order correct.
            // @see unit test: set prop multiple times 
            var pendingQueue = component.$pendingQueue;
            var mountedQueue = component.$mountedQueue;
            if (pendingQueue) {
                mountedQueue.push.apply(mountedQueue, pendingQueue);
                component.$pendingQueue = null;
            }
            mountedQueue.push(callback);
        }
    }
    else {
        if (!QUEUE_MAP.has(component)) {
            QUEUE_MAP.set(component, { component: component, next: null });
        }
        // if (QUEUE.indexOf(component) === -1) {
        // QUEUE.push(component);
        // }
        if (!microTaskPending) {
            microTaskPending = true;
            nextTick(rerender);
        }
        if (isFunction(callback)) {
            (component.$queue || (component.$queue = [])).push(callback);
        }
    }
}
function triggerReceiveEvents(component, changeTraces, init) {
    for (var i = 0; i < changeTraces.length; i++) {
        var _a = changeTraces[i], path = _a.path, newValue = _a.newValue, oldValue = _a.oldValue;
        component.trigger("$receive:".concat(path), newValue, oldValue, init);
    }
}
function rerender() {
    microTaskPending = false;
    var trees = getRerenderTrees();
    var tree = undefined;
    while (tree = trees.shift()) {
        var component = tree.component;
        if (!component.$unmounted) {
            var mountedQueue = component.$mountedQueue = [];
            var vNode = component.$vNode;
            component.$update(vNode, vNode, findDomFromVNode(component.$lastInput, true).parentNode, null, mountedQueue, true);
            callAll(mountedQueue);
            callQueueDepthFirst(tree);
        }
    }
}
/**
 * Remove the unnecessary components if we find that
 * their parents is already in the QUEUE_MAP. The parent
 * component will update its children, so we remove
 * all the children components.
 */
function getRerenderTrees() {
    var trees = [];
    QUEUE_MAP.forEach(function (tree, component) {
        var senior = component;
        while (senior = senior.$senior) {
            var seniorTree = QUEUE_MAP.get(senior);
            if (!isUndefined(seniorTree)) {
                seniorTree.next = tree;
                return;
            }
        }
        trees.push(tree);
    });
    QUEUE_MAP.clear();
    return trees;
}
function callQueueDepthFirst(tree) {
    var loop = function (tree) {
        if (!tree)
            return;
        loop(tree.next);
        callAllQueue(tree.component, '$queue');
    };
    loop(tree);
}
function callAllQueue(component, queueName) {
    var queue = component[queueName];
    if (queue) {
        for (var i = 0; i < queue.length; i++) {
            queue[i].call(component);
        }
        component[queueName] = null;
    }
}
function rollbackToDefault(prop, value, defaultProps) {
    return isUndefined(value) ? defaultProps[prop] : value;
}

var Event$1 = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function () {
    function Event() {
        this.$events = {};
        // internal properties
        this.$blockAddEvent = false;
    }
    Event.prototype.on = function (name, callback) {
        var events = this.$events;
        (events[name] || (events[name] = [])).push(callback);
    };
    Event.prototype.off = function (name, callback) {
        if (isUndefined(name)) {
            this.$events = {};
            return;
        }
        var callbacks = this.$events[name];
        if (isUndefined(callbacks))
            return;
        if (isUndefined(callback)) {
            this.$events[name] = undefined;
            return;
        }
        for (var i = 0; i < callbacks.length; i++) {
            if (callbacks[i] === callback) {
                callbacks.splice(i, 1);
                break;
            }
        }
    };
    // trigger<N = void, T extends (...args: any[]) => void = any>(name: NoInfer<N>, ...args: Parameters<T>): void;
    Event.prototype.trigger = function (name) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var callbacks = this.$events[name];
        if (!isUndefined(callbacks)) {
            callbacks = callbacks.slice();
            for (var i = 0; i < callbacks.length; i++) {
                callbacks[i].apply(void 0, args);
            }
        }
    };
    return Event;
}());

function setTextModel(component, event) {
    var target = event.target;
    component.set(target.$M, target.value);
}
function setRadioModel(component, event) {
    var target = event.target;
    component.set(target.$M, getValue(target));
}
function setCheckboxModel(component, event) {
    var target = event.target;
    var modelName = target.$M;
    var checked = target.checked;
    var trueValue = target.$TV;
    var falseValue = target.$FV;
    var value = component.get(modelName);
    if (isNullOrUndefined(trueValue)) {
        trueValue = true;
    }
    if (isNullOrUndefined(falseValue)) {
        falseValue = false;
    }
    if (isArray(value)) {
        value = value.slice(0);
        var index = value.indexOf(trueValue);
        if (checked) {
            if (index === -1) {
                value.push(trueValue);
            }
        }
        else {
            if (index > -1) {
                value.splice(index, 1);
            }
        }
    }
    else {
        value = checked ? trueValue : falseValue;
    }
    component.set(modelName, value);
}
function isChecked(value, trueValue) {
    if (isArray(value)) {
        return value.indexOf(trueValue) > -1;
    }
    else {
        return value === trueValue;
    }
}
function setSelectModel(component, event) {
    var target = event.target;
    var multiple = target.multiple;
    var options = target.options;
    var value;
    if (multiple) {
        value = [];
        for (var i = 0; i < options.length; i++) {
            var opt = options[i];
            if (opt.selected) {
                value.push(getValue(opt));
            }
        }
    }
    else {
        for (var i = 0; i < options.length; i++) {
            var opt = options[i];
            if (opt.selected) {
                value = getValue(opt);
                break;
            }
        }
    }
    component.set(target.$M, value);
}
function getValue(el) {
    var value = el.$VA;
    return isNullOrUndefined(value) ? el.value : value;
}
function extend(source, extra) {
    if (extra === EMPTY_OBJ)
        return source;
    for (var key in extra) {
        source[key] = extra[key];
    }
    return source;
}
function className(obj) {
    if (isNullOrUndefined(obj))
        return null;
    if (isStringOrNumber(obj))
        return obj;
    var ret = [];
    for (var key in obj) {
        if (obj[key]) {
            ret.push(key);
        }
    }
    return ret.join(' ');
}
function map(data, iter, thisArg) {
    if (isNullOrUndefined(data))
        return;
    if (isObject(data)) {
        var ret_1 = [];
        var callback = function (value, key) {
            var result = iter.call(thisArg, value, key);
            if (isArray(result)) {
                ret_1.push.apply(ret_1, result);
            }
            else {
                ret_1.push(result);
            }
        };
        if (data.forEach) {
            data.forEach(callback);
        }
        else if (isArray(data)) {
            /* istanbul ignore next */
            for (var i = 0; i < data.length; i++) {
                callback(data[i], i);
            }
        }
        else {
            for (var key in data) {
                callback(data[key], key);
            }
        }
        return ret_1;
    }
}
var getPrototypeOf = Object.getPrototypeOf;
function superCall(props, blocks, __proto__) {
    if (!__proto__) {
        __proto__ = getPrototypeOf(this);
        while (!hasOwn.call(__proto__.constructor, 'template')) {
            // if the component has not template, then get the parent's template
            __proto__ = getPrototypeOf(__proto__);
        }
        __proto__ = getPrototypeOf(__proto__);
    }
    while (!hasOwn.call(__proto__.constructor, 'template')) {
        // if the super component has no template, then get super's super's template
        __proto__ = getPrototypeOf(__proto__);
    }
    var superTemplate = __proto__.constructor.template;
    if (isString(superTemplate)) {
        superTemplate = compile$1(superTemplate);
    }
    return superTemplate.call(this, props, blocks, getPrototypeOf(__proto__));
}
var compile$1;
function registerCompile(_compile) {
    compile$1 = _compile;
}

var Vdt = /*#__PURE__*/Object.freeze({
    __proto__: null,
    EMPTY_OBJ: EMPTY_OBJ,
    noop: noop,
    VNode: VNode,
    getTypeForVNodeElement: getTypeForVNodeElement,
    createVNode: createVNode,
    createElementVNode: createElementVNode,
    createComponentVNode: createComponentVNode,
    createUnknownComponentVNode: createUnknownComponentVNode,
    createTextVNode: createTextVNode,
    createCommentVNode: createCommentVNode,
    createUnescapeTextVNode: createUnescapeTextVNode,
    normalizeRoot: normalizeRoot,
    normalizeChildren: normalizeChildren$1,
    createFragment: createFragment,
    createVoidVNode: createVoidVNode,
    directClone: directClone,
    linkEvent: linkEvent,
    render: render,
    patch: patch,
    mount: mount,
    remove: remove,
    unmount: unmount,
    unmountComponentClass: unmountComponentClass,
    findDomFromVNode: findDomFromVNode,
    callAll: callAll,
    removeVNodeDom: removeVNodeDom,
    insertOrAppend: insertOrAppend,
    hooks: hooks,
    createRef: createRef,
    Fragment: Fragment,
    validateProps: validateProps,
    isText: isText,
    isElement: isElement,
    isComponentClass: isComponentClass,
    isComponentFunction: isComponentFunction,
    isFragment: isFragment,
    isComment: isComment,
    hasTextChildren: hasTextChildren,
    hasInvalidChildren: hasInvalidChildren,
    hasVNodeChildren: hasVNodeChildren,
    hasMultipleChildren: hasMultipleChildren,
    setTextModel: setTextModel,
    setRadioModel: setRadioModel,
    setCheckboxModel: setCheckboxModel,
    isChecked: isChecked,
    setSelectModel: setSelectModel,
    extend: extend,
    className: className,
    map: map,
    superCall: superCall,
    registerCompile: registerCompile,
    get compile () { return compile$1; }
});

function watch(key, callback, options, instance) {
    if (instance === void 0) { instance = currentInstance; }
    if (!options || !options.presented) {
        // @ts-ignore
        instance.on("$change:".concat(key), callback);
        if (!options || !options.inited) {
            // @ts-ignore
            instance.on("$receive:".concat(key), callback);
        }
        else {
            // @ts-ignore
            instance.on("$receive:".concat(key), function (newValue, oldValue, init) {
                if (!init) {
                    callback(newValue, oldValue);
                }
            });
        }
    }
    else {
        // @ts-ignore
        instance.on("$changed:".concat(key), callback);
        if (!options.inited) {
            // @ts-ignore
            instance.on("$receive:".concat(key), function (newValue, oldValue, init) {
                var lifecycle = init ? '$mounted' : '$updated';
                var fn = function () {
                    instance.off(lifecycle, fn);
                    callback(newValue, oldValue);
                };
                instance.on(lifecycle, fn);
            });
        }
        else {
            // @ts-ignore
            instance.on("$receive:".concat(key), function (newValue, oldValue, init) {
                if (!init) {
                    var fn_1 = function () {
                        instance.off('$updated', fn_1);
                        callback(newValue, oldValue);
                    };
                    instance.on('$updated', fn_1);
                }
            });
        }
    }
}

var currentInstance = null;
function useInstance() {
    return currentInstance;
}
function setInstance(instance) {
    currentInstance = instance;
}
// type HasDefault<P, U = Partial<P>> = {
// [K in keyof P]: K extends keyof U ?
// Exclude<P[K], undefined> :
// P[K]
// };
var uid = 0;
var Component$1 = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function (_super) {
    __extends(Component, _super);
    function Component(props, $vNode, $SVG, $mountedQueue, $senior) {
        var _this = _super.call(this) || this;
        _this.refs = {};
        _this.$SVG = false;
        // internal properties
        _this.$lastInput = null;
        _this.$blockRender = false;
        _this.$queue = null;
        _this.$pendingQueue = null;
        _this.$provides = null;
        _this.$uid = uid++;
        // lifecyle states
        _this.$inited = false;
        _this.$rendered = false;
        _this.$mounted = false;
        _this.$unmounted = false;
        // should trigger recieve events on initializing or not
        _this.triggerReceiveEvents = null;
        currentInstance = _this;
        _this.$vNode = $vNode;
        _this.$SVG = $SVG;
        _this.$mountedQueue = $mountedQueue;
        _this.$senior = $senior;
        var constructor = _this.constructor;
        var template = constructor.template;
        if (isFunction(template)) {
            _this.$template = template;
        }
        else {
            _this.$template = compile$1(template);
        }
        _this.$props = constructor.defaults();
        if (!isNullOrUndefined($senior)) {
            _this.$provides = $senior.$provides;
        }
        if (!isNullOrUndefined(props)) {
            // should mount props in contructor, because we may get props on hooks
            // that initialize in constructor.
            _this.triggerReceiveEvents = mountProps(_this, props);
        }
        return _this;
        // if (isFunction(this.init)) {
        // const ret = this.init(props);
        // if (ret && ret.then) {
        // (ret as Promise<any>).then(() => componentInited(this, triggerReceiveEvents), err => {
        // if ("production" !== 'production') {
        // console.error('Unhandled promise rejection in init: ', err);
        // }
        // componentInited(this, triggerReceiveEvents);
        // });
        // } else {
        // componentInited(this, triggerReceiveEvents);
        // }
        // } else {
        // componentInited(this, triggerReceiveEvents);
        // }
    }
    Component.prototype.$init = function (props) {
        var _this = this;
        // if (isFunction(this.init)) {
        var triggerReceiveEvents = this.triggerReceiveEvents;
        if (isFunction(this.init)) {
            var ret = this.init(props);
            if (ret && ret.then) {
                ret.then(function () { return componentInited(_this, triggerReceiveEvents); }, function (err) {
                    componentInited(_this, triggerReceiveEvents);
                });
            }
            else {
                componentInited(this, triggerReceiveEvents);
            }
        }
        else {
            componentInited(this, triggerReceiveEvents);
        }
        // } else {
        // if it does not exist init method, it is unnecessary to trigger $receive events
        // @MODIFY: maybe init in constructor
        // if (!isNull(props)) {
        // const defaults = this.props;
        // for (const key in props) {
        // const value = props[key];
        // if (!isUndefined(value)) {
        // defaults[key] = value;
        // }
        // }
        // }
        // componentInited(this, null);
        // }
        currentInstance = null;
    };
    Component.prototype.$render = function (lastVNode, nextVNode, parentDom, anchor, mountedQueue) {
        if (this.$inited) {
            renderSyncComponnet(this, lastVNode, nextVNode, parentDom, anchor, mountedQueue);
        }
        else {
            renderAsyncComponent(this, lastVNode, nextVNode, parentDom, anchor, mountedQueue);
        }
    };
    Component.prototype.$mount = function (lastVNode, nextVNode) {
        this.$mounted = true;
        this.trigger('$mounted', lastVNode, nextVNode);
        if (isFunction(this.mounted)) {
            this.mounted(lastVNode, nextVNode);
        }
    };
    Component.prototype.$update = function (lastVNode, nextVNode, parentDom, anchor, mountedQueue, force) {
        if (this.$inited) {
            updateSyncComponent(this, lastVNode, nextVNode, parentDom, anchor, mountedQueue, force);
        }
        else {
            updateAsyncComponent(this, lastVNode, nextVNode, parentDom, anchor, mountedQueue, force);
        }
    };
    Component.prototype.$unmount = function (vNode, nextVNode) {
        this.trigger('$beforeUnmount', vNode, nextVNode);
        if (isFunction(this.beforeUnmount)) {
            this.beforeUnmount(vNode, nextVNode);
        }
        // if (isNull(nextVNode)) {
        // if nextVNode exists, we will unmount it on $render
        unmount(this.$lastInput, null);
        // }
        this.$unmounted = true;
        this.trigger('$unmounted', vNode, nextVNode);
        this.off();
        if (isFunction(this.unmounted)) {
            this.unmounted(vNode, nextVNode);
        }
    };
    Component.prototype.set = function (key, value, options) {
        var _a;
        if (isObject(key)) {
            options = value;
        }
        else {
            key = (_a = {}, _a[key] = value, _a);
        }
        if (!isUndefined(options) && options.silent) {
            for (var propName in key) {
                set(this.$props, propName, key[propName]);
            }
            return;
        }
        setProps(this, key);
    };
    Component.prototype.get = function (key) {
        if (isUndefined(key))
            return this.$props;
        return get(this.$props, key);
    };
    Component.prototype.forceUpdate = function (callback) {
        if (this.$unmounted)
            return;
        forceUpdate(this, callback);
    };
    // trigger<N = void, T extends (...args: any[]) => void = any>(name: NoInfer<N>, ...args: Parameters<T>): void;
    // trigger<K extends keyof E>(name: K, ...args: Parameters<EventCallback>): void;
    // trigger<K extends keyof E>(name: K, ...args: E[K] extends (...args: infer P) => any ? P : never) {
    Component.prototype.trigger = function (name) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        // call event on props firstly
        var propEvent = this.$props["ev-".concat(name)];
        if (isFunction(propEvent) && !this.$unmounted) {
            propEvent.apply(void 0, args);
        }
        _super.prototype.trigger.apply(this, __spreadArray([name], args));
    };
    Component.prototype.watch = function (key, callback, options) {
        watch(key, callback, options, this);
    };
    Component.defaults = function () { return ({}); };
    return Component;
}(Event$1));

var BaseTransition = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function (_super) {
    __extends(BaseTransition, _super);
    function BaseTransition() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        // private isLeaving: boolean = false;
        _this.leavingVNodesCache = {};
        _this.el = null;
        _this.originalDisplay = '';
        return _this;
    }
    BaseTransition.template = function () {
        var props = this.$props;
        var children = props.children;
        if (isNullOrUndefined(children))
            return null;
        // const {mode} = props;
        var vNode = (isArray(children) ? children[0] : children);
        // if (this.isLeaving) {
        // return null;
        // }
        var enterHooks = resolveTransitionHooks(vNode, props, this, false);
        setTransitionHooks(vNode, enterHooks);
        return vNode;
    };
    BaseTransition.defaults = function () {
        return { show: true };
    };
    BaseTransition.prototype.mounted = function () {
        var lastInput = this.$lastInput;
        // is a void vNode 
        if (lastInput.type & 1 /* Text */)
            return;
        var el = this.el = findDomFromVNode(lastInput, true);
        var display = el.style.display;
        this.originalDisplay = display === 'none' ? '' : display;
        if (!this.$props.show) {
            setDisplay(el, 'none');
        }
    };
    BaseTransition.prototype.updated = function (lastVNode, nextVNode) {
        var lastInput = this.$lastInput;
        var transition = lastInput.transition;
        // maybe the element has been removed
        if (!transition)
            return;
        var lastValue = (lastVNode.props || EMPTY_OBJ).show;
        var nextValue = (nextVNode.props || EMPTY_OBJ).show;
        var oldEl = this.el;
        // maybe the element has changed
        var el = this.el = findDomFromVNode(lastInput, true);
        var hasChagned = el !== oldEl;
        if (lastValue !== nextValue || hasChagned) {
            if (nextValue || isUndefined(nextValue)) {
                transition.beforeEnter(el);
                setDisplay(el, this.originalDisplay);
                transition.enter(el);
            }
            else {
                if (hasChagned) {
                    setDisplay(el, 'none');
                }
                else {
                    transition.leave(el, function () {
                        setDisplay(el, 'none');
                    });
                }
            }
        }
    };
    BaseTransition.prototype.unmounted = function () {
        var el = this.el;
        if (el) {
            el.$TD = undefined;
        }
    };
    return BaseTransition;
}(Component$1));
function resolveTransitionHooks(vNode, props, component, isTransitionGroup) {
    var appear = props.appear, 
    // mode,
    onBeforeEnter = props.onBeforeEnter, onEnter = props.onEnter, onAfterEnter = props.onAfterEnter, onEnterCancelled = props.onEnterCancelled, onBeforeLeave = props.onBeforeLeave, onLeave = props.onLeave, onAfterLeave = props.onAfterLeave, onLeaveCancelled = props.onLeaveCancelled, onBeforeAppear = props.onBeforeAppear, onAppear = props.onAppear, onAfterAppear = props.onAfterAppear, onAppearCancelled = props.onAppearCancelled;
    var key = vNode.key;
    var leavingVNodesCache = component.leavingVNodesCache;
    return {
        beforeEnter: function (el) {
            var hook = onBeforeEnter;
            if (!component.$mounted) {
                if (appear) {
                    hook = onBeforeAppear || onBeforeEnter;
                }
                else {
                    return;
                }
            }
            // for same element (show)
            if (el._leaveCb) {
                el._leaveCb(true);
            }
            // for toggled element with the same key (v-if)
            var leavingVNode = leavingVNodesCache[key];
            if (leavingVNode && vNode.tag === leavingVNode.tag) {
                var el_1 = findDomFromVNode(leavingVNode, true);
                var leaveCb = el_1._leaveCb;
                if (leaveCb) {
                    // Maybe we are patching Fragment and treat the leaving element
                    // as anchor. If we toggle the same key element, the element
                    // will be removed, then the anchor is missing. So we do removing
                    // after patching has ended.
                    component.$mountedQueue.push(leaveCb);
                    // leaveCb();
                }
            }
            hook && hook(el);
        },
        enter: function (el) {
            // maybe a mount enter is processing, and we enter it again by updating its `show` value 
            if (el._enterCb) {
                // el._enterCb(true);
                return;
            }
            var hook = onEnter;
            var afterHook = onAfterEnter;
            var cancelHook = onEnterCancelled;
            if (!component.$mounted) {
                if (!appear)
                    return;
                hook = onAppear || onEnter;
                afterHook = onAfterAppear || onAfterEnter;
                cancelHook = onAppearCancelled || onEnterCancelled;
            }
            var called = false;
            var done = el._enterCb = function (cancelled) {
                if (called)
                    return;
                called = true;
                if (cancelled) {
                    cancelHook && cancelHook(el);
                }
                else {
                    afterHook && afterHook(el);
                }
                el._enterCb = undefined;
            };
            if (hook) {
                hook(el, done);
                if (hook.length <= 1) {
                    done();
                }
            }
            else {
                done();
            }
        },
        leave: function (el, remove) {
            if (el._enterCb) {
                el._enterCb(true);
            }
            if (component.$unmounted) {
                return remove();
            }
            onBeforeLeave && onBeforeLeave(el);
            var called = false;
            var done = el._leaveCb = function (cancelled) {
                if (called)
                    return;
                called = true;
                remove();
                if (cancelled) {
                    onLeaveCancelled && onLeaveCancelled(el);
                }
                else {
                    onAfterLeave && onAfterLeave(el);
                }
                el._leaveCb = undefined;
                if (leavingVNodesCache[key] === vNode) {
                    delete leavingVNodesCache[key];
                }
            };
            leavingVNodesCache[key] = vNode;
            if (onLeave) {
                onLeave(el, done);
                if (onLeave.length <= 1) {
                    done();
                }
            }
            else {
                done();
            }
        },
        show: isTransitionGroup ? true : props.show
    };
}
function setTransitionHooks(vNode, hooks) {
    if (vNode.type & 8 /* ComponentClass */) {
        var component = vNode.children;
        if (!isNullOrUndefined(component) && component.$inited) {
            setTransitionHooks(component.$lastInput, hooks);
        }
        else {
            vNode.transition = hooks;
        }
    }
    else {
        vNode.transition = hooks;
    }
}
function setDisplay(dom, display) {
    dom.$TD = dom.style.display = display;
}

var transitionProp = 'transition';
var animationProp = 'animation';
var endEvents = [];
var nodeOps = /*#__PURE__*/ (function () {
    var addClass;
    var removeClass;
    var raf;
    var caf;
    if (hasDocumentAvailable) {
        raf = window.requestAnimationFrame ||
            window.webkitRequestAnimationFrame ||
            window.mozRequestAnimationFrame ||
            window.msRequestAnimationFrame ||
            window.setTimeout;
        caf = window.cancelAnimationFrame ||
            window.webkitCancelAnimationFrame ||
            window.mozCancelRequestAnimationFrame ||
            window.msAnimationFrame ||
            window.clearTimeout;
        var testEl = document.createElement('div');
        // set handle class function
        if (testEl.classList) {
            addClass = function (dom, className) { return dom.classList.add(className); };
            removeClass = function (dom, className) { return dom.classList.remove(className); };
        }
        else {
            var hasClass_1 = function (dom, className) { return " ".concat(dom.className, " ").indexOf(" ".concat(className, " ")) > -1; };
            addClass = function (dom, className) { return !hasClass_1(dom, className) && (dom.className += ' ' + className); };
            removeClass = function (dom, className) {
                if (hasClass_1(dom, className)) {
                    dom.className = dom.className
                        .replace(new RegExp("(^|\\s)".concat(className, "(?:\\s|$)"), 'g'), '$1')
                        .replace(/\s+/g, ' ') // multiple spaces to one
                        .replace(/^\s*|\s*$/g, ''); // trim the ends
                }
            };
        }
        var EVENT_NAME_MAP = {
            transitionend: {
                'transition': 'transitionend',
                'WebkitTransition': 'webkitTransitionEnd',
                'MozTransition': 'mozTransitionEnd',
                'OTransition': 'oTransitionEnd',
                'msTransition': 'MSTransitionEnd'
            },
            animationend: {
                'animation': 'animationend',
                'WebkitAnimation': 'webkitAnimationEnd',
                'MozAnimation': 'mozAnimationEnd',
                'OAnimation': 'oAnimationEnd',
                'msAnimation': 'MSAnimationEnd'
            }
        };
        var style = testEl.style;
        // On some platforms, in particular some releases of Android 4.x,
        // the un-prefixed "animation" and "transition" properties are defined on the
        // style object but the events that fire will still be prefixed, so we need
        // to check if the un-prefixed events are useable, and if not remove them
        // from the map
        if (!('AnimationEvent' in window)) {
            delete EVENT_NAME_MAP.animationend.animation;
        }
        if (!('TransitionEvent' in window)) {
            delete EVENT_NAME_MAP.transitionend.transition;
        }
        for (var baseEventName in EVENT_NAME_MAP) {
            var baseEvents = EVENT_NAME_MAP[baseEventName];
            for (var styleName in baseEvents) {
                if (styleName in style) {
                    endEvents.push(baseEvents[styleName]);
                    if (baseEventName === 'transitionend') {
                        transitionProp = styleName;
                    }
                    else {
                        animationProp = styleName;
                    }
                    break;
                }
            }
        }
    }
    return { raf: raf, caf: caf, addClass: addClass, removeClass: removeClass };
})();
var TransitionEvents = {
    on: function (dom, eventListener) {
        if (endEvents.length === 0) {
            // If CSS transitions are not supported, trigger an "end animation"
            // event immediately.
            window.setTimeout(eventListener, 0);
            return;
        }
        endEvents.forEach(function (endEvent) {
            dom.addEventListener(endEvent, eventListener, false);
        });
    },
    off: function (dom, eventListener) {
        if (endEvents.length === 0) {
            return;
        }
        endEvents.forEach(function (endEvent) {
            dom.removeEventListener(endEvent, eventListener, false);
        });
    },
    // one: function(node, eventListener) {
    // let listener = function() {
    // eventListener.apply(this, arguments);
    // TransitionEvents.off(node, listener);
    // };
    // TransitionEvents.on(node, listener);
    // }
};
function nextFrame(fn) {
    var innerFrame;
    var raf = nodeOps.raf, caf = nodeOps.caf;
    var frame = raf(function () {
        innerFrame = raf(fn);
    });
    return function () {
        caf(frame);
        caf(innerFrame);
    };
}
// @reference Vue
var endId = 0;
function whenTransitionEnds(dom, cb) {
    var id = dom.$ID = ++endId;
    var _a = getTransitionInfo(dom), timeout = _a.timeout, propCount = _a.propCount;
    var ended = 0;
    var onEnd = function (e) {
        if (e.target === dom && ++ended >= propCount) {
            end();
        }
    };
    var end = function () {
        TransitionEvents.off(dom, onEnd);
        clearTimeout(timer);
        if (id === dom.$ID) {
            cb();
        }
    };
    var timer = setTimeout(function () {
        if (ended < propCount) {
            end();
        }
    }, timeout + 1);
    TransitionEvents.on(dom, onEnd);
}
function getTransitionInfo(dom) {
    var styles = window.getComputedStyle(dom);
    var getStyleProperties = function (key) { return (styles[key] || '').split(', '); };
    var transitionDelays = getStyleProperties(transitionProp + 'Delay');
    var transitionDurations = getStyleProperties(transitionProp + 'Duration');
    var transitionTimeout = getTimeout(transitionDelays, transitionDurations);
    var animationDelays = getStyleProperties(animationProp + 'Delay');
    var animationDurations = getStyleProperties(animationProp + 'Duration');
    var animationTimeout = getTimeout(animationDelays, animationDurations);
    var type = transitionTimeout > animationTimeout ? 0 /* Transition */ : 1 /* Animation */;
    var timeout = Math.max(transitionTimeout, animationTimeout);
    var propCount = type === 0 /* Transition */ ? transitionDurations.length : animationDurations.length;
    return { type: type, timeout: timeout, propCount: propCount };
}
function getTimeout(delays, durations) {
    var l = delays.length;
    return Math.max.apply(null, durations.map(function (d, i) { return toMs(d) + toMs(delays[i % l]); }));
}
function toMs(s) {
    return s.slice(0, -1) * 1000;
}

var Transition = function (props) {
    return createComponentVNode(8 /* ComponentClass */, BaseTransition, resolveTransitionProps(props), props.key, props.ref);
};
var TransitionTypeDefs = {
    name: String,
    // type: String,
    css: Boolean,
    // duration: [String, Number, Object],
    enterFromClass: String,
    enterActiveClass: String,
    enterToClass: String,
    appearFromClass: String,
    appearActiveClass: String,
    appearToClass: String,
    leaveFromClass: String,
    leaveActiveClass: String,
    leaveToClass: String,
};
Transition.typeDefs = TransitionTypeDefs;
function resolveTransitionProps(props) {
    var _a = props.name, name = _a === void 0 ? 'transition' : _a, 
    // type,
    _b = props.css, 
    // type,
    css = _b === void 0 ? true : _b, 
    // duration,
    _c = props.enterFromClass, 
    // duration,
    enterFromClass = _c === void 0 ? "".concat(name, "-enter-from") : _c, _d = props.enterActiveClass, enterActiveClass = _d === void 0 ? "".concat(name, "-enter-active") : _d, _e = props.enterToClass, enterToClass = _e === void 0 ? "".concat(name, "-enter-to") : _e, _f = props.appearFromClass, appearFromClass = _f === void 0 ? enterFromClass : _f, _g = props.appearActiveClass, appearActiveClass = _g === void 0 ? enterActiveClass : _g, _h = props.appearToClass, appearToClass = _h === void 0 ? enterToClass : _h, _j = props.leaveFromClass, leaveFromClass = _j === void 0 ? "".concat(name, "-leave-from") : _j, _k = props.leaveActiveClass, leaveActiveClass = _k === void 0 ? "".concat(name, "-leave-active") : _k, _l = props.leaveToClass, leaveToClass = _l === void 0 ? "".concat(name, "-leave-to") : _l, baseProps = __rest(props, ["name", "css", "enterFromClass", "enterActiveClass", "enterToClass", "appearFromClass", "appearActiveClass", "appearToClass", "leaveFromClass", "leaveActiveClass", "leaveToClass"]);
    if (!css)
        return baseProps;
    // let enterDuration: number | null = null;
    // let leaveDuration: number | null = null;
    // if (isObject(duration)) {
    // enterDuration = duration.enter;
    // leaveDuration = duration.leave;
    // } else if (!isNullOrUndefined(duration)) {
    // enterDuration = leaveDuration = duration;
    // }
    var onBeforeEnter = baseProps.onBeforeEnter, onEnter = baseProps.onEnter, onEnterCancelled = baseProps.onEnterCancelled, onLeave = baseProps.onLeave, onLeaveCancelled = baseProps.onLeaveCancelled, _m = baseProps.onBeforeAppear, onBeforeAppear = _m === void 0 ? onBeforeEnter : _m, _o = baseProps.onAppear, onAppear = _o === void 0 ? onEnter : _o, _p = baseProps.onAppearCancelled, onAppearCancelled = _p === void 0 ? onEnterCancelled : _p;
    var cancelNextFrame = null;
    var cancelFrame = function () {
        if (cancelNextFrame) {
            cancelNextFrame();
            forceReflow();
            cancelNextFrame = null;
        }
    };
    var finishEnter = function (el, isAppear, done) {
        if (isAppear) {
            if (cancelNextFrame) {
                removeTransitionClass(el, appearFromClass);
            }
            else {
                removeTransitionClass(el, appearToClass);
            }
            removeTransitionClass(el, appearActiveClass);
        }
        else {
            if (cancelNextFrame) {
                removeTransitionClass(el, enterFromClass);
            }
            else {
                removeTransitionClass(el, enterToClass);
            }
            removeTransitionClass(el, enterActiveClass);
        }
        done && done();
    };
    var finishLeave = function (el, done) {
        if (cancelNextFrame) {
            removeTransitionClass(el, leaveFromClass);
        }
        else {
            removeTransitionClass(el, leaveToClass);
        }
        removeTransitionClass(el, leaveActiveClass);
        done && done();
    };
    var makeEnterHook = function (isAppear) {
        return function (el, done) {
            var hook = isAppear ? onAppear : onEnter;
            var resolve = function () { return finishEnter(el, isAppear, done); };
            hook && hook(el, resolve);
            cancelNextFrame = nextFrame(function () {
                if (isAppear) {
                    removeTransitionClass(el, appearFromClass);
                    addTransitionClass(el, appearToClass);
                }
                else {
                    removeTransitionClass(el, enterFromClass);
                    addTransitionClass(el, enterToClass);
                }
                if (isUndefined(hook) || hook.length <= 1) {
                    whenTransitionEnds(el, resolve);
                }
                cancelNextFrame = null;
            });
        };
    };
    return __assign(__assign({}, baseProps), { onBeforeEnter: function (el) {
            onBeforeEnter && onBeforeEnter(el);
            addTransitionClass(el, enterFromClass);
            addTransitionClass(el, enterActiveClass);
        }, onEnter: makeEnterHook(false), onEnterCancelled: function (el) {
            finishEnter(el, false);
            cancelFrame();
            onEnterCancelled && onEnterCancelled(el);
        }, onBeforeAppear: function (el) {
            onBeforeAppear && onBeforeAppear(el);
            addTransitionClass(el, appearFromClass);
            addTransitionClass(el, appearActiveClass);
        }, onAppear: makeEnterHook(true), onAppearCancelled: function (el) {
            finishEnter(el, true);
            cancelFrame();
            onAppearCancelled && onAppearCancelled(el);
        }, onLeave: function (el, done) {
            var resolve = function () { return finishLeave(el, done); };
            addTransitionClass(el, leaveFromClass);
            forceReflow();
            addTransitionClass(el, leaveActiveClass);
            cancelNextFrame = nextFrame(function () {
                removeTransitionClass(el, leaveFromClass);
                addTransitionClass(el, leaveToClass);
                if (isUndefined(onLeave) || onLeave.length <= 1) {
                    whenTransitionEnds(el, resolve);
                }
                cancelNextFrame = null;
            });
            onLeave && onLeave(el, resolve);
        }, onLeaveCancelled: function (el) {
            finishLeave(el);
            cancelFrame();
            onLeaveCancelled && onLeaveCancelled(el);
        } });
}
function addTransitionClass(el, className) {
    nodeOps.addClass(el, className);
    (el.$TC || (el.$TC = {}))[className] = true;
}
function removeTransitionClass(el, className) {
    nodeOps.removeClass(el, className);
    var transitionClassname = el.$TC;
    if (transitionClassname) {
        delete transitionClassname[className];
        if (Object.keys(transitionClassname).length === 0) {
            el.$TC = undefined;
        }
    }
}
function forceReflow() {
    return document.body.offsetHeight;
}

var TransitionGroup = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function (_super) {
    __extends(TransitionGroup, _super);
    function TransitionGroup() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.lastChildren = [];
        _this.children = [];
        return _this;
    }
    TransitionGroup.template = function () {
        var props = this.$props;
        var tag = props.tag, children = props.children;
        var vNode;
        if (isUndefined(tag)) {
            vNode = createFragment(children, 8 /* HasKeyedChildren */);
        }
        else {
            vNode = createElementVNode(2 /* CommonElement */, tag, children, 8 /* HasKeyedChildren */);
        }
        normalizeChildren$1(vNode, children);
        var lastChildren = this.lastChildren = this.children;
        var nextChildren = vNode.children;
        // convert to array if it has only one child
        var childrenType = vNode.childrenType;
        if (childrenType & 2 /* HasVNodeChildren */) {
            vNode.childrenType = 8 /* HasKeyedChildren */;
            nextChildren = vNode.children = [nextChildren];
        }
        else if (childrenType & 1 /* HasInvalidChildren */) {
            vNode.childrenType = 2 /* HasVNodeChildren */;
            vNode.children = createVoidVNode();
            nextChildren = [];
        }
        this.children = nextChildren;
        var cssTransitionProps = resolveTransitionProps(props);
        for (var i = 0; i < nextChildren.length; i++) {
            var child = nextChildren[i];
            setTransitionHooks(child, resolveTransitionHooks(child, cssTransitionProps, this, true));
        }
        for (var i = 0; i < lastChildren.length; i++) {
            var child = lastChildren[i];
            setTransitionHooks(child, resolveTransitionHooks(child, cssTransitionProps, this, true));
            var dom = findDomFromVNode(child, true);
            child.position = dom.getBoundingClientRect();
        }
        return vNode;
    };
    TransitionGroup.defaults = function () {
        return { move: true };
    };
    TransitionGroup.prototype.mounted = function () { };
    TransitionGroup.prototype.updated = function () {
        var lastChildren = this.lastChildren;
        if (!lastChildren.length) {
            return;
        }
        var props = this.$props;
        if (!props.move)
            return;
        var moveClass = props.moveClass || "".concat(props.name || 'transition', "-move");
        var lastChildrenLength = lastChildren.length;
        for (var i = 0; i < lastChildrenLength; i++) {
            callPendingCbs(lastChildren[i]);
        }
        for (var i = 0; i < lastChildrenLength; i++) {
            var vNode = lastChildren[i];
            vNode.newPosition = findDomFromVNode(vNode, true).getBoundingClientRect();
        }
        var movedChildren = [];
        for (var i = 0; i < lastChildrenLength; i++) {
            var vNode = lastChildren[i];
            if (applyTransition(vNode)) {
                movedChildren.push(vNode);
            }
        }
        forceReflow();
        var _loop_1 = function (i) {
            var vNode = movedChildren[i];
            var el = findDomFromVNode(vNode, true);
            var style = el.style;
            addTransitionClass(el, moveClass);
            style.transform = style.webkitTransform = style.transitionDuration = style.webkitTransitionDuration = '';
            var cb = el._moveCb = function (e) {
                if (e && e.target !== el)
                    return;
                if (!e || transformRegExp.test(e.propertyName)) {
                    TransitionEvents.off(el, cb);
                    el._moveCb = undefined;
                    removeTransitionClass(el, moveClass);
                }
            };
            // TransitionEvents.on(el, cb as EventListener);
            whenTransitionEnds(el, cb);
        };
        for (var i = 0; i < movedChildren.length; i++) {
            _loop_1(i);
        }
    };
    return TransitionGroup;
}(BaseTransition));
var transformRegExp = /transform$/;
function callPendingCbs(vNode) {
    var el = findDomFromVNode(vNode, true);
    if (el._moveCb) {
        el._moveCb();
    }
    // if we call enterCb, it will call onAfterEnter, maybe we set
    // style in the next frame, but the transition has been ended.
    // e.g.
    // onEnter(el) => {
    //     el.style.height = 0; 
    //     nextFrame(() => el.style.heigth = '100px');
    // },
    // onAfterEnter(el) {
    //     el.style.height = '';
    // }
    // if (el._enterCb) {
    // el._enterCb();
    // }
}
function applyTransition(vNode) {
    var position = vNode.position, newPosition = vNode.newPosition;
    var dx = position.left - newPosition.left;
    var dy = position.top - newPosition.top;
    if (dx || dy) {
        var s = findDomFromVNode(vNode, true).style;
        s.transform = s.webkitTransform = "translate(".concat(dx, "px, ").concat(dy, "px)");
        s.transitionDuration = s.webkitTransitionDuration = '0s';
        return vNode;
    }
}

// @reference from vue3.0
function provide(key, value) {
    var provides = currentInstance.$provides;
    if (provides === null) {
        provides = currentInstance.$provides = Object.create(null);
    }
    else {
        var parent_1 = currentInstance.$senior;
        var parentProvides = parent_1 && parent_1.$provides;
        if (provides === parentProvides) {
            provides = currentInstance.$provides = Object.create(parentProvides);
        }
    }
    provides[key] = value;
}
function inject(key, defaultValue) {
    var parent = currentInstance.$senior;
    var provides = parent && parent.$provides;
    if (provides && key in provides) {
        return provides[key];
    }
    else if (!isUndefined(defaultValue)) {
        return defaultValue;
    }
    else ;
}

function makeLifecycle(name) {
    return function (hook) {
        currentInstance.on(name, hook);
    };
}
var onInited = makeLifecycle('$inited');
var onBeforeMount = makeLifecycle('$beforeMount');
var onMounted = makeLifecycle('$mounted');
var onBeforeUpdate = makeLifecycle('$beforeUpdate');
var onUpdated = makeLifecycle('$updated');
var onBeforeUnmount = makeLifecycle('$beforeUnmount');
var onUnmounted = makeLifecycle('$unmounted');

function trimRight(str) {
    var index = str.length;
    while (index-- && isWhiteSpace(str.charCodeAt(index))) { }
    return str.slice(0, index + 1);
}
function isWhiteSpace(charCode) {
    return ((charCode <= 160 && (charCode >= 9 && charCode <= 13) ||
        charCode == 32 ||
        charCode == 160) ||
        charCode == 5760 ||
        charCode == 6158 ||
        (charCode >= 8192 &&
            (charCode <= 8202 ||
                charCode == 8232 ||
                charCode == 8233 ||
                charCode == 8239 ||
                charCode == 8287 ||
                charCode == 12288 ||
                charCode == 65279)));
}
var selfClosingTags = {
    'area': true,
    'base': true,
    'br': true,
    'col': true,
    'command': true,
    'embed': true,
    'hr': true,
    'img': true,
    'input': true,
    'keygen': true,
    'link': true,
    'menuitem': true,
    'meta': true,
    'param': true,
    'source': true,
    'track': true,
    'wbr': true
};
var textTags = {
    style: true,
    script: true,
    textarea: true
};
var directivesMap = {
    /* don't use Directives, because of tree-shaking and performance */
    'v-if': true,
    'v-else-if': true,
    'v-else': true,
    'v-for': true,
    'v-for-value': true,
    'v-for-key': true,
    'v-raw': true,
    // [Directives.If]: true,
    // [Directives.ElseIf]: true,
    // [Directives.Else]: true,
    // [Directives.For]: true,
    // [Directives.ForValue]: true,
    // [Directives.ForKey]: true,
    // // [Directives.Model]: true,
    // [Directives.Raw]: true,
};
function isJSIdentifierPart(ch) {
    return (ch === 95) || ch === 36 || // _ (underscore) $
        (ch >= 65 && ch <= 90) || // A..Z
        (ch >= 97 && ch <= 122) || // a..z
        (ch >= 48 && ch <= 57); // 0..9
}
function isJSXIdentifierPart(ch) {
    return (ch === 58) || (ch === 45) || ch === 46 || isJSIdentifierPart(ch); // : - .
}
function isWhiteSpaceExceptLinebreak(charCode) {
    return charCode !== 10 && // \n
        charCode !== 13 && // \r
        isWhiteSpace(charCode);
}
function isElementNode(node) {
    var type = node.type;
    return type === 3 /* JSXCommonElement */ ||
        type === 4 /* JSXComponent */ ||
        type === 6 /* JSXVdt */ ||
        type === 7 /* JSXBlock */;
}
function throwError(msg, loc, source) {
    var lines = source.split('\n');
    var line = loc.line, column = loc.column;
    var error = new Error("".concat(msg, " (").concat(line, ":").concat(column, ")\n") +
        "> ".concat(line, " | ").concat(lines[line - 1], "\n") +
        "  ".concat(new Array(String(line).length + 1).join(' '), " | ").concat(new Array(column).join(' '), "^"));
    throw error;
}
var attrMaps = {
    'class': 'className',
    'for': 'htmlFor'
};
var getAttrName = function (name) {
    return attrMaps[name] || name;
};
var defaultOptions = {
    delimiters: ['{', '}'],
};
function computeChildrenFlagForVIf(lastFlag, nextFlag) {
    if (lastFlag === 0 /* UnknownChildren */)
        return lastFlag;
    if (lastFlag === 2 /* HasKeyedVNodeChildren */ &&
        nextFlag === 4 /* HasNonKeyedVNodeChildren */) {
        return nextFlag;
    }
    return lastFlag & nextFlag;
}
function computeChildrenFlagForChildren(lastFlag, nextFlag) {
    if (isNull(lastFlag))
        return nextFlag;
    // two array children is unknown children
    if (lastFlag === nextFlag && (lastFlag & 24 /* MultipleChildren */)) {
        return 0 /* UnknownChildren */;
    }
    return lastFlag & nextFlag;
}
function childrenFlagToChildrenType(flag) {
    {
        /* istanbul ignore next */
        switch (flag) {
            case 0 /* UnknownChildren */:
                return 0 /* UnknownChildren */;
            case 1 /* HasInvalidChildren */:
                return 1 /* HasInvalidChildren */;
            case 2 /* HasKeyedVNodeChildren */:
            case 4 /* HasNonKeyedVNodeChildren */:
                return 2 /* HasVNodeChildren */;
            case 8 /* HasKeyedChildren */:
                return 8 /* HasKeyedChildren */;
            case 16 /* HasNonKeyedChildren */:
                return 4 /* HasNonKeyedChildren */;
            case 32 /* HasTextChildren */:
                return 16 /* HasTextChildren */;
            default:
                throwError$1('Unknown flag: ' + flag);
        }
    }
}
var helpersMap = {
    '_$cv': 'createVNode',
    '_$ce': 'createElementVNode',
    '_$cc': 'createUnknownComponentVNode',
    '_$ct': 'createTextVNode',
    '_$ccv': 'createCommentVNode',
    '_$cu': 'createUnescapeTextVNode',
    '_$le': 'linkEvent',
    '_$ma': 'map',
    '_$ex': 'extend',
    '_$stm': 'setTextModel',
    '_$srm': 'setRadioModel',
    '_$scm': 'setCheckboxModel',
    '_$ssm': 'setSelectModel',
    '_$isc': 'isChecked',
    '_$cn': 'className',
    '_$su': 'superCall',
    '_$no': 'noop',
    '_$em': 'EMPTY_OBJ',
    // '_$tr': 'Transtion',
    // '_$tg': 'TransitionGroup',
};

function validateDirectiveValue(name, valueType, tag, tagType, source, loc) {
    if (name === "v-raw" /* Raw */) {
        if (tagType !== 3 /* JSXCommonElement */) {
            throwError("Only html elememt supports v-raw, but got: ".concat(tag), { line: loc.line, column: loc.column - "v-raw" /* Raw */.length }, source);
        }
    }
    else if (name === "v-for-key" /* ForKey */ || name === "v-for-value" /* ForValue */) {
        if (valueType !== 12 /* JSXString */) {
            throwError("'".concat(name, "' must be a literal string."), loc, source);
        }
    }
    else if (name === "v-for" /* For */ || name === "v-if" /* If */ || name === "v-else-if" /* ElseIf */) {
        if (valueType !== 8 /* JSXExpression */) {
            throwError("'".concat(name, "' must be a expression."), loc, source);
        }
    }
    else if (name === "v-else" /* Else */) {
        if (valueType !== 14 /* JSXNone */) {
            throwError("'".concat(name, "' should not have value."), loc, source);
        }
    }
}
function validateAttributeForBlock(tag, name, value, loc, source) {
    switch (name) {
        case "v-if" /* If */:
        case "v-else-if" /* ElseIf */:
        case "v-else" /* Else */:
        case "v-for" /* For */:
        case "v-for-key" /* ForKey */:
        case "v-for-value" /* ForValue */:
            break;
        case 'args':
            if (value.type !== 12 /* JSXString */) {
                throwError("The 'args' attribute of block element '<b:".concat(tag, ">' must be a literal string."), loc, source);
            }
            break;
        case 'params':
            if (value.type !== 8 /* JSXExpression */) {
                throwError("The 'params' attribute of block element '<b:".concat(tag, ">' must be a expression that value is an array."), loc, source);
            }
            break;
        default:
            throwError("Block element '<b:".concat(tag, ">' does not support the attribute '").concat(name, "'."), loc, source);
    }
}

var tagNameRegexp = /^<\w+:?\s*[\{\w\/>]/;
var Parser = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function () {
    function Parser(source, options) {
        this.index = 0;
        this.line = 1;
        this.column = 1;
        this.options = defaultOptions;
        this.source = trimRight(source);
        this.length = this.source.length;
        if (options) {
            this.options = __assign(__assign({}, defaultOptions), options);
        }
        this.ast = this.parse(true, 0);
    }
    Parser.prototype.parse = function (isRoot, spaces) {
        var nodes = [];
        var braces = { count: 0 };
        while (this.index < this.length && braces.count >= 0) {
            nodes.push(this.advance(braces, isRoot, spaces));
        }
        return nodes;
    };
    Parser.prototype.advance = function (braces, isRoot, spaces) {
        var ch = this.char();
        if (isRoot && this.isJSImport()) {
            return this.scanJSImport();
        }
        if (ch !== '<') {
            return this.scanJS(braces, isRoot, spaces);
        }
        return this.scanJSX();
    };
    Parser.prototype.scanJSImport = function () {
        var start = this.index;
        var loc = this.getLocation();
        this.updateIndex(7); // 'import '.length
        while (this.index < this.length) {
            var ch = this.char();
            if (ch === '\'' || ch === '"') {
                this.scanString();
                var start_1 = void 0;
                do {
                    start_1 = this.index;
                    this.skipWhitespaceAndJSComment();
                    if (this.char() === ';') {
                        this.updateIndex();
                    }
                } while (start_1 !== this.index);
                break;
            }
            else {
                this.updateIndex();
            }
        }
        return { type: 1 /* JSHoist */, value: this.getValue(start), loc: loc };
    };
    Parser.prototype.scanJS = function (braces, isRoot, spaces) {
        var delimiters = this.options.delimiters;
        var loc = this.getLocation();
        var value = [];
        var start = this.index;
        var spacesRemain = spaces;
        var newLine = true;
        var leadSpaces = 0;
        var shouldTrimRight = false;
        while (this.index < this.length) {
            this.skipJSComment();
            var ch = this.char();
            var tmp = void 0;
            if (ch === '\'' || ch === '"' || ch === '`' ||
                // is a RegExp, treat it as literal sting
                ch === '/' &&
                    // is not /* and //, this is comment
                    (tmp = this.char(this.index + 1)) && tmp !== '*' && tmp !== '/' && (
                // is the first char
                this.index === 0 ||
                    // is not </, this is a end tag
                    (tmp = this.char(this.index - 1)) && tmp !== '<' &&
                        // is not a division sign
                        // FIXME: expect `if (a > 1) /test/`
                        (tmp = this.getLastCharCode()) && !isJSIdentifierPart(tmp) && tmp !== 41 // )
                )) {
                // skip tag(<div>) in quotes
                this.scanString();
                newLine = false;
                continue;
            }
            else if (this.isTagStart()) {
                break;
            }
            else if (isRoot && this.isJSImport()) {
                break;
            }
            else if (ch === '\n') {
                value.push(this.getValue(start));
                start = this.index + 1;
                spacesRemain = spaces;
                newLine = true;
                leadSpaces = 0;
                this.updateLine();
            }
            else if (newLine && ch === ' ') {
                if (spacesRemain) {
                    spacesRemain--;
                    start++;
                }
                else {
                    leadSpaces++;
                }
            }
            else {
                if (ch === '{') {
                    braces.count++;
                }
                else if (braces.count > 0 && ch === '}') {
                    braces.count--;
                }
                else if (this.isExpect(delimiters[1])) {
                    // for parse break
                    braces.count--;
                    shouldTrimRight = true;
                    break;
                }
                newLine = false;
            }
            this.updateIndex();
        }
        value.push(this.getValue(start));
        if (shouldTrimRight) {
            this.trimRightForValue(value);
        }
        return { type: 0 /* JS */, value: value, spaces: leadSpaces, loc: loc };
    };
    Parser.prototype.scanJSX = function () {
        this.expect('<');
        if (this.isExpect('!--')) {
            // is html comment
            return this.parseJSXComment();
        }
        return this.parseJSXElement();
    };
    Parser.prototype.scanString = function () {
        var loc = this.getLocation();
        var str = '';
        var quote = this.char();
        this.updateIndex();
        this.index;
        while (this.index < this.length) {
            var ch = this.char();
            if (ch.charCodeAt(0) === 10) {
                this.updateLine();
            }
            this.updateIndex();
            if (ch === quote) {
                quote = '';
                break;
            }
            else if (ch === '\\') {
                str += this.char(this.updateIndex());
            }
            else {
                str += ch;
            }
        }
        if (quote !== '') {
            this.error('Unclosed quote', loc);
        }
        return { type: 12 /* JSXString */, value: str, loc: loc };
    };
    Parser.prototype.parseJSXElement = function () {
        var flag = this.charCode();
        var loc = this.getLocation();
        var start = this.index;
        var type;
        if (flag >= 65 && flag <= 90 /* upper case */) {
            // is a component 
            type = 4 /* JSXComponent */;
        }
        else if (this.charCode(this.index + 1) === 58 /* : */) {
            // is a directive
            start += 2;
            switch (flag) {
                case 116: // t
                    type = 6 /* JSXVdt */;
                    break;
                case 98: // b
                    type = 7 /* JSXBlock */;
                    break;
                /* istanbul ignore next */
                default:
                    this.error('Unknown directive ' + String.fromCharCode(flag) + ':');
            }
            this.updateIndex(2);
        }
        else {
            // is an element
            type = 3 /* JSXCommonElement */;
        }
        while (this.index < this.length) {
            if (!isJSXIdentifierPart(this.charCode())) {
                break;
            }
            this.updateIndex();
        }
        var value = this.getValue(start);
        var _a = this.parseJSXAttribute(value, type), attributes = _a.attributes, directives = _a.directives, keyed = _a.keyed, hasVRaw = _a.hasVRaw;
        var children = this.parseJSXChildren(value, type, attributes, hasVRaw, loc);
        return { type: type, value: value, attributes: attributes, directives: directives, children: children, keyed: keyed, loc: loc };
    };
    Parser.prototype.parseJSXComment = function () {
        this.expect('!--');
        var start = this.index;
        var loc = this.getLocation();
        while (this.index < this.length) {
            if (this.isExpect('-->')) {
                break;
            }
            else if (this.charCode() === 10) {
                this.updateLine();
            }
            this.updateIndex();
        }
        var value = this.getValue(start);
        this.expect('-->');
        return { type: 5 /* JSXComment */, value: value, loc: loc };
    };
    Parser.prototype.parseJSXAttribute = function (tag, type) {
        var attributes = [];
        var directives = {};
        var keyed = false;
        var hasVRaw = false;
        var value;
        while (this.index < this.length) {
            this.skipWhitespace();
            if (this.char() === '/' || this.char() === '>') {
                break;
            }
            var delimiters = this.options.delimiters;
            if (this.isExpect(delimiters[0])) {
                // support dynamic attributes
                var expression = this.parseJSXExpression();
                if (expression.value.length) {
                    // ignore empty expression
                    attributes.push(expression);
                }
                continue;
            }
            var loc = this.getLocation();
            var name_1 = this.parseJSXAttributeName();
            if (!keyed && name_1 === 'key') {
                keyed = true;
            }
            if (this.char() === '=') {
                this.updateIndex();
                value = this.parseJSXAttributeValue();
            }
            else {
                // treat no-value attribute as true
                value = {
                    type: 14 /* JSXNone */,
                    loc: this.getLocation(),
                };
            }
            {
                if (directivesMap[name_1]) {
                    validateDirectiveValue(name_1, value.type, tag, type, this.source, value.loc);
                }
                if (type === 7 /* JSXBlock */) {
                    validateAttributeForBlock(tag, name_1, value, value.loc, this.source);
                }
            }
            if (name_1 === "v-if" /* If */ || name_1 === "v-else-if" /* ElseIf */ || name_1 === "v-else" /* Else */) {
                directives[name_1] = { type: 9 /* JSXDirectiveIf */, name: name_1, value: value, next: null, loc: loc };
            }
            else {
                var attr = { type: 11 /* JSXAttribute */, name: name_1, value: value, loc: loc };
                if (directivesMap[name_1]) {
                    if (name_1 === "v-raw" /* Raw */) {
                        hasVRaw = true;
                    }
                    directives[name_1] = attr;
                }
                else {
                    attributes.push(attr);
                }
            }
        }
        return { attributes: attributes, directives: directives, keyed: keyed, hasVRaw: hasVRaw };
    };
    Parser.prototype.parseJSXAttributeName = function () {
        var start = this.index;
        while (this.index < this.length) {
            var ch = this.charCode();
            if (!isJSXIdentifierPart(ch)) {
                break;
            }
            this.updateIndex();
        }
        return this.getValue(start);
    };
    Parser.prototype.parseJSXAttributeValue = function () {
        var delimiters = this.options.delimiters;
        var value;
        if (this.isExpect(delimiters[0])) {
            value = this.parseJSXExpression();
        }
        else {
            var quote = this.char();
            if (quote !== '\'' && quote !== '"' && quote !== '`') {
                this.error('String value of attribute must start with a quote.');
            }
            value = this.scanString();
        }
        return value;
    };
    Parser.prototype.parseJSXChildren = function (tag, type, attributes, hasVRaw, loc) {
        var children = [];
        if (type === 3 /* JSXCommonElement */ && selfClosingTags[tag]) {
            // self closing tag
            if (this.char() === '/') {
                this.updateIndex();
            }
            this.expect('>');
        }
        else if (this.char() === '/') {
            // unknown self closing tag
            this.updateIndex();
            this.expect('>');
        }
        else {
            this.expect('>');
            if (textTags[tag]) {
                // if it is a text element, treat children as innerHTML attribute
                var attrLoc = this.getLocation();
                var children_1 = this.parseJSXChildrenValue(tag, type, hasVRaw, true, loc);
                if (children_1.length) {
                    attributes.push({
                        type: 11 /* JSXAttribute */,
                        name: tag === 'textarea' ? 'value' : 'innerHTML',
                        value: {
                            type: 13 /* JSXStrings */,
                            value: children_1,
                            loc: attrLoc,
                        },
                        loc: loc,
                    });
                }
            }
            else {
                children = this.parseJSXChildrenValue(tag, type, hasVRaw, false, loc);
            }
        }
        return children;
    };
    Parser.prototype.parseJSXChildrenValue = function (tag, type, hasVRaw, isTextTag, loc) {
        var children = [];
        var endTag = tag + '>';
        switch (type) {
            case 7 /* JSXBlock */:
                endTag = '</b:' + endTag;
                break;
            case 6 /* JSXVdt */:
                endTag = '</t:' + endTag;
                break;
            default:
                endTag = '</' + endTag;
                break;
        }
        if (hasVRaw) {
            while (this.index < this.length) {
                if (this.isExpect(endTag)) {
                    break;
                }
                children.push(this.scanJSXText([endTag]));
            }
        }
        else {
            this.skipWhitespaceBetweenTags(endTag);
            var directiveIf = null;
            while (this.index < this.length) {
                if (this.isExpect(endTag)) {
                    break;
                }
                var child = this.parseJSXChild(endTag, isTextTag);
                if (isElementNode(child)) {
                    var directives = child.directives;
                    var tmp = void 0;
                    var isElse = false;
                    if (tmp = directives["v-if" /* If */]) {
                        directiveIf = tmp;
                    }
                    else if ((tmp = directives["v-else-if" /* ElseIf */]) ||
                        (tmp = directives["v-else" /* Else */]) && (isElse = true)) {
                        directiveIf.next = child;
                        directiveIf = isElse ? null : tmp;
                        continue;
                    }
                }
                children.push(child);
            }
        }
        this.parseJSXClosingTag(endTag, loc);
        return children;
    };
    Parser.prototype.parseJSXChild = function (endTag, isTextTag) {
        var _this = this;
        var delimiters = this.options.delimiters;
        var child;
        if (this.isExpect(delimiters[0])) {
            child = this.parseJSXExpression();
            this.skipWhitespaceBetweenTags(endTag, false);
        }
        else if (isTextTag) {
            child = this.scanJSXText([endTag, delimiters[0]]);
        }
        else if (this.isTagStart()) {
            child = this.scanJSX();
            this.skipWhitespaceBetweenTags(endTag);
        }
        else {
            child = this.scanJSXText([function () {
                    return _this.isExpect(endTag) || _this.isTagStart();
                }, delimiters[0]]);
        }
        return child;
    };
    Parser.prototype.parseJSXClosingTag = function (endTag, loc) {
        this.expect('</', "Unclosed tag: ".concat(endTag), loc);
        while (this.index < this.length) {
            if (!isJSXIdentifierPart(this.charCode())) {
                break;
            }
            this.updateIndex();
        }
        this.skipWhitespace();
        this.expect('>');
    };
    Parser.prototype.scanJSXText = function (stopChars) {
        var start = this.index;
        var l = stopChars.length;
        var loc = this.getLocation();
        var i;
        var charCode;
        loop: while (this.index < this.length) {
            charCode = this.charCode();
            if (isWhiteSpace(charCode)) {
                if (charCode === 10) {
                    this.updateLine();
                }
            }
            else {
                for (i = 0; i < l; i++) {
                    var stopChar = stopChars[i];
                    if (typeof stopChar === 'function' && stopChar() ||
                        this.isExpect(stopChar)) {
                        break loop;
                    }
                }
            }
            this.updateIndex();
        }
        return { type: 2 /* JSXText */, value: this.getValue(start), loc: loc };
    };
    Parser.prototype.parseJSXExpression = function () {
        var delimiters = this.options.delimiters;
        var loc = this.getLocation();
        this.expect(delimiters[0]);
        this.skipWhitespaceAndJSComment();
        var spaceColumn = this.getFirstSpaceColumn();
        var type = 8 /* JSXExpression */;
        var value;
        if (this.isExpect('=')) {
            // if the lead char is '=', then treat it as unescape string
            this.expect('=');
            this.skipWhitespace();
            type = 10 /* JSXUnescapeText */;
        }
        if (this.isExpect(delimiters[1])) {
            value = [];
        }
        else {
            value = this.parse(false, spaceColumn);
        }
        this.expect(delimiters[1], "Unclosed delimiter", loc);
        return { type: type, value: value, loc: loc };
    };
    Parser.prototype.getLocation = function () {
        return { line: this.line, column: this.column };
    };
    Parser.prototype.getValue = function (start, index) {
        if (index === void 0) { index = this.index; }
        return this.source.slice(start, index);
    };
    Parser.prototype.char = function (index) {
        if (index === void 0) { index = this.index; }
        return this.source.charAt(index);
    };
    Parser.prototype.charCode = function (index) {
        if (index === void 0) { index = this.index; }
        return this.source.charCodeAt(index);
    };
    Parser.prototype.isJSImport = function () {
        return this.isExpect('import ');
    };
    Parser.prototype.isExpect = function (expect, index) {
        if (index === void 0) { index = this.index; }
        return this.source.slice(index, index + expect.length) === expect;
    };
    Parser.prototype.updateIndex = function (index) {
        if (index === void 0) { index = 1; }
        var oldIndex = this.index;
        this.index = oldIndex + index;
        this.column += index;
        return oldIndex;
    };
    Parser.prototype.updateLine = function () {
        this.line++;
        this.column = 0;
    };
    Parser.prototype.error = function (msg, loc) {
        throwError(msg, loc || { line: this.line, column: this.column }, this.source);
    };
    Parser.prototype.skipWhitespaceAndJSComment = function () {
        var start;
        do {
            start = this.index;
            this.skipWhitespace();
            this.skipJSComment();
        } while (start !== this.index);
    };
    Parser.prototype.skipJSComment = function () {
        var start;
        do {
            start = this.index;
            if (this.char() === '/') {
                var ch = this.char(this.index + 1);
                if (ch === '/') {
                    this.updateIndex(2);
                    while (this.index < this.length) {
                        var code = this.charCode();
                        if (code === 10) {
                            // is \n
                            break;
                        }
                        this.updateIndex();
                    }
                }
                else if (ch === '*') {
                    this.updateIndex(2);
                    while (this.index < this.length) {
                        if (this.isExpect('*/')) {
                            this.updateIndex(2);
                            break;
                        }
                        else if (this.charCode() === 10) {
                            this.updateLine();
                        }
                        this.updateIndex();
                    }
                }
            }
        } while (start !== this.index);
    };
    Parser.prototype.getLastCharCode = function () {
        var start = this.index - 1;
        var _start;
        do {
            _start = start;
            while (start >= 0) {
                var code = this.charCode(start);
                if (!isWhiteSpaceExceptLinebreak(code)) {
                    break;
                }
                start--;
            }
            // only check multi-line comments '/* comment */'
            while (start >= 0) {
                if (this.char(start) === '/' && this.char(start - 1) === '*') {
                    start -= 2;
                    while (start >= 0) {
                        if (this.char(start) === '*' && this.char(start - 1) === '/') {
                            start -= 2;
                            break;
                        }
                        start--;
                    }
                }
                break;
            }
        } while (start !== _start);
        return this.charCode(start);
    };
    Parser.prototype.getFirstSpaceColumn = function () {
        var start = this.index - 1;
        var hasSet = false;
        var _start;
        do {
            var ch = this.char(start);
            if (ch === '\n' || start === 0) {
                if (!hasSet)
                    _start = start;
                break;
            }
            if (ch === ' ') {
                if (!hasSet) {
                    _start = start;
                    hasSet = true;
                }
            }
            else {
                hasSet = false;
            }
            start--;
        } while (true);
        return this.column - (this.index - _start);
    };
    Parser.prototype.isTagStart = function (index) {
        if (index === void 0) { index = this.index; }
        return this.char(index) === '<' &&
            (this.isExpect('<!--', index) ||
                tagNameRegexp.test(this.source.slice(index)));
    };
    Parser.prototype.expect = function (str, msg, loc) {
        if (!this.isExpect(str)) {
            this.error(msg || 'Expect string ' + str, loc);
        }
        this.updateIndex(str.length);
    };
    Parser.prototype.skipWhitespace = function () {
        while (this.index < this.length) {
            var code = this.charCode();
            if (!isWhiteSpace(code)) {
                break;
            }
            else if (code === 10) {
                // is \n
                this.updateLine();
            }
            this.updateIndex();
        }
    };
    Parser.prototype.skipWhitespaceBetweenTags = function (endTag, skipBeforeDelimiter) {
        if (skipBeforeDelimiter === void 0) { skipBeforeDelimiter = true; }
        var delimiters = this.options.delimiters;
        var start = this.index;
        while (start < this.length) {
            var code = this.charCode(start);
            if (isWhiteSpace(code)) {
                start++;
            }
            else if (this.isExpect(endTag, start) ||
                this.isTagStart(start) ||
                // skip whitespaces between tag starting and expression
                // but not skip before tag ending 
                (skipBeforeDelimiter && this.isExpect(delimiters[0], start))) {
                this.skipWhitespace();
                break;
            }
            else {
                break;
            }
        }
    };
    Parser.prototype.trimRightForValue = function (value) {
        for (var i = value.length - 1; i >= 0; i--) {
            var code = value[i];
            var lastIndex = code.length - 1;
            var j = lastIndex;
            for (; j >= 0; j--) {
                var charCode = code.charCodeAt(j);
                if (!isWhiteSpace(charCode)) {
                    break;
                }
            }
            if (j === -1) {
                // remove the whole line code
                value.pop();
            }
            else {
                if (j !== lastIndex) {
                    // trim right
                    value[i] = code.slice(0, j + 1);
                }
                break;
            }
        }
    };
    return Parser;
}());

var fakeLoc = { line: 0, column: 0 };
var numberRegExp = /^\d+$/;
var Visitor = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function () {
    function Visitor(nodes) {
        this.enterStringExpression = false;
        this.hoist = [];
        this.queue = [];
        this.queueStack = [this.queue];
        this.current = this.queue;
        this.line = 1;
        this.column = 0;
        this.indentLevel = 0;
        this.spaces = '';
        this.spacesStatck = [''];
        this.expressionSpacesStack = [];
        this.helpers = {};
        this.declares = {};
        this.tmpIndex = 0;
        this.hoistDeclares = [];
        this.functionHead = this.pushQueue();
        this.append("function($props, $blocks, $__proto__) {");
        this.indent();
        this.append("$blocks || ($blocks = {});");
        this.newline();
        this.append("$props || ($props = {});");
        this.newline();
        this.append('var $this = this;');
        this.popQueue();
        this.newline();
        this.visit(nodes, true);
        this.append(';');
        this.dedent();
        this.append('};');
    }
    Visitor.prototype.getCode = function () {
        var helpers = [];
        for (var key in this.helpers) {
            helpers.push("var ".concat(key, " = Vdt.").concat(helpersMap[key], ";\n"));
        }
        return [
            this.hoist.join(''),
            "var Vdt = _$vdt;\n",
            helpers.join(''),
            this.getHositDeclares(),
            "\nreturn ",
            this.functionHead.join(''),
            this.getDelares(),
            this.queue.join(''),
        ].join('');
    };
    Visitor.prototype.getModuleCode = function (moduleName) {
        if (moduleName === void 0) { moduleName = 'vdt/runtime'; }
        var helpers = this.pushQueue();
        var keys = Object.keys(this.helpers);
        var length = keys.length;
        if (length) {
            helpers.push("import {");
            this.indent();
            for (var i = 0; i < length; i++) {
                var key = keys[i];
                helpers.push("".concat(helpersMap[key], " as ").concat(key, ","));
                if (i !== length - 1) {
                    this.newline();
                }
            }
            this.dedent();
            this.append("} from '".concat(moduleName, "';\n"));
        }
        this.popQueue();
        return [
            helpers.join(''),
            this.hoist.join(''),
            this.getHositDeclares(),
            '\nexport default ',
            this.functionHead.join(''),
            this.getDelares(),
            this.queue.join(''),
        ].join('');
    };
    Visitor.prototype.getDelares = function () {
        var declares = this.pushQueue();
        this.indent();
        for (var key in this.declares) {
            var declare = this.declares[key];
            this.append("var ".concat(key, " = "));
            this.append(isArray(declare) ? declare.join('') : declare);
            this.append(';');
            this.newline();
        }
        this.indentLevel--;
        this.popQueue();
        return declares.join('');
    };
    Visitor.prototype.getHositDeclares = function () {
        var declares = [];
        var hoistDeclares = this.hoistDeclares;
        for (var i = 0; i < hoistDeclares.length; i++) {
            var _a = hoistDeclares[i], key = _a[0], code = _a[1];
            declares.push("var ".concat(key, " = ").concat(code.join(''), ";\n"));
        }
        return declares.join('');
    };
    Visitor.prototype.visit = function (nodes, isRoot) {
        var length = nodes.length;
        var lastIndex = length - 1;
        this.expressionSpacesStack.push(this.spaces);
        var oldLength = this.spacesStatck.length;
        for (var i = 0; i < nodes.length; i++) {
            // if is root, add `return` keyword
            if (isRoot && i === lastIndex) {
                this.append('return ');
            }
            this.visitNode(nodes[i], isRoot, true);
        }
        var newLength = this.spacesStatck.length;
        this.expressionSpacesStack.pop();
        while (newLength > oldLength) {
            this.popSpaces();
            newLength--;
        }
    };
    Visitor.prototype.visitNode = function (node, isRoot, textToVNode) {
        var type = node.type;
        switch (type) {
            case 3 /* JSXCommonElement */:
            case 4 /* JSXComponent */:
            case 7 /* JSXBlock */:
            case 6 /* JSXVdt */:
                return this.visitJSXElement(node, isRoot, textToVNode);
            case 2 /* JSXText */:
                return this.visitJSXText(node, textToVNode);
            case 12 /* JSXString */:
                this.visitString(node, false);
                return;
            case 0 /* JS */:
                this.visitJS(node);
                return;
            case 8 /* JSXExpression */:
                return this.visitJSXExpression(node);
            case 1 /* JSHoist */:
                this.visitJSHoist(node);
                return;
            case 5 /* JSXComment */:
                return this.visitJSXComment(node);
            case 10 /* JSXUnescapeText */:
                return this.visitJSXUnescapeText(node);
            case 13 /* JSXStrings */:
                this.visitStrings(node);
                return;
            case 14 /* JSXNone */:
                this.append('true');
                return;
        }
    };
    Visitor.prototype.visitJS = function (node) {
        // const spaces = this.spaces;
        var stack = this.expressionSpacesStack;
        this.spaces = stack[stack.length - 1];
        var lines = node.value;
        var length = lines.length;
        for (var i = 0; i < length; i++) {
            var code = lines[i];
            this.append(code);
            if (i !== length - 1) {
                this.newline();
            }
        }
        // this.spaces = spaces;
        this.pushSpaces(node.spaces);
    };
    Visitor.prototype.visitJSHoist = function (node) {
        this.hoist.push(node.value);
    };
    Visitor.prototype.visitJSXElement = function (node, isRoot, textToVNode) {
        var _this = this;
        return this.visitJSXDirective(node, function (hasFor) {
            switch (node.type) {
                case 3 /* JSXCommonElement */:
                    if (node.value === 'template') {
                        // <template> is a fake tag, we only need handle its children and directives
                        return _this.visitJSXChildren(node.children, hasFor ? true : textToVNode);
                    }
                    return _this.visitJSXCommonElement(node);
                case 4 /* JSXComponent */:
                    return _this.visitJSXComponent(node);
                case 7 /* JSXBlock */:
                    return _this.visitJSXBlock(node, true);
                case 6 /* JSXVdt */:
                    return _this.visitJSXVdt(node, isRoot);
            }
        });
    };
    Visitor.prototype.visitJSXCommonElement = function (node) {
        var tag = node.value;
        if (this.hasExpressionProp(node.attributes)) {
            this.addHelper('_$cv');
            this.append("_$cv('".concat(tag, "', "));
            var _a = this.visitJSXAttribute(node, false), key_1 = _a.key; _a.hasProps;
            if (node.children.length) {
                this.append(', ');
                this.visitJSXChildren(node.children);
            }
            this.append(')');
            return key_1 ? 2 /* HasKeyedVNodeChildren */ : 4 /* HasNonKeyedVNodeChildren */;
        }
        this.addHelper('_$ce');
        this.append("_$ce(".concat(getTypeForVNodeElement(tag), ", '").concat(tag, "'"));
        this.pushQueue();
        this.append(', ');
        var childrenFlag = this.visitJSXChildren(node.children);
        this.append(", ".concat(childrenFlagToChildrenType(childrenFlag)));
        if (childrenFlag !== 1 /* HasInvalidChildren */) {
            this.flush(this.popQueue());
            this.pushQueue();
        }
        var propsQueue = this.pushQueue();
        this.append(', ');
        var _b = this.visitJSXAttribute(node, true), className = _b.className, key = _b.key, ref = _b.ref, hasProps = _b.hasProps;
        this.popQueue();
        this.append(', ');
        if (className) {
            this.flush(this.popQueue());
            this.visitJSXAttributeClassName(className);
            this.pushQueue();
        }
        else {
            this.append('null');
        }
        this.flush(propsQueue);
        var ret = this.visitProps(hasProps, key, ref, false);
        this.append(')');
        return ret;
    };
    Visitor.prototype.visitJSXComponent = function (node) {
        var name = node.value;
        // switch (name) {
        // case 'Transition':
        // name = '_$tr';
        // this.addHelper(name as '_$tr');
        // break;
        // case 'TransitionGroup':
        // name = '_$tg';
        // this.addHelper(name as '_$tg');
        // break;
        // default:
        // break;
        // } 
        var blocks = this.getJSXBlocksAndSetChildren(node);
        if (blocks.length) {
            node.attributes.push({
                type: 11 /* JSXAttribute */,
                name: '$blocks',
                value: blocks,
                loc: fakeLoc,
            });
        }
        this.addHelper('_$cc');
        this.append("_$cc(".concat(name));
        this.pushQueue();
        this.append(', ');
        var _a = this.visitJSXAttribute(node, false), key = _a.key, ref = _a.ref, hasProps = _a.hasProps;
        var ret = this.visitProps(hasProps, key, ref, true);
        this.append(')');
        return ret;
    };
    Visitor.prototype.visitJSXExpression = function (node) {
        var value = node.value;
        if (!value.length) {
            this.append('null');
            return 1 /* HasInvalidChildren */;
        }
        else {
            this.visit(value, false);
            return 0 /* UnknownChildren */;
        }
    };
    Visitor.prototype.visitJSXText = function (node, textToVNode) {
        if (textToVNode) {
            this.addHelper('_$ct');
            this.append('_$ct(');
            this.visitString(node, false);
            this.append(')');
            return 4 /* HasNonKeyedVNodeChildren */;
        }
        this.visitString(node, false);
        return 32 /* HasTextChildren */;
    };
    Visitor.prototype.visitStrings = function (node) {
        var value = node.value;
        var lastIndex = value.length - 1;
        for (var i = 0; i <= lastIndex; i++) {
            this.visitNode(value[i], false, false);
            if (i !== lastIndex) {
                this.append(' + ');
            }
        }
    };
    Visitor.prototype.visitJSXUnescapeText = function (node) {
        this.addHelper('_$cu');
        this.append('_$cu(');
        this.visit(node.value, false);
        this.append(')');
        return 4 /* HasNonKeyedVNodeChildren */;
    };
    Visitor.prototype.visitJSXBlock = function (node, shouldCall) {
        var _a = this.getJSXBlocksAttribute(node), params = _a.params, args = _a.args;
        var name = node.value;
        if (shouldCall) {
            this.addDeclare('_$blocks', '{}');
            this.addDeclare('__$blocks', '{}');
        }
        this.append('(');
        this.indent();
        this.append("(_$blocks['".concat(name, "'] = function($super"));
        if (args) {
            this.append(', ');
            this.visitString(args, true);
        }
        this.append(') {');
        this.indent();
        this.append('return ');
        this.visitJSXChildren(node.children);
        this.append(';');
        this.dedent();
        this.append("}),");
        this.newline();
        this.append("(__$blocks['".concat(name, "'] = function($super, data) {"));
        this.indent();
        this.append("var block = $blocks['".concat(name, "'];"));
        this.newline();
        this.append("var callBlock = function() {");
        this.indent();
        this.append("return _$blocks['".concat(name, "'].call($this, $super, data);"));
        this.dedent();
        this.append('};');
        this.newline();
        this.append("return block ?");
        this.indent();
        this.append("block.call($this, callBlock, data) :");
        this.newline();
        this.append("callBlock();");
        this.indentLevel--;
        this.dedent();
        this.append('})');
        // if it is in ancestor, we should call this block
        if (shouldCall) {
            this.append(',');
            this.newline();
            this.addHelper('_$no');
            if (params) {
                this.append("__$blocks['".concat(name, "'](_$no, "));
                this.visitJSXAttributeValue(params);
                this.append(')');
            }
            else {
                this.append("__$blocks['".concat(name, "'](_$no)"));
            }
        }
        this.dedent();
        this.append(')');
        return 0 /* UnknownChildren */;
    };
    Visitor.prototype.visitJSXVdt = function (node, isRoot) {
        var blocks = this.getJSXBlocksAndSetChildren(node);
        var name = node.value;
        if (name === 'super') {
            name = '_$su';
            this.addHelper(name);
        }
        this.append("".concat(name, ".call($this, "));
        this.visitJSXAttribute(node, false);
        this.append(", ");
        if (blocks.length) {
            this.visitJSXBlocks(blocks, isRoot);
        }
        else {
            this.append(isRoot ? '$blocks' : 'null');
        }
        this.append(', $__proto__)');
        return 0 /* UnknownChildren */;
    };
    Visitor.prototype.visitJSXComment = function (node) {
        this.addHelper('_$ccv');
        this.append('_$ccv(');
        this.visitString(node, false);
        this.append(')');
        return 4 /* HasNonKeyedVNodeChildren */;
    };
    Visitor.prototype.visitString = function (node, noQuotes) {
        var value = node.value.replace(/([\'\"\\])/g, '\\$1').replace(/[\r\n]/g, '\\n');
        if (!noQuotes) {
            value = "'".concat(value, "'");
        }
        this.append(value);
    };
    Visitor.prototype.visitJSXDirective = function (node, body) {
        var _this = this;
        var directiveFor = {};
        var directiveIf = null;
        var directives = node.directives;
        for (var key in directives) {
            var directive = directives[key];
            switch (key) {
                case "v-if" /* If */:
                    directiveIf = directive;
                    break;
                case "v-for" /* For */:
                    directiveFor.data = directive.value;
                    break;
                case "v-for-value" /* ForValue */:
                    directiveFor.value = directive.value;
                    break;
                case "v-for-key" /* ForKey */:
                    directiveFor.key = directive.value;
                    break;
            }
        }
        // handle v-for firstly
        if (directiveFor.data) {
            if (directiveIf) {
                return this.visitJSXDirectiveFor(directiveFor, function () {
                    return _this.visitJSXDirectiveIf(directiveIf, function () { return body(true); });
                });
            }
            else {
                return this.visitJSXDirectiveFor(directiveFor, function () { return body(true); });
            }
        }
        else if (directiveIf) {
            return this.visitJSXDirectiveIf(directiveIf, function () { return body(false); });
        }
        else {
            return body(false);
        }
    };
    Visitor.prototype.visitJSXDirectiveFor = function (directive, body) {
        this.addHelper('_$ma');
        this.append('_$ma(');
        this.visitJSXAttributeValue(directive.data);
        this.append(', function(');
        if (directive.value) {
            this.visitString(directive.value, true);
        }
        else {
            this.append('$value');
        }
        this.append(', ');
        if (directive.key) {
            this.visitString(directive.key, true);
        }
        else {
            this.append('$key');
        }
        this.append(') {');
        this.indent();
        this.append('return ');
        var childrenFlag = body();
        this.append(';');
        this.dedent();
        this.append('}, $this)');
        if (childrenFlag & 10 /* KeydChildren */) {
            return 8 /* HasKeyedChildren */;
        }
        else if (childrenFlag === 0 /* UnknownChildren */) {
            return childrenFlag;
        }
        return 16 /* HasNonKeyedChildren */;
    };
    Visitor.prototype.visitJSXDirectiveIf = function (directive, body) {
        var indentLevel = this.indentLevel;
        var hasElse = false;
        this.append('(');
        this.visitJSXAttributeValue(directive.value);
        this.append(')');
        this.append(' ?');
        this.indent();
        var childrenFlag = body();
        this.append(' :');
        this.newline();
        var next = directive.next;
        while (next) {
            var nextDirectives = next.directives;
            var elseIfDir = nextDirectives["v-else-if" /* ElseIf */];
            if (elseIfDir) {
                this.visitJSXAttributeValue(elseIfDir.value);
                this.append(' ?');
                this.indent();
                childrenFlag = computeChildrenFlagForVIf(childrenFlag, this.visitNode(next, false, false));
                this.append(' :');
                this.newline();
                next = elseIfDir.next;
                continue;
            }
            else {
                // should be Directives.Else
                childrenFlag = computeChildrenFlagForVIf(childrenFlag, this.visitNode(next, false, false));
                hasElse = true;
            }
            break;
        }
        if (!hasElse) {
            this.append('undefined');
            childrenFlag = computeChildrenFlagForVIf(childrenFlag, 1 /* HasInvalidChildren */);
        }
        this.indentLevel = indentLevel;
        return childrenFlag;
    };
    Visitor.prototype.visitJSXChildren = function (nodes, textToVNode) {
        var length = nodes.length;
        if (!length) {
            this.append('null');
            return 1 /* HasInvalidChildren */;
        }
        var childrenFlag = null;
        if (length === 1) {
            var child = nodes[0];
            var hasIndent = false;
            if (isElementNode(child)) {
                hasIndent = true;
                this.append('(');
                this.indent();
            }
            childrenFlag = this.visitNode(child, false, isUndefined(textToVNode) ? false : textToVNode);
            if (hasIndent) {
                this.dedent();
                this.append(')');
            }
        }
        else {
            this.append('[');
            this.indent();
            var lastIndex = nodes.length - 1;
            for (var i = 0; i < nodes.length; i++) {
                var child = nodes[i];
                child.type;
                var flag = this.visitNode(child, false, true);
                childrenFlag = computeChildrenFlagForChildren(childrenFlag, flag);
                if (i !== lastIndex) {
                    this.append(',');
                    this.newline();
                }
            }
            this.dedent();
            this.append(']');
            if (childrenFlag === 2 /* HasKeyedVNodeChildren */) {
                childrenFlag = 8 /* HasKeyedChildren */;
            }
            else if (childrenFlag === 4 /* HasNonKeyedVNodeChildren */) {
                childrenFlag = 16 /* HasNonKeyedChildren */;
            }
        }
        return childrenFlag;
    };
    Visitor.prototype.visitJSXAttribute = function (node, isCommonElement) {
        var _this = this;
        var attributes = node.attributes;
        var className = null;
        var key = null;
        var ref = null;
        if (!attributes.length) {
            this.append('null');
            return { className: className, key: key, ref: ref, hasProps: false };
        }
        // use a queue to save props, so we can extract it when there isn't dynamic prop
        var propsQueue = this.pushQueue();
        // const isCommonElement = node.type === ASTTypes.JSXCommonElement;
        var modelMeta = {};
        var models = [];
        var hasDynamicProp = this.hasDynamicProp(attributes, isCommonElement);
        var indentLevel = this.indentLevel;
        var spaces = this.spaces;
        if (!hasDynamicProp) {
            this.indentLevel = 0;
            this.spaces = '';
        }
        var isFirstAttr = true;
        var addAttribute = function (name) {
            if (isFirstAttr) {
                _this.append('{');
                _this.indent();
                isFirstAttr = false;
            }
            else {
                _this.append(',');
                _this.newline();
            }
            if (name) {
                _this.append("'".concat(name, "': "));
            }
        };
        for (var i = 0; i < attributes.length; i++) {
            var attr = attributes[i];
            if (attr.type === 8 /* JSXExpression */) {
                addAttribute();
                this.visitJSXAttributeValue(attr);
                continue;
            }
            var name_1 = getAttrName(attr.name);
            var value = attr.value;
            switch (name_1) {
                case 'className':
                    if (!isCommonElement) {
                        addAttribute(name_1);
                        this.visitJSXAttributeClassName(value);
                    }
                    className = value;
                    continue;
                case 'key':
                    if (!isCommonElement) {
                        addAttribute(name_1);
                        this.visitJSXAttributeValue(value);
                    }
                    key = value;
                    continue;
                case 'ref':
                    if (!isCommonElement) {
                        addAttribute(name_1);
                        this.visitJSXAttributeRef(value, true);
                    }
                    ref = value;
                    continue;
                case '$blocks':
                    addAttribute(name_1);
                    this.visitJSXBlocks(value, false);
                    continue;
                case 'type':
                    // save the type for v-model of input element
                    addAttribute(name_1);
                    this.visitJSXAttributeValue(value);
                    modelMeta.type = value;
                    continue;
                case 'value':
                    addAttribute(name_1);
                    this.visitJSXAttributeValue(value);
                    modelMeta.value = value;
                    break;
                case 'v-model-true':
                    addAttribute('trueValue');
                    this.visitJSXAttributeValue(value);
                    modelMeta.trueValue = value;
                    break;
                case 'v-model-false':
                    addAttribute('falseValue');
                    this.visitJSXAttributeValue(value);
                    modelMeta.falseValue = value;
                    break;
                default:
                    if (name_1 === 'v-model') {
                        models.push({ name: 'value', value: value });
                        continue;
                    }
                    else if (name_1.substr(0, 8) === 'v-model:') {
                        models.push({ name: name_1.substr(8), value: value });
                        continue;
                    }
                    else {
                        addAttribute(name_1);
                        this.visitJSXAttributeValue(value);
                    }
                    break;
            }
        }
        for (var i = 0; i < models.length; i++) {
            this.visitJSXAttributeModel(node, models[i], modelMeta, addAttribute);
        }
        if (!isFirstAttr) {
            this.dedent();
            this.append('}');
        }
        else {
            this.append('null');
        }
        this.indentLevel = indentLevel;
        this.spaces = spaces;
        this.popQueue();
        if (isFirstAttr || hasDynamicProp) {
            this.flush(propsQueue);
        }
        else {
            this.append(this.addHoistDeclare(propsQueue));
        }
        return { className: className, key: key, ref: ref, hasProps: !isFirstAttr };
    };
    Visitor.prototype.hasExpressionProp = function (attributes) {
        for (var i = 0; i < attributes.length; i++) {
            var attr = attributes[i];
            if (attr.type === 8 /* JSXExpression */) {
                return true;
            }
        }
        return false;
    };
    Visitor.prototype.hasDynamicProp = function (attributes, isCommonElement) {
        var detect = function (name, attr) {
            if (name === 'v-model' || name.substr(0, 8) === 'v-model:') {
                return true;
            }
            var value = attr.value;
            if (value.type === 8 /* JSXExpression */) {
                // if value is number / true / false, treat it as static value
                var tmp = value.value;
                var length_1 = tmp.length;
                if (!length_1)
                    return false; // is an empty expresion, it will be generated to null
                if (length_1 === 1 &&
                    (tmp = tmp[0], tmp.type === 0 /* JS */) &&
                    (tmp = tmp.value, tmp.length === 1)) {
                    var jsValue = tmp[0];
                    switch (jsValue) {
                        case 'true':
                        case 'false':
                        case 'null':
                        case 'undefined':
                            return false;
                        default:
                            if (numberRegExp.test(jsValue))
                                return false;
                            break;
                    }
                }
                return true;
            }
            if (value.type === 13 /* JSXStrings */) {
                var values = value.value;
                for (var i = 0; i < values.length; i++) {
                    if (values[i].type === 8 /* JSXExpression */) {
                        return true;
                    }
                }
            }
            return false;
        };
        for (var i = 0; i < attributes.length; i++) {
            var attr = attributes[i];
            if (attr.type === 8 /* JSXExpression */) {
                return true;
            }
            var name_2 = attr.name;
            switch (name_2) {
                case 'class':
                case 'className':
                case 'key':
                    if (isCommonElement)
                        continue;
                    if (detect(name_2, attr))
                        return true;
                    break;
                case 'ref':
                    if (!isCommonElement)
                        return true;
                    break;
                case '$blocks':
                case 'children':
                    return true;
                default:
                    if (detect(name_2, attr))
                        return true;
                    break;
            }
        }
        return false;
    };
    Visitor.prototype.visitJSXAttributeClassName = function (value) {
        if (value.type === 8 /* JSXExpression */) {
            this.addHelper('_$cn');
            this.append('_$cn(');
            this.visitJSXAttributeValue(value);
            this.append(')');
        }
        else {
            this.visitJSXAttributeValue(value);
        }
    };
    Visitor.prototype.visitJSXAttributeValue = function (value) {
        if (isArray(value)) {
            this.visitJSXChildren(value);
        }
        else {
            this.visitNode(value, false, false);
        }
    };
    Visitor.prototype.visitJSXAttributeRef = function (value, isComponent) {
        if (value.type === 12 /* JSXString */) {
            this.addDeclare('_$refs', 'this.refs');
            // if it is a component, the ref will use twice, so we extract it as variable
            var indentLevel = void 0;
            var refQueue = void 0;
            if (isComponent) {
                refQueue = this.pushQueue();
                indentLevel = this.indentLevel;
                this.indentLevel = 0;
            }
            this.append("function(i) {_$refs[");
            this.visitJSXAttributeValue(value);
            this.append("] = i}");
            if (isComponent) {
                this.indentLevel = indentLevel;
                this.popQueue();
                this.append(this.addDeclare("_$ref_".concat(value.value), refQueue));
            }
        }
        else {
            this.visit(value.value, false);
        }
    };
    Visitor.prototype.visitJSXAttributeModel = function (node, _a, modelMeta, addAttribute) {
        var _this = this;
        var name = _a.name, value = _a.value;
        var shouldSetValue = true;
        var handleRadioOrCheckbox = function (head, middle, tail, defaultValue) {
            shouldSetValue = false;
            addAttribute('checked');
            _this.append(head);
            _this.visitJSXAttributeValue(value);
            _this.append(middle);
            if (tail) {
                _this.visitJSXAttributeValue(tail);
            }
            else {
                _this.append(defaultValue);
            }
        };
        if (node.type === 3 /* JSXCommonElement */) {
            var setModelFnName = '_$stm';
            var eventName = 'change';
            addAttribute("$model:".concat(name));
            this.visitJSXAttributeValue(value);
            switch (node.value) {
                case 'input':
                    var type = modelMeta.type;
                    if (type) {
                        switch (type.value) {
                            case 'radio':
                                handleRadioOrCheckbox('$this.get(', ') === ', modelMeta.value, "'on'");
                                setModelFnName = '_$srm';
                                break;
                            case 'checkbox':
                                this.addHelper('_$isc');
                                handleRadioOrCheckbox('_$isc($this.get(', '), ', modelMeta.trueValue, 'true');
                                this.append(')');
                                setModelFnName = '_$scm';
                                break;
                            // case 'number':
                            // TODO: shoud cast to number
                            default:
                                eventName = 'input';
                                break;
                        }
                    }
                    else {
                        eventName = 'input';
                    }
                    break;
                case 'select':
                    setModelFnName = '_$ssm';
                    break;
                default:
                    eventName = 'input';
                    break;
            }
            addAttribute("ev-$model:".concat(eventName));
            this.addHelper('_$le');
            this.addHelper(setModelFnName);
            this.append("_$le($this, ".concat(setModelFnName, ")"));
        }
        else {
            // is a component
            addAttribute("ev-$model:".concat(name));
            this.append("function($v) {");
            this.indent();
            this.append("$this.set(");
            this.visitJSXAttributeValue(value);
            this.append(', $v);');
            this.dedent();
            this.append('}');
        }
        if (shouldSetValue) {
            addAttribute(name);
            this.append('$this.get('),
                this.visitJSXAttributeValue(value);
            this.append(')');
        }
    };
    Visitor.prototype.visitJSXBlocks = function (blocks, isRoot) {
        this.addHelper('_$ex');
        this.addHelper('_$em');
        this.append('function($blocks) {');
        this.indent();
        this.append("var _$blocks = {}, __$blocks = _$ex({}, $blocks);");
        this.newline();
        this.append('return (');
        this.indent();
        for (var i = 0; i < blocks.length; i++) {
            this.visitJSXBlock(blocks[i], false);
            this.append(',');
            this.newline();
        }
        this.append('__$blocks');
        this.dedent();
        this.append(');');
        this.dedent();
        this.append("}.call($this, ".concat(isRoot ? '$blocks' : '_$em', ")"));
    };
    Visitor.prototype.getJSXBlocksAndSetChildren = function (node) {
        var blocks = [];
        var children = [];
        var nodeChildren = node.children;
        for (var i = 0; i < nodeChildren.length; i++) {
            var child = nodeChildren[i];
            if (child.type === 7 /* JSXBlock */) {
                blocks.push(child);
            }
            else {
                children.push(child);
            }
        }
        if (children.length) {
            node.attributes.push({
                type: 11 /* JSXAttribute */,
                name: 'children',
                value: children,
                loc: fakeLoc,
            });
        }
        return blocks;
    };
    Visitor.prototype.getJSXBlocksAttribute = function (node) {
        var ret = { args: null, params: null };
        var attributes = node.attributes;
        for (var i = 0; i < attributes.length; i++) {
            var attr = attributes[i];
            switch (attr.name) {
                case 'args':
                    ret.args = attr.value;
                    break;
                case 'params':
                    ret.params = attr.value;
                    break;
            }
        }
        return ret;
    };
    Visitor.prototype.visitProps = function (hasProps, key, ref, isComponent) {
        var childrenFlag = 4 /* HasNonKeyedVNodeChildren */;
        if (hasProps) {
            this.flush(this.popQueue());
            this.pushQueue();
        }
        this.append(', ');
        if (key) {
            this.flush(this.popQueue());
            this.visitJSXAttributeValue(key);
            this.pushQueue();
            childrenFlag = 2 /* HasKeyedVNodeChildren */;
        }
        else {
            this.append('null');
        }
        this.append(', ');
        if (ref) {
            this.flush(this.popQueue());
            this.visitJSXAttributeRef(ref, isComponent);
            this.pushQueue();
        }
        else {
            this.append('null');
        }
        this.popQueue();
        return childrenFlag;
    };
    Visitor.prototype.append = function (code) {
        this.current.push(code);
        // if (true [> sourceMap <]) {
        // TODO 
        // }
    };
    Visitor.prototype.addHelper = function (helper) {
        this.helpers[helper] = true;
    };
    Visitor.prototype.addHoistDeclare = function (code) {
        var key = "_$tmp".concat(this.tmpIndex++);
        this.hoistDeclares.push([key, code]);
        return key;
    };
    Visitor.prototype.addDeclare = function (key, code) {
        this.declares[key] = code;
        return key;
    };
    Visitor.prototype.pushQueue = function () {
        this.current = [];
        this.queueStack.push(this.current);
        return this.current;
    };
    Visitor.prototype.popQueue = function () {
        var stack = this.queueStack;
        var queue = stack.pop();
        this.current = stack[stack.length - 1];
        return queue;
    };
    Visitor.prototype.flush = function (queue) {
        for (var i = 0; i < queue.length; i++) {
            this.append(queue[i]);
        }
    };
    Visitor.prototype.indent = function () {
        this.indentLevel++;
        this.newline();
    };
    Visitor.prototype.dedent = function () {
        this.indentLevel--;
        this.newline();
    };
    Visitor.prototype.newline = function () {
        this.append('\n' + "    ".repeat(this.indentLevel) + this.spaces);
    };
    Visitor.prototype.pushSpaces = function (spaces) {
        if (spaces) {
            this.spaces += ' '.repeat(spaces);
        }
        this.spacesStatck.push(this.spaces);
    };
    Visitor.prototype.popSpaces = function () {
        var stack = this.spacesStatck;
        stack.pop();
        this.spaces = stack[stack.length - 1];
    };
    return Visitor;
}());

var cache$1 = {};
function compile(source) {
    if (cache$1[source])
        return cache$1[source];
    var parser = new Parser(source);
    var visitor = new Visitor(parser.ast);
    var code = visitor.getCode();
    return (cache$1[source] = new Function('_$vdt', code)(Vdt));
}
registerCompile(compile);

var FakePromise = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function () {
    function FakePromise(executor) {
        this.resolved = false;
        this.callbacks = [];
        executor(this.resolve.bind(this));
    }
    FakePromise.all = function (promises) {
        if (promises.all)
            return promises.all;
        var callbacks = [];
        var resolved = false;
        var resolvedCount = 0;
        promises.then = isAllPromisesResolved;
        promises.value.forEach(function (promise) {
            promise.then(isAllPromisesResolved);
        });
        function callCallback() {
            var cb;
            while (cb = callbacks.shift()) {
                cb();
            }
            promises.done = true;
        }
        function isAllPromisesResolved() {
            resolvedCount++;
            if (promises.value.length === resolvedCount) {
                resolved = true;
                callCallback();
            }
        }
        return (promises.all = {
            then: function (cb) {
                callbacks.push(cb);
                if (!promises.value.length || resolved) {
                    callCallback();
                }
                return this;
            }
        });
    };
    FakePromise.prototype.then = function (cb) {
        this.callbacks.push(cb);
        if (this.resolved) {
            this.resolve();
        }
    };
    FakePromise.prototype.resolve = function () {
        this.resolved = true;
        var cb;
        while (cb = this.callbacks.shift()) {
            cb();
        }
    };
    return FakePromise;
}());
// let id = 0;
var FakePromises = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function () {
    function FakePromises() {
        this.value = [];
        this.then = null;
        this.done = false;
        this.all = null;
    }
    // public id = id++;
    FakePromises.prototype.add = function (promise) {
        this.value.push(promise);
        var then = this.then;
        if (then) {
            promise.then(then);
        }
    };
    FakePromises.prototype.reset = function () {
        this.value = [];
        this.then = null;
        this.done = false;
        this.all = null;
    };
    return FakePromises;
}());

var internalInstanceKey;
function precacheFiberNode(node, placeholder) {
    if (!internalInstanceKey) {
        var keys = Object.keys(placeholder);
        internalInstanceKey = keys[0];
    }
    var fiber = placeholder[internalInstanceKey];
    node[internalInstanceKey] = fiber;
    return fiber;
}
var listeningMarker;
var bind = Function.prototype.bind;
// excerpt from react definition
var IS_EVENT_HANDLE_NON_MANAGED_NODE = 1;
var IS_NON_DELEGATED = 2;
Function.prototype.bind = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var obj = args[0], domEventName = args[1], eventSystemFlags = args[2], targetContainer = args[3], nativeEvent = args[4];
    if (obj === null && isString(domEventName) && isNumber(eventSystemFlags) && targetContainer instanceof Element) {
        var isReactListening = false;
        if (!listeningMarker) {
            var keys = Object.keys(targetContainer);
            var key = keys.find(function (key) { return key.startsWith('_reactListening'); });
            if (key) {
                listeningMarker = key;
                isReactListening = true;
            }
            else {
                isReactListening = false;
            }
        }
        else {
            isReactListening = targetContainer[listeningMarker] && nativeEvent instanceof Event;
        }
        if (isReactListening && (eventSystemFlags & IS_EVENT_HANDLE_NON_MANAGED_NODE) === 0 && (eventSystemFlags & IS_NON_DELEGATED) === 0) {
            /**
             * Because we only add listeners to the react root container, the comment root container in Wrapper will ignore doing this.
             * But React will check the container (isMatchingRootContainer). it will always return false since the comment parentNode is not
             * the targetContainer. Therefore, we fake the parentNode by returning the targetContainer to skip the check in React.
             * Eventually we restore the value after calling the function.
             */
            var _fn_1 = this;
            var fn = function (name, eventSystemFlags, targetContainer, nativeEvent) {
                var targetInst = getClosestInstanceFromNode(nativeEvent.target);
                var commentRoot = getCommentRoot(targetInst);
                var containerInfo;
                if (commentRoot) {
                    containerInfo = commentRoot.stateNode.containerInfo;
                    commentRoot.stateNode.containerInfo = targetContainer;
                }
                var ret = _fn_1(name, eventSystemFlags, targetContainer, nativeEvent);
                if (commentRoot) {
                    commentRoot.stateNode.containerInfo = containerInfo;
                }
                return ret;
            };
            return bind.call(fn, null, domEventName, eventSystemFlags, targetContainer, nativeEvent);
        }
    }
    return bind.call.apply(bind, __spreadArray([this], args));
};
function getClosestInstanceFromNode(targetNode) {
    var targetInst = targetNode[internalInstanceKey];
    if (targetInst) {
        // Don't return HostRoot or SuspenseComponent here.
        return targetInst;
    } // If the direct event target isn't a React owned DOM node, we need to look
    // to see if one of its parents is a React owned DOM node.
    var parentNode = targetNode.parentNode;
    while (parentNode) {
        targetInst = parentNode[internalInstanceKey];
        if (targetInst) {
            return targetInst;
        }
        targetNode = parentNode;
        parentNode = targetNode.parentNode;
    }
    return null;
}
function getCommentRoot(targetInst) {
    var node = targetInst;
    while (true) {
        if (node === null)
            return;
        var nodeTag = node.tag;
        if (nodeTag === 3 /* HostRoot */) {
            return node;
        }
        node = node.return;
    }
}

var Context = createContext(null);
var containerComment = ' react-mount-point-unstable ';
var Wrapper = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function () {
    function Wrapper($props, $vNode, $SVG, $mountedQueue, $senior) {
        this.$props = $props;
        this.$vNode = $vNode;
        this.$SVG = $SVG;
        this.$mountedQueue = $mountedQueue;
        this.$senior = $senior;
        this.$inited = true;
        this.container = document.createComment(containerComment);
        this.isEmptyReactComponent = false;
        this.$lastInput = createVNode('div');
        // fakeInput.transition = $vNode.transition;
        // fakeInput.transition = {
        // // only need leave hook to prevent Intact from removing the real dom,
        // // because it will be removed by React.
        // leave: noop
        // } as unknown as TransitionHooks;
    }
    Wrapper.prototype.$init = function (props) { };
    Wrapper.prototype.$render = function (lastVNode, vNode, parentDom, anchor) {
        if (lastVNode) {
            removeVNodeDom(lastVNode, parentDom);
        }
        var container = this.container;
        /**
         * React will insert child before container, if container is a comment node.
         * So we should insert the container into the corresponding position
         */
        if (anchor) {
            parentDom.insertBefore(container, anchor);
        }
        else {
            parentDom.appendChild(container);
        }
        rewriteParentElementApi(parentDom, !existPortal(this));
        this.render(vNode, parentDom);
    };
    Wrapper.prototype.$update = function (lastVNode, vNode, parentDom, anchor) {
        this.render(vNode, parentDom);
    };
    Wrapper.prototype.$unmount = function (vNode, nextVNode) {
        var _this = this;
        var container = this.container;
        /**
         * If we update intact component lead to this unmountion,
         * the dom will be removed by react immediately,
         * so we set a flag `hasRemoved` to indicate this case.
         * But maybe the wrapped element returns null, and react
         * will not remove the void text node that we created in render method.
         * So if it is in this case, we should remove the void text node manually.
         */
        var dom = this.$lastInput.dom;
        var hasRemoved = false;
        render$1(null, container, function () {
            hasRemoved = true;
            if (_this.isEmptyReactComponent) {
                container.parentNode.removeChild(dom);
            }
        });
        if (hasRemoved) {
            this.$lastInput.dom = container;
        }
    };
    Wrapper.prototype.render = function (vNode, parentDom) {
        var parent = this.$senior;
        var parentComponent = getParent(this);
        var instance = this;
        var container = this.container;
        var vnode = getReactElement(vNode);
        /**
         * pass the $senior as value instead of parentComponent
         * because parentComponent is the the toplevel component and
         * not always equal to the $senior
         * e.g. <A><B><div><C /></div></B></A>
         */
        vnode = createElement(Context.Provider, { value: parent }, vnode);
        // if the parent component has providers, pass them to subtree
        var providers = parentComponent.$reactProviders;
        providers.forEach(function (value, provider) {
            vnode = createElement(provider, { value: value }, vnode);
        });
        this.isEmptyReactComponent = false;
        var promise = new FakePromise(function (resolve) {
            unstable_renderSubtreeIntoContainer(parentComponent, vnode, container, function () {
                /**
                 * add dom to the $lastInput for findDomFromVNode
                 * the real dom will be inserted before the container
                 * but the react component may return null,
                 * so we can't get the previousSibling
                 *
                 * We will add the real element to the container
                 * by rewriting the parentNode.insertBefore api
                 *
                 * If we wrap an React component that returns Intact component directly
                 * e.g. <A><Context.Provider><B /><Context.Provider></A>
                 * the realElement will be the <template>
                 */
                var dom = container._realElement;
                if (!dom ||
                    dom.tagName === 'TEMPLATE' &&
                        dom.getAttribute('data-intact-react') !== null) {
                    // maybe this react component return null,
                    // we create a text as the dom
                    dom = document.createTextNode('');
                    parentDom.insertBefore(dom, container);
                    dom._isEmpty = true;
                }
                if (dom._isEmpty) {
                    instance.isEmptyReactComponent = true;
                }
                dom._mountPoint = container;
                instance.$lastInput.dom = dom;
                // console.log(rootContainer);
                // console.dir(container);
                // debugger;
                // getRootContainer(parentComponent);
                // Object.defineProperty((container as any)._reactRootContainer._internalRoot, 'containerInfo', {
                // get() {
                // return getRootContainer(parentComponent); 
                // }
                // });
                resolve();
            });
        });
        // if the promised has be resolved, it indicate that this render is sync.
        // Maybe we call update in intact directly, and it is not in React's render context
        if (!promise.resolved) {
            parentComponent.$promises.add(promise);
        }
    };
    return Wrapper;
}());
function getParent(instance) {
    var $senior = instance.$senior;
    do {
        if ($senior._reactInternals) {
            return $senior;
        }
    } while ($senior = $senior.$senior);
    // should not hit this
    /* istanbul ignore next */
    return null;
}
function existPortal(instance) {
    var $senior = instance.$senior;
    do {
        if ($senior.$isPortal) {
            return true;
        }
        if ($senior.constructor === Wrapper) {
            return false;
        }
    } while ($senior = $senior.$senior);
    return false;
}
function rewriteParentElementApi(parentElement, preventListener) {
    if (!parentElement._hasRewrite) {
        var removeChild_1 = parentElement.removeChild;
        parentElement.removeChild = function (child) {
            if (child.parentNode) {
                removeChild_1.call(parentElement, child);
                var realElement = child._realElement;
                if (realElement) {
                    child._realElement = null;
                    realElement._mountPoint = null;
                }
            }
            else {
                // if the node has been removed, then remove the mount point
                var container = child._mountPoint;
                removeChild_1.call(parentElement, container);
                child._mountPoint = null;
                container._realElement = null;
            }
        };
        var insertBefore_1 = parentElement.insertBefore;
        parentElement.insertBefore = function (child, beforeChild) {
            if (beforeChild.nodeType === 8 && beforeChild.nodeValue === containerComment) {
                // it's the wrapper container, we add the real element to it
                beforeChild._realElement = child;
            }
            insertBefore_1.call(parentElement, child, beforeChild);
        };
        // let react don't add listeners to the root container
        if (preventListener) {
            parentElement[listeningMarker] = true;
        }
        parentElement._hasRewrite = true;
    }
}
function getReactElement(vNode) {
    var props = vNode.props;
    var vnode = props.vnode;
    // react vNode has been frozen, so we must clone it to change
    var _props = null;
    for (var key in props) {
        if (key === 'vnode')
            continue;
        if (!_props)
            _props = {};
        var value = props[key];
        if (key.substr(0, 3) === 'ev-') {
            _props[eventsMap[key]] = value;
        }
        else {
            _props[key] = value;
        }
    }
    return _props ? cloneElement(vnode, _props) : vnode;
}
var eventsMap = {
    'ev-click': 'onClick',
    'ev-contextmenu': 'onContextMenu',
    'ev-mouseenter': 'onMouseEnter',
    'ev-mouseleave': 'onMouseLeave',
};

var cid = 'IntactReactFunctional';
function functionalWrapper(Component) {
    function Ctor(props, context) {
        if (context) {
            // invoked by React
            var vNodes = Component(normalizeProps(props, Component.events), true);
            if (isArray(vNodes)) {
                return createElement(Fragment$1, null, vNodes.map(function (vNode, index) {
                    return normalizeIntactVNodeToReactVNode(vNode, index);
                }));
            }
            return normalizeIntactVNodeToReactVNode(vNodes, 0);
        }
        else {
            // invoked by Intact
            return Component(props);
        }
    }
    var element = forwardRef(function (props, ref) {
        if (ref)
            props = __assign(__assign({}, props), { forwardRef: ref });
        return createElement(Ctor, props);
    });
    element._$type = Ctor;
    element.$cid = cid;
    return element;
}
function normalizeIntactVNodeToReactVNode(vNode, key) {
    var props = vNode.props;
    return createElement(vNode.tag, props ? __assign({ key: key }, props) : { key: key });
}

function normalize(vNode) {
    if (isInvalid(vNode))
        return null;
    // if a element has one child which is string or number
    // intact will set text content directly to update its children
    // this will lead to that the container which return
    // by Wrapper has been removed.
    // so we should convert string or number child
    // to VNode to let intact update it one by one
    if (isStringOrNumber(vNode)) {
        return createTextVNode(vNode);
    }
    // maybe return by functional component, see unit test: `render intact functional component`
    if (isNumber(vNode.type)) {
        return vNode;
    }
    if (isIntactComponent(vNode)) {
        var key = vNode.key;
        var type = vNode.type;
        var tag = type.$cid === cid ? type._$type : type;
        var props = normalizeProps(vNode.props, tag.events);
        if (!isNullOrUndefined(key)) {
            props.key = key;
        }
        return createComponentVNode(4, tag, props, key, normalizeRef(vNode.ref));
    }
    if (vNode.type === Fragment$1) {
        return normalizeChildren(vNode.props.children);
    }
    var ret = createComponentVNode(4, Wrapper, { vnode: vNode }, vNode.key);
    // transition has two functions
    // 1. prevent Intact from removing the real dom,
    //    because it will be removed by React.
    // TODO: 2. let intact never call clearDom method to remove all children
    //    because it will cause `react-mount-point-unstable` to be missing
    // ret.transition = {
    // leave: noop,
    // } as unknown as TransitionHooks;
    return ret;
}
function normalizeChildren(vNodes) {
    if (isArray(vNodes)) {
        var ret_1 = [];
        vNodes.forEach(function (vNode) {
            if (isArray(vNode)) {
                ret_1.push.apply(ret_1, normalizeChildren(vNode));
            }
            else {
                ret_1.push(normalize(vNode));
            }
        });
        return ret_1;
    }
    return normalize(vNodes);
}
function normalizeProps(props, events) {
    // if (!props) return null;
    var blocks = null;
    var normalizedProps = {};
    var tmp;
    for (var key in props) {
        var value = props[key];
        if (key === 'children') {
            normalizedProps[key] = normalizeChildren(value);
        }
        else if (tmp = getEventName(key, events)) {
            normalizedProps[tmp] = value;
        }
        else if (key.startsWith('slot')) {
            if (!blocks)
                blocks = normalizedProps.$blocks = {};
            blocks[hyphenate(key.substring(4))] = normalizeBlock(value);
        }
        else if (key === 'forwardRef') {
            normalizedProps.ref = value;
        }
        else {
            normalizedProps[key] = value;
        }
    }
    return normalizedProps;
}
function normalizeRef(ref) {
    return isObject(ref) ? function (i) { ref.current = i; } : ref;
}
function isIntactComponent(vNode) {
    var type = vNode.type;
    return !!type.$cid;
}
function getEventName(propName, events) {
    var third = propName[2];
    if (!third)
        return;
    var first = propName[0];
    var second = propName[1];
    var tmp;
    if (first === 'o' && second === 'n') {
        if (third === '$') {
            // e.g. on$change-value
            return "ev-$".concat(propName.substring(3).replace(/\-/g, ':'));
        }
        else if ((tmp = third.charCodeAt(0)) && tmp >= 65 && tmp <= 90) {
            var eventName = propName.substring(2);
            if (eventName === 'Change') {
                // i.e onChange -> ev-change
                return "ev-change";
            }
            else if (eventName.startsWith('Change')) {
                // e.g. onChangeValue -> ev-$change:value
                return "ev-$change:".concat(lowerFirst(eventName.substring(6)));
            }
            else {
                // e.g. onClick -> ev-click; onKeyPress -> ev-keypress
                var loweredFirst = lowerFirst(eventName.replace(/-/g, ':'));
                if (!events || !events[loweredFirst]) {
                    // is native event
                    loweredFirst = loweredFirst.toLowerCase();
                }
                return "ev-".concat(loweredFirst);
            }
        }
    }
}
function cache(fn) {
    var cache = Object.create(null);
    return function (str) {
        return cache[str] || (cache[str] = fn(str));
    };
}
var lowerFirst = cache(function (str) {
    return str.charAt(0).toLowerCase() + str.slice(1);
});
var hyphenateRE = /\B([A-Z])/g;
var hyphenate = cache(function (str) {
    return (str.charAt(0) + str.slice(1).replace(hyphenateRE, '-$1')).toLowerCase();
});
function normalizeBlock(block) {
    if (isFunction(block)) {
        return function (parent) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            return normalizeChildren(block.apply(void 0, args));
        };
    }
    return function () {
        return normalizeChildren(block);
    };
}

var PROMISES = '_$IntactReactPromises';
var Component = /*#__PURE__*/ /*#__PURE__*/ /** @class */ (function (_super) {
    __extends(Component, _super);
    function Component(props, $vNodeOrContext, $SVG, $mountedQueue, $senior) {
        var _this = this;
        if ($vNodeOrContext && isComponentClass($vNodeOrContext)) {
            // Intact component in intact
            _this = _super.call(this, props, $vNodeOrContext, $SVG, $mountedQueue, $senior) || this;
        }
        else {
            // super(null, null as any, false, [], null);
            // Intact component in React
            // @ts-ignore
            var normalizedProps = normalizeProps(props, _this.constructor.events);
            var parent_1 = $vNodeOrContext;
            var mountedQueue = getMountedQueue(parent_1);
            _this = _super.call(this, normalizedProps, null, false, mountedQueue, parent_1) || this;
            _this.$inited = true;
            // create $vNode
            _this.$vNode = createComponentVNode(4, _this.constructor, normalizedProps);
            _this.$elementRef = createRef$1();
            if (_this.constructor.$doubleVNodes) {
                _this.$elementAlternateRef = createRef$1();
            }
            _this.$isReact = true;
            var promises = _this.$promises = new FakePromises();
            _this.$parentPromises = inject(PROMISES, null);
            provide(PROMISES, promises);
        }
        return _this;
    }
    Component.prototype.render = function () {
        // save all Context.Proiver, because React will clear the value
        // on commit phase, and we cannot get it when we render react element
        // in Wrapper
        var providers = this.$reactProviders = new Map();
        var returnFiber = this._reactInternals;
        var _loop_1 = function () {
            var tag = returnFiber.tag;
            if (tag === 10) { // is ContextProiver
                var type = returnFiber.type;
                providers.set(type, type._context._currentValue);
            }
            else if (tag === 3) { // HostRoot
                /**
                 * React will update from root, if root has pendingContext, it will compare
                 * the last value and the current value to change `didPerformWorkStatckCursor`,
                 * if the cursor is true, all children will be updated
                 * @FIXME: Maybe we can use all Consumer to wrap the placeholder element
                 *
                 * always let hasContextChanged return true to make React update the component,
                 * even if it props has not changed
                 * see unit test: `shuold update children when provider's children...`
                 * issue: https://github.com/ksc-fe/kpc/issues/533
                 **/
                var stateNode = returnFiber.stateNode;
                if (!stateNode.__intactReactDefinedProperty) {
                    var context_1;
                    Object.defineProperty(stateNode, 'pendingContext', {
                        get: function () {
                            return context_1 || (returnFiber.context ? __assign({}, returnFiber.context) : Object.create(null));
                        },
                        set: function (v) {
                            context_1 = v;
                        }
                    });
                    stateNode.__intactReactDefinedProperty = true;
                }
                return "break";
            }
        };
        while (returnFiber = returnFiber.return) {
            var state_1 = _loop_1();
            if (state_1 === "break")
                break;
        }
        var vNode = createTemplateVNode(this.$elementRef, '1');
        return this.constructor.$doubleVNodes ?
            [vNode, createTemplateVNode(this.$elementAlternateRef, '2')] :
            vNode;
    };
    Component.prototype.componentDidMount = function () {
        var _this = this;
        var vNode = this.$vNode;
        var placeholder = this.$elementRef.current;
        var parentElement = this.$parentElement = placeholder.parentElement;
        // React will assign a emptyObject which is frozen to refs on mountClassInstance
        // we have to re-assign it
        this.refs = {};
        // we may call hooks in init lifecycle, so set instance here,
        // $init method will set it to null
        setInstance(this);
        this.$init(vNode.props);
        vNode.children = this;
        var sibling = placeholder;
        while (sibling = sibling.nextSibling) {
            if (sibling.nodeType !== 8 || sibling.nodeValue !== containerComment) {
                break;
            }
        }
        this.$render(null, vNode, parentElement, sibling, this.$mountedQueue);
        this.$done(function () {
            handleElementOnMounted(parentElement, placeholder, findDomFromVNode(vNode, true));
            if (_this.constructor.$doubleVNodes) {
                handleElementOnMounted(parentElement, _this.$elementAlternateRef.current, findDomFromVNode(vNode, false));
            }
        });
    };
    Component.prototype.componentDidUpdate = function () {
        var normalizedProps = normalizeProps(this.props, this.constructor.events);
        var vNode = createComponentVNode(4, this.constructor, normalizedProps);
        vNode.children = this;
        var lastVNode = this.$vNode;
        this.$vNode = vNode;
        var promises = this.$promises;
        if (promises.done) {
            promises.reset();
            var mountedQueue = this.$mountedQueue = getMountedQueue(this.$senior);
            this.$update(lastVNode, vNode, this.$parentElement, null, mountedQueue, false);
            this.$done(null);
        }
        else {
            this.$update(lastVNode, vNode, this.$parentElement, null, this.$mountedQueue, false);
        }
    };
    Component.prototype.componentWillUnmount = function () {
        this.$unmount(this.$vNode, null);
    };
    Component.prototype.$done = function (callback) {
        var _this = this;
        var done = function () {
            var p = FakePromise.all(_this.$promises);
            if (callback) {
                return p.then(callback);
            }
            return p;
        };
        var $parentPromises = this.$parentPromises;
        if ($parentPromises && !$parentPromises.done) {
            $parentPromises.add(new FakePromise(function (resolve) {
                done().then(resolve);
            }));
        }
        else {
            done().then(function () {
                callAll(_this.$mountedQueue);
            });
        }
    };
    Component.prototype.setState = function () { };
    Component.$cid = 'IntactReact';
    Component.normalize = normalizeChildren;
    Component.functionalWrapper = functionalWrapper;
    Component.contextType = Context;
    Component.FakePromise = FakePromise;
    // if the flag is true, it indicate the component returns 2 vNodes,
    // only support two, becasue kpc components don't return vNodes exceed 2.
    Component.$doubleVNodes = false;
    return Component;
}(Component$1));
Component.prototype.isReactComponent = true;
function getMountedQueue(parent) {
    if (parent) {
        parent.$mountedQueue;
        var parentPromises = parent.$provides[PROMISES];
        if (!parentPromises.done) {
            return parent.$mountedQueue;
        }
    }
    return [];
}
function createTemplateVNode(ref, key) {
    return createElement('template', {
        ref: ref,
        key: key,
        'data-intact-react': '',
    });
}
function handleElementOnMounted(parentElement, placeholder, element) {
    parentElement.removeChild(placeholder);
    // replace some properties like React do
    var fiber = precacheFiberNode(element, placeholder);
    // if we return a React element directly, we can not replace its props by placeholder's props,
    // otherwise its event props will be removed
    // updateFiberProps(element, placeholder);
    fiber.stateNode = element;
    var alternate = fiber.alternate;
    if (alternate) {
        alternate.stateNode = element;
    }
}

export { Component, EMPTY_OBJ, Fragment, Transition, TransitionGroup, VNode, addTransitionClass, callAll, className, compile, createCommentVNode, createComponentVNode, createElementVNode, createFragment, createRef, createTextVNode, createUnescapeTextVNode, createUnknownComponentVNode, createVNode, createVoidVNode, directClone, extend, findDomFromVNode, getTypeForVNodeElement, hasInvalidChildren, hasMultipleChildren, hasTextChildren, hasVNodeChildren, hooks, inject, insertOrAppend, isChecked, isComment, isComponentClass, isComponentFunction, isElement, isFragment, isText, linkEvent, map, mount, nextTick, noop, normalizeChildren as normalize, normalizeChildren$1 as normalizeChildren, normalizeRoot, onBeforeMount, onBeforeUnmount, onBeforeUpdate, onInited, onMounted, onUnmounted, onUpdated, patch, provide, registerCompile, remove, removeTransitionClass, removeVNodeDom, render, setCheckboxModel, setInstance, setRadioModel, setSelectModel, setTextModel, superCall, unmount, unmountComponentClass, useInstance, validateProps, watch, whenTransitionEnds };
